// DEMSurfacePropDlg.cpp : implementation file
//

#include <math.h>
#include "stdafx.h"
#include "dem3d.h"
#include "Dem3dDoc.h"
#include "Dem3dView.h"
#include "DEMSurfacePropDlg.h"
#include "mfcutils.h"

#ifndef INCLUDED_SG_SCENENODE_HXX
#include "scenegraph/sg_SceneNode.hxx"
#endif

#ifndef INCLUDED_SG_DEM_HXX
#include "scenegraph/sg_DEM.hxx"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// DEMSurfacePropDlg dialog


DEMSurfacePropDlg::DEMSurfacePropDlg(CDem3dView* view,
                                     sg_ShapeNode* demNode,
                                     sg_DEM* demShape,
                                     CWnd* pParent /*=NULL*/)
	: CDialog(DEMSurfacePropDlg::IDD, pParent),
     _view(view),
     _demNode(demNode),
     _demShape(demShape)
{
	//{{AFX_DATA_INIT(DEMSurfacePropDlg)
	m_shadows = FALSE;
	m_renderType = -1;
	m_boundBox = FALSE;
	m_loResApprox = FALSE;
	//}}AFX_DATA_INIT
}


void
DEMSurfacePropDlg::setDEMNode(sg_ShapeNode* demNode)
{
   _demNode = demNode;
   UpdateData(FALSE);
}

void
DEMSurfacePropDlg::setDEMShape(sg_DEM* demShape)
{
   _demShape = demShape;
   UpdateData(FALSE);
}

void DEMSurfacePropDlg::DoDataExchange(CDataExchange* pDX)
{

   // Do stuff that does not require the windows for the controls to be created
   // already (this is only important the first time in here).

   if (_demNode != 0)
      {
         if (pDX->m_bSaveAndValidate == FALSE)
            {
               m_shadows = _demNode->isLit() ? TRUE : FALSE;

               sg_Appearance::ShadingStyle shadeStyle;
               shadeStyle = _demNode->getShadingStyle();

               sg_Appearance::PolygonMode polyMode;
               polyMode = _demNode->getPolygonMode();

               if (shadeStyle == sg_Appearance::FLAT)
                  if (polyMode == sg_Appearance::FILL)
                     m_renderType = DEMSurfacePropDlg::FLAT_SHADED;
                  else
                     m_renderType = DEMSurfacePropDlg::WIREFRAME;
               else
                  if (shadeStyle == sg_Appearance::SMOOTH)
                     if (_demShape->isGouraud())
                        m_renderType = DEMSurfacePropDlg::GOURAUD_SHADED;
                     else
                        m_renderType = DEMSurfacePropDlg::SMOOTH_SHADED;

               m_boundBox = _view->demBoundBox();
               m_loResApprox = _view->loResApprox();
            }
      }

	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(DEMSurfacePropDlg)
	DDX_Control(pDX, IDC_DEMSURFDLG_SPECINTEN, m_specularIntenSlider);
	DDX_Control(pDX, IDC_DEMSURFDLG_SHINY, m_shinySlider);
	DDX_Check(pDX, IDC_DEMSURFDLG_SHADOWS, m_shadows);
	DDX_Radio(pDX, IDC_DEMSURFDLG_FLAT, m_renderType);
	DDX_Check(pDX, IDC_BOUNDBOX, m_boundBox);
	DDX_Check(pDX, IDC_LORES, m_loResApprox);
	//}}AFX_DATA_MAP

   if (_demNode != 0)
      {
         if (pDX->m_bSaveAndValidate == FALSE)
            {
               // Initialize dialog from current surface properties.

               ::setScaledValue(m_shinySlider, 0.0, 128.0,
                                _demNode->getShininess());
               sg_RGBA sc = _demNode->getSpecularColor();
               double mag = sqrt(pow(sc.red,2) + pow(sc.green,2) + pow(sc.blue,2));
               ::setScaledValue(m_specularIntenSlider, 0.0, 1.0, mag);
            }
         else
            {
               // Update surface settings from dialog.

               if (m_shadows)
                  _demNode->setLighting(true);
               else
                  _demNode->setLighting(false);

               switch(m_renderType)
                  {
                     case DEMSurfacePropDlg::FLAT_SHADED:
                        _demNode->setShadingStyle(sg_Appearance::FLAT);
                        _demNode->setPolygonMode(sg_Appearance::FILL);
                        _demShape->setGouraud(false);
                        break;
                     case DEMSurfacePropDlg::SMOOTH_SHADED:
                        _demNode->setShadingStyle(sg_Appearance::SMOOTH);
                        _demNode->setPolygonMode(sg_Appearance::FILL);
                        _demShape->setGouraud(false);
                        break;
                     case DEMSurfacePropDlg::GOURAUD_SHADED:
                        _demNode->setShadingStyle(sg_Appearance::SMOOTH);
                        _demNode->setPolygonMode(sg_Appearance::FILL);
                        _demShape->setGouraud(true);
                        break;
                     case DEMSurfacePropDlg::WIREFRAME:
                        _demNode->setShadingStyle(sg_Appearance::FLAT);
                        _demNode->setPolygonMode(sg_Appearance::LINE);
                        _demShape->setGouraud(false);
                        break;
                  }

               _demNode->setShininess(::getScaledValue(m_shinySlider, 0.0, 128.0));
               double val = ::getScaledValue(m_specularIntenSlider, 0.0, 1.0);
               _demNode->setSpecularColor(sg_RGBA(val, val, val));

               if (m_boundBox)
                  _view->setDEMBoundBox(true);
               else
                  _view->setDEMBoundBox(false);

               if (m_loResApprox)
                  _view->setLoResApprox(true);
               else
                  _view->setLoResApprox(false);
            }
      }

}


BEGIN_MESSAGE_MAP(DEMSurfacePropDlg, CDialog)
	//{{AFX_MSG_MAP(DEMSurfacePropDlg)
	ON_BN_CLICKED(IDAPPLY, OnApply)
	ON_BN_CLICKED(IDC_DEMSURFDLG_FLAT, OnDemSurfDlgFlat)
	ON_BN_CLICKED(IDC_DEMSURFDLG_GOURAUD, OnDemSurfDlgGouraud)
	ON_BN_CLICKED(IDC_DEMSURFDLG_SMOOTH, OnDemSurfDlgSmooth)
	ON_BN_CLICKED(IDC_DEMSURFDLG_WIRE, OnDemSurfDlgWire)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// DEMSurfacePropDlg message handlers

void DEMSurfacePropDlg::OnApply() 
{
   UpdateData(TRUE);
   _view->startRender();
}

void DEMSurfacePropDlg::OnCancel() 
{
   ShowWindow(SW_HIDE);
}

void DEMSurfacePropDlg::OnOK() 
{
   UpdateData(TRUE);
   ShowWindow(SW_HIDE);
   _view->startRender();
}

void DEMSurfacePropDlg::OnDemSurfDlgFlat() 
{
	// TODO: Add your control notification handler code here
	
}

void DEMSurfacePropDlg::OnDemSurfDlgGouraud() 
{
	// TODO: Add your control notification handler code here
	
}

void DEMSurfacePropDlg::OnDemSurfDlgSmooth() 
{
	// TODO: Add your control notification handler code here
	
}

void DEMSurfacePropDlg::OnDemSurfDlgWire() 
{
	// TODO: Add your control notification handler code here
	
}
