
#include "stdafx.h"
#include "dem3d.h"
#include "Dem3dStatus.h"


Dem3dStatus::Dem3dStatus(CStatusBar* statBar)
           : _statBar(statBar),
             _progressCtrl(0),
             _elevUnits(METERS)
{
   if (_statBar == 0)
      return;

   //_cameraPane = _statBar->CommandToIndex(ID_INDICATOR_CAMERA);
   //_lightPane = _statBar->CommandToIndex(ID_INDICATOR_LIGHT);
   _toolPane = _statBar->CommandToIndex(ID_INDICATOR_TOOL);
   _vertExagPane = _statBar->CommandToIndex(ID_INDICATOR_VERTEXAG);
   _resolutionPane = _statBar->CommandToIndex(ID_INDICATOR_RESOL);
   _progressPane = _statBar->CommandToIndex(ID_INDICATOR_STATUS); 
   _coordsPane = _statBar->CommandToIndex(ID_INDICATOR_XYZ);

   //setCameraAltAz(0.0, 0.0);
   //setLightAltAz(0.0, 0.0);
   if (_statBar != 0)
      _statBar->SetPaneText(_toolPane, "");
   setResolution(0.0);
   setVertExag(0.0);
   setCoords(0.0, 0.0, 0.0);
}

void
Dem3dStatus::setCameraAltAz(double altitude, double azimuth)
{
   if (_statBar == 0)
      return;

   CString txt;
   txt.Format("CAM(Alt,Az): %4.1f, %4.1f", altitude, azimuth);
   _statBar->SetPaneText(_toolPane, txt);
}

void
Dem3dStatus::setLightAltAz(double altitude, double azimuth)
{
   if (_statBar == 0)
      return;

   CString txt;
   txt.Format("LIGHT(Alt,Az): %4.1f, %4.1f", altitude, azimuth);
   _statBar->SetPaneText(_toolPane, txt);
}

void
Dem3dStatus::setWaterLevel(double elevation)
{
   if (_statBar == 0)
      return;

   char eu_txt[10];
   switch(_elevUnits)
   {
      case(FEET): strcpy(eu_txt, "ft");
      case(METERS): strcpy(eu_txt, "m");
   }

   CString txt;
   txt.Format("Water Level: %7.2f%s", elevation,eu_txt);
   _statBar->SetPaneText(_toolPane, txt);
}

void
Dem3dStatus::setVertExag(double ve)
{
   if (_statBar == 0)
      return;

   CString txt;
   txt.Format("Vert Exag: %4.1f", ve);
   _statBar->SetPaneText(_vertExagPane, txt);
}

void
Dem3dStatus::setResolution(double res)
{
   if (_statBar == 0)
      return;

   CString txt;
   txt.Format("Res: %3d%%", (long) (res * 100.0));
   _statBar->SetPaneText(_resolutionPane, txt);
}

void
Dem3dStatus::setProgress(double percent)
{
   // I tried creating the progress control in the ctor, but
   // the window for the status pane hasn't been created by then.
   // We delay creation until the first call to setProgress in the 
   // hope that the status pane window is there.
   if (_progressCtrl == 0)
   {
      RECT rc;
	   _statBar->GetItemRect (_progressPane, &rc);
	
      _progressCtrl = new CProgressCtrl;
	   VERIFY (_progressCtrl->Create(WS_CHILD | WS_VISIBLE, rc, _statBar, 1));
      _progressCtrl->SetRange(0,100);
   }

   if (percent > 100.0)
      percent = 100.0;

   if (percent < 0.0)
      percent = 0.0;

   _progressCtrl->SetPos((long) percent);
}

void
Dem3dStatus::setCoords(double x, double y, double z, bool good)
{
   if (_statBar == 0)
      return;
   if (good == true)
   {
      char eu_txt[10];
      switch(_elevUnits)
      {
         case(FEET): strcpy(eu_txt, "ft");
         case(METERS): strcpy(eu_txt, "m");
      }

      CString txt;
      txt.Format("X,Y,Z: %11.2f, %11.2f, %7.2f%s", x,y,z,eu_txt);
      _statBar->SetPaneText(_coordsPane, txt);
   }
   else
      _statBar->SetPaneText(_coordsPane,"X,Y,Z: Outside DEM or Void Area");
}

void
Dem3dStatus::setElevationUnits(Units u)
{
   _elevUnits = u;
}

void
Dem3dStatus::resize()
{
   delete _progressCtrl;
   _progressCtrl = 0;
}