// HSVIndicator.cpp : implementation file
//

#include "stdafx.h"
#include "dem3d.h"
#include "HSVIndicator.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#ifndef INCLUDED_SG_HSV_H
#include "scenegraph/sg_hsv.h"
#endif

/////////////////////////////////////////////////////////////////////////////
// CHSVIndicator

CHSVIndicator::CHSVIndicator()
             : _bitmap(0),
               _lowColorStart(0.0),
               _saturation(1.0),
               _value(1.0),
               _range(360.0)
{
}

CHSVIndicator::~CHSVIndicator()
{
}

void
CHSVIndicator::setLowColorStart(double deg)
{
   if (deg > 360.0)
      deg = 360.0;
   else
      if (deg < 0.0)
         deg = 0.0;
   _lowColorStart = deg;
}

void
CHSVIndicator::setSaturation(double sat)
{
   if (sat > 1.0)
      sat = 1.0;
   else
      if (sat < 0.0)
         sat = 0.0;
   _saturation = sat;
}

void
CHSVIndicator::setValue(double val)
{
   if (val > 1.0)
      val = 1.0;
   else
      if (val < 0.0)
         val = 0.0;
   _value = val;
}

void
CHSVIndicator::setRange(double deg)
{
   if (deg < 1.0)
      deg = 1.0;
   _range = deg;
}

double
CHSVIndicator::getLowColorStart() const
{
   return _lowColorStart;
}

double
CHSVIndicator::getSaturation() const
{
   return _saturation;
}

double
CHSVIndicator::getValue() const
{
   return _value;
}

double
CHSVIndicator::getRange() const
{
   return _range;
}

BEGIN_MESSAGE_MAP(CHSVIndicator, CStatic)
	//{{AFX_MSG_MAP(CHSVIndicator)
	ON_WM_CTLCOLOR_REFLECT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHSVIndicator message handlers

BOOL CHSVIndicator::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	dwStyle = dwStyle | SS_BITMAP;

	return CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}


HBRUSH CHSVIndicator::CtlColor(CDC* pDC, UINT nCtlColor) 
{
	// TODO: Change any attributes of the DC here
	
   CRect rect;
   GetClientRect(rect);
   long width = rect.right;
   long height = rect.bottom;

   delete _bitmap;
   _bitmap = new CBitmap;
   _bitmap->CreateCompatibleBitmap(pDC, width, height);
   pDC->SelectObject(_bitmap);

   double dtheta = _range / height;
   double theta = _lowColorStart;
   double r,g,b;
   long red, green, blue;
   CPen* oldPen;
   for (long i = 0; i < height; i++)
      {
         // Determine (r,g,b) color for current scanline.
         double temp = theta;
         if (temp > 360.0)
            temp = temp - 360.0;
         sg_hsv2rgb(temp,_saturation,_value,r,g,b);
         red = static_cast<long>(r * 255);
         green = static_cast<long>(g * 255);
         blue = static_cast<long>(b * 255);
         CPen hsvPen(PS_SOLID,1,RGB(red,green,blue));
         oldPen = pDC->SelectObject(&hsvPen);
         pDC->MoveTo(0,height - i - 1);
         pDC->LineTo(width - 1,height - i - 1);
         pDC->SelectObject(oldPen);
         theta = theta + dtheta;
      }


	// TODO: Return a non-NULL brush if the parent's handler should not be called
	return NULL;
}
