// HeaderInfoDlg.cpp : implementation file
//

#include "stdafx.h"
#include "dem3d.h"
#include "HeaderInfoDlg.h"

#ifndef INCLUDED_HF_DEM_HXX
#include "heightfield/hf_DEM.hxx"
#endif

#ifndef INCLUDED_HF_USGSDEM_HXX
#include "heightfield/hf_usgsDEM.hxx"
#endif

#ifndef INCLUDED_DEMHEADER_HXX
#include "dem\DEMHeader.hxx"
#endif

#include <strstream>
using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// HeaderInfoDlg dialog


HeaderInfoDlg::HeaderInfoDlg(CWnd* pParent /*=NULL*/)
	: CDialog(HeaderInfoDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(HeaderInfoDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

// This is a quick hack for a wordwrap, cause i couldn't get MFC to do it.
string
HeaderInfoDlg::formatString(string input)
{
   int x;


   if (input.size() > 80)
   {
      for(x = 80; x < input.size(); x+=80)
      {
         input.insert(x, "\r\n");
      }
   }
   return input;
}

void
HeaderInfoDlg::showSDTSInfo(hf_DEM* heightfield)
{
	strstream str;
   const char* crlf = "\r\n";
   long i;
   

 
   str << "Lineage Module Comment:" << crlf << formatString(heightfield->getLineageComment()) << crlf << crlf;
   str << "Positional Accuracy Module Comment:" << crlf << formatString(heightfield->getPositionalComment()) << crlf << crlf;
   str << "Logical Consistency Module Comment:" << crlf << formatString(heightfield->getLogicalComment()) << crlf << crlf;
   str << "Elevation Units: " << heightfield->getWorldUnits();
   switch(heightfield->getWorldUnits())
      {
         case(0): str << " (Feet)";    break;
         case(1): str << " (Meters)";  break;
      }
   str << crlf;

// XXX fix this
#if 0
   str << "Ground Coordinates for corners of DEM:" << crlf;
   DEMPointVector const& points = heightfield->getDEMCorners();
   for (i = 0; i < 4; i++)
      {
         str << "   (" << points[i].getX() << "," << points[i].getY();
         str << ")" << crlf;
      }
#endif

    str << "Minimum Elevation: " << heightfield->getMinElev() << crlf;
    str << "Maximum Elevation: " << heightfield->getMaxElev() << crlf << crlf;
    str << "The following have units of measure consistent with those used" << crlf;
    str << "for the ground coordinate system and elevation data:" << crlf;
    str << "   Spatial Resolution (X): " << heightfield->getXRes() << crlf;
    str << "   Spatial Resolution (Y): " << heightfield->getYRes() << crlf;
    str << "   Spatial Resolution (Z): " << heightfield->getZRes() << crlf;

    str << ends;
    m_edit1.SetWindowText(str.str());
    str.freeze(false);

}

void
HeaderInfoDlg::showUSGSInfo(hf_DEM* heightfield)
{
	strstream str;
   const char* crlf = "\r\n";
   long i;

   if (heightfield == 0)
      return;


   str << "Quadrangle Name: '" << heightfield->getQuadName() << "'" << crlf;

   str << "Mapping Center Code: '" << heightfield->getMappingCenterCode() << "'" << crlf;
   str << "Level Code: " << heightfield->getLevelCode() << crlf;
   str << "Elevation Pattern: " << heightfield->getElevPattern() << crlf;

   str << "Ground Reference System Code: " << heightfield->getGroundRefSysCode();
         
   switch(heightfield->getGroundRefSysCode())
      {
         case(0): str << " (Geographic)";  break;
         case(1): str << " (UTM)";         break;
         case(2): str << " (State Plane)"; break;
      }
   str << crlf;

   str << "Ground Reference System Zone: " << heightfield->getGroundRefSysZone() << crlf;

   str << "Ground Reference System Units: " << heightfield->getGroundRefSysUnits();
   switch(heightfield->getGroundRefSysUnits())
      {
         case(0): str << " (Radians)";     break;
         case(1): str << " (Feet)";        break;
         case(2): str << " (Meters)";      break;
         case(3): str << " (Arc Seconds)"; break;
      }
   str << crlf;

   str << "Elevation Units: " << heightfield->getWorldUnits();
   switch(heightfield->getWorldUnits())
      {
         case(0): str << " (Feet)";    break;
         case(1): str << " (Meters)";  break;
      }
   str << crlf;

   str << "Ground Coordinates for corners of DEM:" << crlf;
   DEMPointVector const& points = heightfield->getDEMCorners();
   for (i = 0; i < 4; i++)
      {
         str << "   (" << points[i].getX() << "," << points[i].getY();
         str << ")" << crlf;
      }

    str << "Minimum Elevation: " << heightfield->getMinElev() << crlf;
    str << "Maximum Elevation: " << heightfield->getMaxElev() << crlf << crlf;
    str << "The following have units of measure consistent with those used" << crlf;
    str << "for the ground coordinate system and elevation data:" << crlf;
    str << "   Spatial Resolution (X): " << heightfield->getXRes() << crlf;
    str << "   Spatial Resolution (Y): " << heightfield->getYRes() << crlf;
    str << "   Spatial Resolution (Z): " << heightfield->getZRes() << crlf;
    str << crlf;
    str << "Number of profiles in DEM: " << heightfield->getProfileColumns() << crlf;

    str << ends;
    m_edit1.SetWindowText(str.str());
    str.freeze(false);
}


void
HeaderInfoDlg::updateHeaderInfo(hf_DEM* heightfield)
{
	strstream str;

   if (heightfield == 0)
      {
         str << "No DEM Loaded.";
         str << ends;
         m_edit1.SetWindowText(str.str());
         str.freeze(false);
         return;
      }

   if ( heightfield->getHeightfieldType() == hf_DEM::USGS_DEM )
      {
         showUSGSInfo(heightfield);
      }
   else if ( heightfield->getHeightfieldType() == hf_DEM::SDTS_DEM )
      {
         showSDTSInfo(heightfield);
      }
}



void HeaderInfoDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(HeaderInfoDlg)
	DDX_Control(pDX, IDC_EDIT1, m_edit1);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(HeaderInfoDlg, CDialog)
	//{{AFX_MSG_MAP(HeaderInfoDlg)
	ON_BN_CLICKED(IDOK2, OnOk2)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// HeaderInfoDlg message handlers

void HeaderInfoDlg::OnOk2() 
{
	ShowWindow(SW_HIDE);
}
