// OpenGLView.cpp : implementation of the COpenGLView class
//

#include "stdafx.h"
#include "dem3d.h"

#include "OpenGLView.h"

#include "gl/gl.h"
#include "gl/glu.h"

#ifndef INCLUDED_SG_SCENENODE_HXX
#include "scenegraph/sg_SceneNode.hxx"
#endif

#ifndef INCLUDED_SG_SCENEITERATOR_HXX
#include "scenegraph/sg_SceneIterator.hxx"
#endif

#ifndef INCLUDED_SG_SCENEVISITOR_HXX
#include "scenegraph/sg_SceneVisitor.hxx"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

unsigned char threeto8[8] = 
{
	0, 0111>>1, 0222>>1, 0333>>1, 0444>>1, 0555>>1, 0666>>1, 0377
};

unsigned char twoto8[4] = 
{
	0, 0x55, 0xaa, 0xff
};

unsigned char oneto8[2] = 
{
	0, 255
};

static int defaultOverride[13] = 
{
	0, 3, 24, 27, 64, 67, 88, 173, 181, 236, 247, 164, 91
};

static PALETTEENTRY defaultPalEntry[20] = 
{
	{ 0,   0,   0,    0 },
	{ 0x80,0,   0,    0 },
	{ 0,   0x80,0,    0 },
	{ 0x80,0x80,0,    0 },
	{ 0,   0,   0x80, 0 },
	{ 0x80,0,   0x80, 0 },
	{ 0,   0x80,0x80, 0 },
	{ 0xC0,0xC0,0xC0, 0 },

	{ 192, 220, 192,  0 },
	{ 166, 202, 240,  0 },
	{ 255, 251, 240,  0 },
	{ 160, 160, 164,  0 },

	{ 0x80,0x80,0x80, 0 },
	{ 0xFF,0,   0,    0 },
	{ 0,   0xFF,0,    0 },
	{ 0xFF,0xFF,0,    0 },
	{ 0,   0,   0xFF, 0 },
	{ 0xFF,0,   0xFF, 0 },
	{ 0,   0xFF,0xFF, 0 },
	{ 0xFF,0xFF,0xFF, 0 }
};

/////////////////////////////////////////////////////////////////////////////
// COpenGLView

IMPLEMENT_DYNCREATE(COpenGLView, CView)

BEGIN_MESSAGE_MAP(COpenGLView, CView)
	//{{AFX_MSG_MAP(COpenGLView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COpenGLView construction/destruction

COpenGLView::COpenGLView()
           : _sceneRoot(0),
             _cameraNode(0),
             _sceneRenderer(0),
             _sceneIter(0),
             _hiResRenderSteps(0),
             _loResTimeOut(1000),
             _backgroundColor(0.0, 0.0, 0.0)
{
	// TODO: add construction code here

  _sceneRenderer = new sg_RenderVisitor;

	m_pDC = NULL;
	m_pOldPalette = NULL;

}

COpenGLView::~COpenGLView()
{
}

BOOL COpenGLView::PreCreateWindow(CREATESTRUCT& cs)
{
  // An OpenGL  window must be created with the following flags and
  // must not include CS_PARENTDC for the class style.

  cs.style = cs.style | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// COpenGLView drawing

void COpenGLView::OnDraw(CDC* pDC)
{
//	COpenGLDoc* pDoc = GetDocument();
//	ASSERT_VALID(pDoc);

   SwapBuffers(m_pDC->GetSafeHdc());

}

/////////////////////////////////////////////////////////////////////////////
// COpenGLView diagnostics

#ifdef _DEBUG
void COpenGLView::AssertValid() const
{
	CView::AssertValid();
}

void COpenGLView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// COpenGLView message handlers

int COpenGLView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
  initOpenGL();
	
  // Register our OnIdle handler with the specially modified application.
  CDem3dApp* oglApp;
  oglApp = dynamic_cast<CDem3dApp*>(AfxGetApp());
  if (oglApp == 0)
  {
    AfxMessageBox("Couldn't register COpenGLView::OnIdle()");
    AfxAbort();
  }

  oglApp->setOpenGLView(this);

	return 0;
}

void COpenGLView::OnDestroy() 
{
	HGLRC	hrc;

	hrc = ::wglGetCurrentContext();

  ::wglMakeCurrent(NULL,  NULL);

  if (hrc)
    ::wglDeleteContext(hrc);

  if (m_pOldPalette)
    m_pDC->SelectPalette(m_pOldPalette, FALSE);

  if (m_pDC)
    delete m_pDC;

	CView::OnDestroy();

  delete _sceneIter;
  delete _sceneRenderer;
}

BOOL COpenGLView::OnEraseBkgnd(CDC* pDC) 
{
  return true;  // OpenGL will erase the background.
}

void COpenGLView::OnSize(UINT nType, int w, int h) 
{
	CView::OnSize(nType, w, h);
	
  if (_cameraNode == 0)
    return;

  disableLoResTimer();

  double aspect;
	
	if (h <= 0)
		aspect = 1.0;
	else
		aspect = (1.0 * w) / h;

	_cameraNode->setViewPort(0,0,w,h);
	_cameraNode->setAspect(aspect);

  enableLoResTimer();
  setRenderMode(LORES);
  startRender();
}

bool COpenGLView::initOpenGL()
{
  Init();

	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
	glColor3f(1.0,0.0,0.0);

  return TRUE;
}



void COpenGLView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
	
 	glClearColor(0.0,0.0,0.0,0.0);
	glClearDepth(1.0);
	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_SMOOTH);	
  glEnable(GL_DITHER);

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}

void
COpenGLView::setSceneRoot(sg_CompositeNode* rootNode)
{
  _sceneRoot = rootNode;

  delete _sceneIter;
  _sceneIter = new sg_RenderIter(_sceneRoot);
}

sg_CompositeNode*
COpenGLView::getSceneRoot() const
{
 return _sceneRoot;
}

void
COpenGLView::setCameraNode(sg_CameraNode* cameraNode)
{
   _cameraNode = cameraNode;
}

sg_CameraNode*
COpenGLView::getCameraNode() const
{
   return _cameraNode;
}

void
COpenGLView::setBackgroundColor(sg_RGBA const& color)
{
   _backgroundColor = color;
}

sg_RGBA
COpenGLView::getBackgroundColor() const
{
   return _backgroundColor;
}

void
COpenGLView::startRender()
{
   if (_sceneRoot == 0)
      return;

   delete _sceneRenderer;
   _sceneRenderer = new sg_RenderVisitor;

   if (_renderMode == HIRES)
      {
         preRenderHiRes();
         _sceneIter->first();
      }

   if (_renderMode == LORES)
      {
         preRenderLoRes();
         render();
         postRenderLoRes();
      }
}

void
COpenGLView::setRenderMode(COpenGLView::RenderMode mode)
{
   _renderMode = mode;
}

BOOL
COpenGLView::OnIdle(LONG lCount)
{
   if (_sceneRoot == 0)
      return 0;   // No scenegraph, so nothing to do.

   if (_sceneIter->isDone())
      return 0;   // Scene completely rendered, so nothing to do.

   if (render() > 0)
      return TRUE;
   else
      return FALSE;
}

long
COpenGLView::render()
{
  if (_sceneRoot == 0)
    return 0;  // No scene to render, so we're finished.

  if (_renderMode == HIRES)
  {
    if (_sceneIter->isDone())
    {
      postRenderHiRes();
      return 0;
    }

    if (_sceneIter->currentItem()->isActive())
    {
      _sceneIter->currentItem()->accept(*_sceneRenderer);
      _hiResRenderSteps--;
      if (_sceneRenderer->finishedWithNode())
        _sceneIter->next();
    }
    else
      _sceneIter->next();

    if ((_hiResRenderSteps % 20) == 0)
    {
      SwapBuffers(m_pDC->GetSafeHdc());
    }

    if (_sceneIter->isDone())
    {
      SwapBuffers(m_pDC->GetSafeHdc());
      postRenderHiRes();
      return 0;
    }

    return 1;
  }

   // In LORES mode, render an entire frame at once.

   _sceneIter->first();
   while(!_sceneIter->isDone())
      {
         if (_sceneIter->currentItem()->isActive())
            {
               _sceneIter->currentItem()->accept(*_sceneRenderer);
               while (!_sceneRenderer->finishedWithNode())
                  {
                     _sceneIter->currentItem()->accept(*_sceneRenderer);
                  }
            }
         _sceneIter->next();
      }

   if (_loResTimerFlag)
      SetTimer(1,_loResTimeOut,0);

   return 0;
}

void
COpenGLView::preRenderLoRes()
{
   glEnable(GL_DEPTH_TEST);
   glShadeModel(GL_FLAT);
   glEnable(GL_LIGHTING);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
//    glClearColor(0.0, 0.0, 0.0, 0.0);

   glClearColor(_backgroundColor.red,
                _backgroundColor.green,
                _backgroundColor.blue,
                1.0);

   glClearDepth(1.0);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	

   // XXX Begin calulation of lo res frame rate.
}

void
COpenGLView::postRenderLoRes()
{
   glFlush();
   SwapBuffers(m_pDC->GetSafeHdc());

   // XXX Finish calculation of lo res frame rate and adjust
   // DEM lo resolution appropriately.
}

void
COpenGLView::preRenderHiRes()
{
   glClearColor(_backgroundColor.red,
                _backgroundColor.green,
                _backgroundColor.blue,
                1.0);

   glClearDepth(1.0);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
      

   if (_sceneRoot == 0)
      return;

   // Make a pass over the scene graph to determine how many
   // render steps will be needed for the entire scene.

	sg_StepCountVisitor scv;
	sg_PreOrderIter preIter(_sceneRoot);
	preIter.first();
	while (!preIter.isDone())
	   {
		   preIter.currentItem()->accept(scv);
         preIter.next();
      }
   _hiResRenderSteps = scv.getTotalSteps();
	

}

void
COpenGLView::postRenderHiRes()
{   
   if (_sceneRoot == 0)
      return;

   glFlush();
   SwapBuffers(m_pDC->GetSafeHdc());
}

void COpenGLView::OnTimer(UINT nIDEvent) 
{
   KillTimer(1);
   setRenderMode(HIRES);
   startRender();
}

void
COpenGLView::enableLoResTimer()
{
   _loResTimerFlag = true;
}

void
COpenGLView::disableLoResTimer()
{
   KillTimer(1);
   _loResTimerFlag = false;
}

void
COpenGLView::setLoResTimeOut(long millisec)
{
   _loResTimeOut = millisec;
}

// GL helper functions

void COpenGLView::Init()
{
   PIXELFORMATDESCRIPTOR pfd;
   int         n;
	HGLRC		hrc;

   m_pDC = new CClientDC(this);

   ASSERT(m_pDC != NULL);

   if (!bSetupPixelFormat())
      return;

   n = ::GetPixelFormat(m_pDC->GetSafeHdc());
   ::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

   CreateRGBPalette();

   hrc = wglCreateContext(m_pDC->GetSafeHdc());
   wglMakeCurrent(m_pDC->GetSafeHdc(), hrc);

   glClearDepth(1.0f);
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_DITHER);
}

BOOL COpenGLView::bSetupPixelFormat()
{
    static PIXELFORMATDESCRIPTOR pfd = 
	{
        sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
        1,                              // version number
        PFD_DRAW_TO_WINDOW |            // support window
          PFD_SUPPORT_OPENGL |          // support OpenGL
          PFD_DOUBLEBUFFER,             // double buffered
        PFD_TYPE_RGBA,                  // RGBA type
        24,                             // 24-bit color depth
        0, 0, 0, 0, 0, 0,               // color bits ignored
        0,                              // no alpha buffer
        0,                              // shift bit ignored
        0,                              // no accumulation buffer
        0, 0, 0, 0,                     // accum bits ignored
        32,                             // 32-bit z-buffer
        0,                              // no stencil buffer
        0,                              // no auxiliary buffer
        PFD_MAIN_PLANE,                 // main layer
        0,                              // reserved
        0, 0, 0                         // layer masks ignored
    };

    int pixelformat;

    if ( (pixelformat = ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)) == 0 )
    {
        MessageBox("ChoosePixelFormat failed");
        return FALSE;
    }

    if (SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd) == FALSE)
    {
        MessageBox("SetPixelFormat failed");
        return FALSE;
    }

    return TRUE;
}

unsigned char COpenGLView::ComponentFromIndex(int i, UINT nbits, UINT shift)
{
    unsigned char val;

    val = (unsigned char) (i >> shift);
    switch (nbits) 
	{

    case 1:
        val &= 0x1;
        return oneto8[val];
    case 2:
        val &= 0x3;
        return twoto8[val];
    case 3:
        val &= 0x7;
        return threeto8[val];

    default:
        return 0;
    }
}


void COpenGLView::CreateRGBPalette()
{
    PIXELFORMATDESCRIPTOR pfd;
    LOGPALETTE *pPal;
    int n, i;
 
    n = ::GetPixelFormat(m_pDC->GetSafeHdc());
    ::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

    if (pfd.dwFlags & PFD_NEED_PALETTE)
    {
        n = 1 << pfd.cColorBits;
        pPal = (PLOGPALETTE) new char[sizeof(LOGPALETTE) + n * sizeof(PALETTEENTRY)];

        ASSERT(pPal != NULL);

        pPal->palVersion = 0x300;
        pPal->palNumEntries = n;
        for (i=0; i<n; i++)
        {
            pPal->palPalEntry[i].peRed =
                    ComponentFromIndex(i, pfd.cRedBits, pfd.cRedShift);
            pPal->palPalEntry[i].peGreen =
                    ComponentFromIndex(i, pfd.cGreenBits, pfd.cGreenShift);
            pPal->palPalEntry[i].peBlue =
                    ComponentFromIndex(i, pfd.cBlueBits, pfd.cBlueShift);
            pPal->palPalEntry[i].peFlags = 0; //PC_RESERVED;
        }

        /* fix up the palette to include the default GDI palette */
        if ((pfd.cColorBits == 8)                           &&
            (pfd.cRedBits   == 3) && (pfd.cRedShift   == 0) &&
            (pfd.cGreenBits == 3) && (pfd.cGreenShift == 3) &&
            (pfd.cBlueBits  == 2) && (pfd.cBlueShift  == 6)
           )
        {
			for (i = 1 ; i <= 12 ; i++)
                pPal->palPalEntry[defaultOverride[i]] = defaultPalEntry[i];
        }

        m_cPalette.CreatePalette(pPal);
        delete pPal;

        m_pOldPalette = m_pDC->SelectPalette(&m_cPalette, FALSE);
        m_pDC->RealizePalette();
    }
}
