// OpenGLView.h : interface of the COpenGLView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_OPENGLVIEW_H__51BC43AD_B840_11D0_80B5_00A024E9DFA1__INCLUDED_)
#define AFX_OPENGLVIEW_H__51BC43AD_B840_11D0_80B5_00A024E9DFA1__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class sg_CompositeNode;
class sg_CameraNode;
class sg_RenderVisitor;
class sg_RenderIter;
class COpenGLDoc;

#ifndef INCLUDED_SG_TYPES_HXX
#include "scenegraph/sg_Types.hxx"
#endif

class COpenGLView : public CView
{
protected: // create from serialization only
	COpenGLView();
	DECLARE_DYNCREATE(COpenGLView)

// Attributes
public:

// Operations
public:

   void setSceneRoot(sg_CompositeNode* rootNode);
      // Sets the root of the scene graph to be rendered in this view.

   sg_CompositeNode* getSceneRoot() const;
      // Returns the root of the scene graph that's rendered in this view.

   void setCameraNode(sg_CameraNode* cameraNode);
      // Sets the camera node of the scene being rendered in this view.

   sg_CameraNode* getCameraNode() const;
      // Returns the camera node of the scene graph being rendered in this view.

   void setBackgroundColor(sg_RGBA const& color);

   sg_RGBA getBackgroundColor() const;


   // Rendering related member functions.

   virtual
   void startRender();
      // Causes the scene graph to be rendered in the current rendering
      // mode the next time idle processing is done.

   enum RenderMode {LORES, HIRES};

   virtual void setRenderMode(RenderMode mode);

   virtual BOOL OnIdle(LONG lCount);
      // This function needs to be called from the applications OnIdle
      // message handler.
   

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(COpenGLView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnInitialUpdate();
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~COpenGLView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

   CPalette    m_cPalette;
   CPalette    *m_pOldPalette;
	CClientDC	*m_pDC;

   sg_RGBA _backgroundColor;   // Default sg_Black.

	void Init();
	void CreateRGBPalette(void);
   BOOL bSetupPixelFormat(void);
   unsigned char ComponentFromIndex(int i, UINT nbits, UINT shift);

   RenderMode _renderMode;

   sg_CompositeNode* _sceneRoot;
   sg_CameraNode* _cameraNode;
   sg_RenderVisitor* _sceneRenderer;
   sg_RenderIter* _sceneIter;

   bool _loResTimerFlag;
   long _loResTimeOut;  // Default 1000
   long _hiResRenderSteps;  // Number of steps left to completely render scene in HIRES.

   void enableLoResTimer();
   void disableLoResTimer();
   void setLoResTimeOut(long millisec);

   void InitializePalette();

   virtual long render();
   virtual void preRenderLoRes();
   virtual void postRenderLoRes();
   virtual void preRenderHiRes();
   virtual void postRenderHiRes();


// Generated message map functions
protected:
	virtual bool initOpenGL();
	//{{AFX_MSG(COpenGLView)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDestroy();
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnTimer(UINT nIDEvent);
	//}}AFX_MSG
   // afx_msg BOOL OnQueryNewPalette();
	// afx_msg void OnPaletteChanged(CWnd* pFocusWnd);
   // afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	DECLARE_MESSAGE_MAP()
};



/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_OPENGLVIEW_H__51BC43AD_B840_11D0_80B5_00A024E9DFA1__INCLUDED_)
