// TerraceConfigDlg.cpp : implementation file
//

#include "stdafx.h"
#include "dem3d.h"
#include "TerraceConfigDlg.h"
#include "Dem3dDoc.h"
#include "Dem3dView.h"

#ifndef INCLUDED_SG_DEMVERTEXSHADER_HXX
#include "scenegraph/sg_DEMVertexShader.hxx"
#endif



#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// TerraceConfigDlg dialog


TerraceConfigDlg::TerraceConfigDlg(CDem3dView* view,
                                   CWnd* pParent /*=NULL*/)
	             : CDialog(TerraceConfigDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(TerraceConfigDlg)
	m_contourInterval = 0;
	//}}AFX_DATA_INIT

   _view = view;
   _shader = 0;
}


void
TerraceConfigDlg::setShader(sg_ContourShader* shader)
{
   _shader = shader;
   UpdateData(FALSE);
   Invalidate();
}



void TerraceConfigDlg::DoDataExchange(CDataExchange* pDX)
{
   if (_shader != 0)
      {
         if (pDX->m_bSaveAndValidate == FALSE)
            {
               // Initialize the color patches from the current shader settings.
               m_contourInterval = _shader->getContourInterval();
               m_color1.setPatchColor(_shader->getColor1());
               m_color2.setPatchColor(_shader->getColor2());
               m_edgeColor.setPatchColor(_shader->getEdgeColor());
            }
      }

	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(TerraceConfigDlg)
	DDX_Control(pDX, IDC_TERRACE_EDGECOLOR, m_edgeColor);
	DDX_Control(pDX, IDC_TERRACE_COLOR2, m_color2);
	DDX_Control(pDX, IDC_TERRACE_COLOR1, m_color1);
	DDX_Text(pDX, IDC_TERRACE_CONTOURINT, m_contourInterval);
	DDV_MinMaxLong(pDX, m_contourInterval, 0, 100000);
	//}}AFX_DATA_MAP


   if (_shader != 0)
      {
         if (pDX->m_bSaveAndValidate == TRUE)
            {
               // Use the values currently in the controls to set the shader.
               _shader->setContourInterval(m_contourInterval);

               sg_RGBA color;
               m_color1.getPatchColor(color);
               _shader->setColor1(color);

               m_color2.getPatchColor(color);
               _shader->setColor2(color);

               m_edgeColor.getPatchColor(color);
               _shader->setEdgeColor(color);
            }
      }
}


BEGIN_MESSAGE_MAP(TerraceConfigDlg, CDialog)
	//{{AFX_MSG_MAP(TerraceConfigDlg)
	ON_BN_CLICKED(ID_APPLY, OnApply)
	ON_BN_CLICKED(IDC_TERRACE_B1, OnTerraceB1)
	ON_BN_CLICKED(IDC_TERRACE_B2, OnTerraceB2)
	ON_BN_CLICKED(IDC_TERRACE_B3, OnTerraceB3)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// TerraceConfigDlg message handlers

void TerraceConfigDlg::OnApply() 
{
   UpdateData(TRUE);
   _view->startRender();
}

void TerraceConfigDlg::OnTerraceB1() 
{
   CColorDialog clrdlg;	

   if (clrdlg.DoModal() == IDOK)
      {
         COLORREF clr = clrdlg.GetColor();
         m_color1.setPatchColor(clr);
         Invalidate();
      }
}

void TerraceConfigDlg::OnTerraceB2() 
{
   CColorDialog clrdlg;	

   if (clrdlg.DoModal() == IDOK)
      {
         COLORREF clr = clrdlg.GetColor();
         m_color2.setPatchColor(clr);
         Invalidate();
      }	
}

void TerraceConfigDlg::OnTerraceB3() 
{
   CColorDialog clrdlg;	

   if (clrdlg.DoModal() == IDOK)
      {
         COLORREF clr = clrdlg.GetColor();
         m_edgeColor.setPatchColor(clr);
         Invalidate();
      }	
}


void TerraceConfigDlg::OnCancel() 
{
   ShowWindow(SW_HIDE);
}

void TerraceConfigDlg::OnOK() 
{
   UpdateData(TRUE);
   ShowWindow(SW_HIDE);
   _view->startRender();
}
