// WaterDlg.cpp : implementation file
//

#include "stdafx.h"
#include "dem3d.h"
#include "Dem3dDoc.h"
#include "Dem3dView.h"
#include "WaterDlg.h"
#include "mfcutils.h"

#ifndef INCLUDED_SG_DEM_HXX
#include "scenegraph/sg_DEM.hxx"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// WaterDlg dialog


WaterDlg::WaterDlg(CDem3dView* view,
                   sg_DEM* demShape,
                   CWnd* pParent)
	: CDialog(WaterDlg::IDD, pParent),
     _view(view),
     _demShape(demShape)
{
	//{{AFX_DATA_INIT(WaterDlg)
	m_transparent = FALSE;
	m_waterZ = _T("");
	m_levelRange = _T("");
	//}}AFX_DATA_INIT
}


void WaterDlg::DoDataExchange(CDataExchange* pDX)
{
   if (pDX->m_bSaveAndValidate == FALSE)
   {
      m_waterColor.setPatchColor(_demShape->getWaterColor());
      m_transparent = _demShape->getTransparentWater();
      m_waterZ.Format("%6.2f",_demShape->getWaterLevel());

      m_levelRange.Format("[%6.2f,%6.2f]",_demShape->getMinElev(),
                 _demShape->getMaxElev());
   }

	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(WaterDlg)
	DDX_Control(pDX, IDC_WATER_COLOR, m_waterColor);
	DDX_Control(pDX, IDC_WATERALPHA, m_alphaSlider);
	DDX_Check(pDX, IDC_TRANSPARENT, m_transparent);
	DDX_Text(pDX, IDC_WATER_Z, m_waterZ);
	DDX_Text(pDX, IDC_LEVELRANGE, m_levelRange);
	//}}AFX_DATA_MAP

   if (pDX->m_bSaveAndValidate == FALSE)
      ::setScaledValue(m_alphaSlider, 0.0, 1.0,
                       _demShape->getTransAlpha());
   else
   {
      _demShape->setWaterLevel(atof(m_waterZ));

      sg_RGBA color;
      m_waterColor.getPatchColor(color);
      _demShape->setWaterColor(color);

      if (m_transparent == TRUE)
         _demShape->setTransparentWater(true);
      else
         _demShape->setTransparentWater(false);

      _demShape->setTransAlpha(::getScaledValue(m_alphaSlider, 0.0, 1.0));
   }
}


BEGIN_MESSAGE_MAP(WaterDlg, CDialog)
	//{{AFX_MSG_MAP(WaterDlg)
	ON_BN_CLICKED(IDC_WATER_B1, OnWaterColor)
	ON_BN_CLICKED(ID_APPLY, OnApply)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// WaterDlg message handlers

void WaterDlg::OnWaterColor() 
{
   CColorDialog clrdlg;	

   if (clrdlg.DoModal() == IDOK)
      {
         COLORREF clr = clrdlg.GetColor();
         m_waterColor.setPatchColor(clr);
         Invalidate();
      }
}

void WaterDlg::OnApply() 
{
   UpdateData(TRUE);
   _view->startRender();
}

void WaterDlg::OnCancel() 
{
   ShowWindow(SW_HIDE);
}

void WaterDlg::OnOK() 
{
   UpdateData(TRUE);
   ShowWindow(SW_HIDE);
   _view->startRender();
}

void
WaterDlg::setDEMShape(sg_DEM* demShape)
{
   _demShape = demShape;
   UpdateData(FALSE);
}

void
WaterDlg::update()
{
   UpdateData(FALSE);
//   Invalidate();
}
