// dem3dDoc.cpp : implementation of the CDem3dDoc class
//

#include "stdafx.h"
#include "dem3d.h"

#include "dem3dDoc.h"
#include "MainFrm.h"
#include "Dem3dView.h"
#include "Dem3dStatus.h"

#include <iostream>
#include <fstream>
using namespace std;

#ifndef INCLUDED_HF_DEM_HXX
#include "heightfield/hf_DEM.hxx"
#endif

#ifndef INCLUDED_HF_USGSDEM_HXX
#include "heightfield/hf_usgsDEM.hxx"
#endif

#ifndef INCLUDED_HF_SDTSDEM_HXX
#include "heightfield/hf_sdtsDEM.hxx"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDem3dDoc

IMPLEMENT_DYNCREATE(CDem3dDoc, CDocument)

BEGIN_MESSAGE_MAP(CDem3dDoc, CDocument)
	//{{AFX_MSG_MAP(CDem3dDoc)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(CDem3dDoc, CDocument)
	//{{AFX_DISPATCH_MAP(CDem3dDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//      DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// Note: we add support for IID_IDem3d to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {E0AB4207-ABDE-11D0-80AB-02608CDA2E56}
static const IID IID_IDem3d =
{ 0xe0ab4207, 0xabde, 0x11d0, { 0x80, 0xab, 0x2, 0x60, 0x8c, 0xda, 0x2e, 0x56 } };

BEGIN_INTERFACE_MAP(CDem3dDoc, CDocument)
	INTERFACE_PART(CDem3dDoc, IID_IDem3d, Dispatch)
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDem3dDoc construction/destruction

CDem3dDoc::CDem3dDoc()
{
	// TODO: add one-time construction code here

	EnableAutomation();
	AfxOleLockApp();

   _heightfield = 0;
}

CDem3dDoc::~CDem3dDoc()
{
	AfxOleUnlockApp();
}

BOOL CDem3dDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CDem3dDoc serialization

void CDem3dDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDem3dDoc diagnostics

#ifdef _DEBUG
void CDem3dDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDem3dDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDem3dDoc commands

BOOL CDem3dDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;

	// Determine the heightfield type
	HEIGHTFIELDTYPE type = getHeightfieldType( lpszPathName );

	switch ( type )
	{
		case USGS_DEM:
			{
				_heightfield = new hf_usgsDEM();
				break;
			}
		case SDTS_DEM:
			{
				_heightfield = new hf_sdtsDEM();
				break;
			}
		case UNKNOWN_HEIGHTFIELD:
			{
				AfxMessageBox("Unknown Heightfield Type");
				return FALSE;
			}
	}

   // We'd like to put a Progress Control in CMainFrame's status bar. Get the
   // first (and only) view, then get its parent (the CMainFrame).
   POSITION pos = GetFirstViewPosition();
   CView* pFirstView = GetNextView( pos );

   CDem3dView* demView = dynamic_cast<CDem3dView*>(pFirstView);
   Dem3dStatus* status = demView->getDem3dStatus();

   string temp(lpszPathName);

   long totalSteps = _heightfield->fill(temp);
   long stepsLeft;
   while (stepsLeft > 0)
      {
         stepsLeft = _heightfield->fill(temp);
         status->setProgress((totalSteps - stepsLeft) * 100.0 / totalSteps);
      }

   status->setProgress(0.0);

	return TRUE;
}

void CDem3dDoc::DeleteContents() 
{
   delete _heightfield;
   _heightfield = 0;

	CDocument::DeleteContents();
}

hf_DEM* CDem3dDoc::getHeightField()
{
   return _heightfield;
}

void CDem3dDoc::OnFileOpen() 
{
	// Create a file open dialog box which allows you to open DLG or DRG files
	CString filter( "USGS DEM Files (*.dem, *.1)|*.dem; *.1|DEM in SDTS Files (*catd.ddf)|*catd.ddf|All Files (*.*)|*.*||" );
	CFileDialog openDialog( TRUE, NULL, NULL, OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST, filter);


	if ( openDialog.DoModal() == IDOK )
	{
		
		// Get the position of the first filename
		POSITION filenamePosition = openDialog.GetStartPosition();
		CString  filename;

		// Open each file selected
		filename = openDialog.GetNextPathName( filenamePosition );

		// Add the selected heightfield to the recently used file list
		AfxGetApp()->AddToRecentFileList( filename );

      AfxGetApp()->OpenDocumentFile( filename );

	};
	return;	
}

CString
CDem3dDoc::getFileExtension( const CString& filename ) const
{
	int lastPeriodPos = filename.ReverseFind( '.' );

	// If there is a period in the filename, get everything right
	// of the period, otherwise return an empty string
	if ( lastPeriodPos >= 0 )
	{
		int length = filename.GetLength();
		CString extension = filename.Right( length - lastPeriodPos - 1 );

		// Trim the string
		extension.TrimRight();
		extension.TrimLeft();

		return extension;
	}
	else
	{
		return CString("");
	}
}

HEIGHTFIELDTYPE
CDem3dDoc::getHeightfieldType( const CString& filename ) const
{
	// Get the filenames extension
	CString extension = getFileExtension( filename );

	// Check to see if the overlay is a DLG
	if ( extension.CompareNoCase( "1" ) == 0 )
	{
		return USGS_DEM;
	}
	else if ( extension.CompareNoCase( "DEM" ) == 0 )
	{
		return USGS_DEM;
	}
	else if ( extension.CompareNoCase( "DDF" ) == 0 )
	{
		return SDTS_DEM;
	}

	// Unknown type
	return UNKNOWN_HEIGHTFIELD;
}
