// dem3dView.cpp : implementation of the CDem3dView class
//

#include <math.h>
#include "stdafx.h"
#include "mmsystem.h"			// for time functions
#include "dem3d.h"
#include "MainFrm.h"

#include "dem3dDoc.h"
#include "dem3dView.h"
#include "HeaderInfoDlg.h"
#include "DemResDlg.h"
#include "VertExagDlg.h"
#include "heightfield/hf_DEM.hxx"
#include "CameraDlg.h"
#include "LightDlg.h"
#include "BackgroundConfigDlg.h"
#include "DaylightConfigDlg.h"
#include "SlopeConfigDlg.h"
#include "TerraceConfigDlg.h"
#include "GradientDlg.h"
#include "HSVConfigDlg.h"
#include "FrameCounter.hxx"
#include "DEMSurfacePropDlg.h"
#include "WaterDlg.h"

#ifndef INCLUDED_DEM3DSTATUS_H
#include "Dem3dStatus.h"
#endif

#ifndef INCLUDED_SG_SCENENODE_HXX
#include "scenegraph/sg_SceneNode.hxx"
#endif

#ifndef INCLUDED_SG_SCENEITERATOR_HXX
#include "scenegraph/sg_SceneIterator.hxx"
#endif

#ifndef INCLUDED_SG_SCENEVISITOR_HXX
#include "scenegraph/sg_SceneVisitor.hxx"
#endif

#ifndef INCLUDED_SG_SHAPE_HXX
#include "scenegraph/sg_Shape.hxx"
#endif

#ifndef INCLUDED_SG_TRANSFORMATION_HXX
#include "scenegraph/sg_Transformation.hxx"
#endif

#ifndef INCLUDED_SG_COLORS_HXX
#include "scenegraph/sg_Colors.hxx"
#endif

#ifndef INCLUDED_SG_CONTROLLER_HXX
#include "scenegraph/sg_Controller.hxx"
#endif

#ifndef INCLUDED_SG_MAGICCARPETCONTROLLER_H
#include "scenegraph/sg_MagicCarpetController.h"
#endif

#ifndef INCLUDED_SG_DEM_HXX
#include "scenegraph/sg_DEM.hxx"
#endif

#ifndef INCLUDED_SG_UTILS_HXX
#include "scenegraph/sg_Utils.hxx"
#endif

#ifndef INCLUDED_SG_DEMVERTEXSHADER_HXX
#include "scenegraph/sg_DEMVertexShader.hxx"
#endif

#include "sg_Win32Font.hxx"
#include "sg_Win32Text.hxx"
#include "sg_Compass.hxx"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDem3dView

IMPLEMENT_DYNCREATE(CDem3dView, COpenGLView)

BEGIN_MESSAGE_MAP(CDem3dView, COpenGLView)
	//{{AFX_MSG_MAP(CDem3dView)
	ON_COMMAND(ID_DEM_VIEWHEADERINFO, OnDemViewHeaderInfo)
	ON_COMMAND(ID_SHADERS_SLOPE, OnShadersSlope)
	ON_COMMAND(ID_SHADERS_TERRACE, OnShadersTerrace)
	ON_COMMAND(ID_TOOLS_MOVECAMERA, OnToolsMoveCamera)
	ON_COMMAND(ID_TOOLS_MOVELIGHTS, OnToolsMovelLights)
	ON_COMMAND(ID_SHADERS_DAYLIGHT, OnShadersDaylight)
	ON_COMMAND(ID_SHADERS_HEADING, OnShadersHeading)
	ON_COMMAND(ID_SHADERS_HSV, OnShadersHSV)
	ON_COMMAND(ID_SHADERS_INTERVAL, OnShadersInterval)
	ON_WM_CREATE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_COMMAND(ID_SCENE_GRID, OnSceneGrid)
	ON_UPDATE_COMMAND_UI(ID_SCENE_GRID, OnUpdateSceneGrid)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_MOVECAMERA, OnUpdateToolsMoveCamera)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_MOVELIGHTS, OnUpdateToolsMoveLights)
	ON_UPDATE_COMMAND_UI(ID_SHADERS_DAYLIGHT, OnUpdateShadersDaylight)
	ON_UPDATE_COMMAND_UI(ID_SHADERS_HEADING, OnUpdateShadersHeading)
	ON_UPDATE_COMMAND_UI(ID_SHADERS_HSV, OnUpdateShadersHSV)
	ON_UPDATE_COMMAND_UI(ID_SHADERS_INTERVAL, OnUpdateShadersInterval)
	ON_UPDATE_COMMAND_UI(ID_SHADERS_SLOPE, OnUpdateShadersSlope)
	ON_UPDATE_COMMAND_UI(ID_SHADERS_TERRACE, OnUpdateShadersTerrace)
	ON_COMMAND(ID_SCENE_CAPTION, OnSceneCaption)
	ON_UPDATE_COMMAND_UI(ID_SCENE_CAPTION, OnUpdateSceneCaption)
	ON_COMMAND(ID_SCENE_COMPASS, OnSceneCompass)
	ON_UPDATE_COMMAND_UI(ID_SCENE_COMPASS, OnUpdateSceneCompass)
	ON_COMMAND(ID_DEM_RESOLUTION, OnDemResolution)
	ON_COMMAND(ID_DEM_VERTICALEXAGGERATION, OnDemVerticalExageration)
	ON_UPDATE_COMMAND_UI(ID_DEM_RESOLUTION, OnUpdateDemResolution)
	ON_UPDATE_COMMAND_UI(ID_DEM_VERTICALEXAGGERATION, OnUpdateDemVerticalExag)
	ON_UPDATE_COMMAND_UI(ID_DEM_VIEWHEADERINFO, OnUpdateDemViewHeaderInfo)
	ON_COMMAND(ID_SHADERS_OPTIONS_TERRACE, OnShadersOptionsTerrace)
	ON_UPDATE_COMMAND_UI(ID_SHADERS_OPTIONS_TERRACE, OnUpdateShadersOptionsTerrace)
	ON_WM_TIMER()
	ON_COMMAND(ID_SHADERS_OPTIONS_DAYLIGHT, OnShadersOptionsDaylight)
	ON_UPDATE_COMMAND_UI(ID_SHADERS_OPTIONS_DAYLIGHT, OnUpdateShadersOptionsDaylight)
	ON_COMMAND(ID_SHADERS_OPTIONS_SLOPE, OnShadersOptionsSlope)
	ON_UPDATE_COMMAND_UI(ID_SHADERS_OPTIONS_SLOPE, OnUpdateShadersOptionsSlope)
	ON_COMMAND(ID_SCENE_OPTIONS_GRID, OnSceneOptionsGrid)
	ON_UPDATE_COMMAND_UI(ID_SCENE_OPTIONS_GRID, OnUpdateSceneOptionsGrid)
	ON_COMMAND(ID_SCENE_BACKGROUNDCOLOR, OnSceneBackGroundColor)
	ON_UPDATE_COMMAND_UI(ID_SCENE_BACKGROUNDCOLOR, OnUpdateSceneBackgroundColor)
	ON_COMMAND(ID_SHADERS_GRADIENT, OnShadersGradient)
	ON_UPDATE_COMMAND_UI(ID_SHADERS_GRADIENT, OnUpdateShadersGradient)
	ON_COMMAND(ID_SHADERS_OPTIONS_GRADIENT, OnShadersOptionsGradient)
	ON_UPDATE_COMMAND_UI(ID_SHADERS_OPTIONS_GRADIENT, OnUpdateShadersOptionsGradient)
	ON_COMMAND(ID_SHADERS_LAKE, OnShadersLake)
	ON_UPDATE_COMMAND_UI(ID_SHADERS_LAKE, OnUpdateShadersLake)
	ON_COMMAND(ID_SHADERS_OPTIONS_HSV, OnShadersOptionsHSV)
	ON_UPDATE_COMMAND_UI(ID_SHADERS_OPTIONS_HSV, OnUpdateShadersOptionsHSV)
	ON_COMMAND(ID_DEM_SURFPROP, OnDemSurfProp)
	ON_UPDATE_COMMAND_UI(ID_DEM_SURFPROP, OnUpdateDemSurfProp)
	ON_COMMAND(ID_SCENE_LIGHT, OnSceneLight)
	ON_UPDATE_COMMAND_UI(ID_SCENE_LIGHT, OnUpdateSceneLight)
	ON_WM_SIZE()
	ON_COMMAND(ID_SCENE_CAMERA, OnSceneCamera)
	ON_COMMAND(ID_TOOLS_WATERLEVEL, OnToolsWaterLevel)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_WATERLEVEL, OnUpdateToolsWaterLevel)
	ON_COMMAND(ID_SCENE_WATERLEVEL, OnSceneWaterLevel)
	ON_UPDATE_COMMAND_UI(ID_SCENE_WATERLEVEL, OnUpdateSceneWaterLevel)
	ON_COMMAND(ID_SCENE_OPTIONS_WATER, OnSceneOptionsWater)
	ON_UPDATE_COMMAND_UI(ID_SCENE_OPTIONS_WATER, OnUpdateSceneOptionsWater)
//	ON_COMMAND(ID_TOOLS_MAGICTEST, OnToolsMagicTest)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDem3dView construction/destruction

CDem3dView::CDem3dView()
{
   _heightfield = 0;
   _demShadeStyle = sg_Appearance::FLAT;
   _headerInfoDlg = 0;   // We'll create this the first time we need it.
   _demResDlg = 0;
   _vertExagDlg = 0;
   _statBar = 0;
   _gridNode = 0;
   _gridShape = 0;
   _demNode = 0;
   _demShape = 0;
   _hsvShader = 0;
   _curShader = 0;
   _hsvShader = 0;
   _daylightShader = 0;
   _slopeShader = 0;
   _rangeShader = 0;
   _contourShader = 0;
   _headingShader = 0;
   _gradientShader = 0;
   _lakeShader = 0;

   _light = 0;
   _lightIconNode = 0;
   _lightIconShape = 0;
   _lightIconAlwaysOn = false;

   _demBoundBox = false;
   _loResApprox = true;
   _drawWater = false;

   _daylightDlg = 0;
   _hsvDlg = 0;
   _terraceDlg = 0;
   _slopeDlg = 0;
   _gradientDlg = 0;

   _demSurfDlg = 0;
   _backgroundDlg = 0;
   _cameraDlg = 0;
   _lightDlg = 0;
   _waterDlg = 0;

   _cameraController = 0;
   _magicController = 0;
   _lightController = 0;
   _waterController = 0;

   _introText = 0;
   _introTextNode = 0;
   _demNameText = 0;
   _demNameNode = 0;
   _aboutBoxText = 0;
   _aboutBoxNode = 0;
   _compass = 0;
   _compassNode = 0;
   _defaultFont = 0;

   _demLoRes = 0.05;
   _demHiRes = 1.0;

   _frameCounter = new FrameCounter(2);
}

CDem3dView::~CDem3dView()
{
   delete _headerInfoDlg;
   delete _demResDlg;
   delete _vertExagDlg;
   delete _hsvShader;
   delete _daylightShader;
   delete _slopeShader;
   delete _rangeShader;
   delete _contourShader;
   delete _headingShader;

  if (_introTextNode != 0)
      {
         getSceneRoot()->removeNode(_introTextNode);
         delete _introText;
         delete _introTextNode;
      }

   getSceneRoot()->removeNode(_compassNode);
   delete _compass;
   delete _compassNode;
   delete _defaultFont;
   delete _frameCounter;
}

Dem3dStatus*
CDem3dView::getDem3dStatus()
{
   return _statBar;
}




/////////////////////////////////////////////////////////////////////////////
// CDem3dView printing

BOOL CDem3dView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CDem3dView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CDem3dView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CDem3dView diagnostics

#ifdef _DEBUG
void CDem3dView::AssertValid() const
{
	CView::AssertValid();
}

void CDem3dView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CDem3dDoc* CDem3dView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDem3dDoc)));
	return (CDem3dDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDem3dView message handlers

void CDem3dView::OnDemViewHeaderInfo() 
{
   if (_headerInfoDlg == 0)
      {
         _headerInfoDlg = new HeaderInfoDlg;
         _headerInfoDlg->Create(IDD_HEADERDLG);
      }

// XXX --- COME BACK TO THIS SOON ---
   CDem3dDoc* doc = GetDocument();
   _headerInfoDlg->updateHeaderInfo(doc->getHeightField());

   _headerInfoDlg->ShowWindow(SW_SHOW);
}

void CDem3dView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
   CDem3dDoc* doc = GetDocument();

   // Initialize my status bar wrapper.
   if (_statBar == 0)
   {
      CMainFrame* pMainFrame = dynamic_cast<CMainFrame*>(GetParent());
	   _statBar = new Dem3dStatus(pMainFrame->getStatusBar());
   }

   _heightfield = doc->getHeightField();
   if (_heightfield == 0)
      {
         disableLoResTimer();
         return;
      }

   if (_demNode != 0)
      getSceneRoot()->removeNode(_demNode);

#if 0
   if (_curController != 0)
      _curController->deactivate();
   _curController = _cameraController;
   _curController->activate();
#endif
   setController(Camera);

   delete _demShape;
   _demShape = new sg_DEM(_heightfield);
   _demShape->setDrawBoundingBox(true);

   delete _waterController;
   _waterController = new sg_WaterLevelController(getCameraNode(), _demShape);
                                        
   _drawWater = false;                                                  
   if (_waterDlg != 0)
   {
      _waterDlg->setDEMShape(_demShape);
      _waterDlg->update();
      _waterDlg->Invalidate();
   }

   delete _hsvShader;
   delete _daylightShader;
   delete _slopeShader;
   delete _rangeShader;
   delete _contourShader;
   delete _headingShader;
   delete _gradientShader;
   delete _lakeShader;
   _hsvShader =      new sg_HSVShader(_heightfield);
   _hsvShader->setLowColorStart(226.8);
   _hsvShader->setSaturation(0.65);
   _hsvShader->setRange(206.0);
   _daylightShader = new sg_DaylightShader(_heightfield);
   _slopeShader =    new sg_SlopeShader(_heightfield);
   _rangeShader =    new sg_RangeShader(_heightfield);
	vector<sg_RGBA> naturalColors;
	naturalColors.push_back(sg_Color_Brown3);
	naturalColors.push_back(sg_Color_Brown1);
	naturalColors.push_back(sg_Color_DarkGreen);
	naturalColors.push_back(sg_Color_SeaGreen);
	naturalColors.push_back(sg_Color_SpringGreen);
	naturalColors.push_back(sg_Color_Gray);
	naturalColors.push_back(sg_Color_White);
	_rangeShader->setColors(naturalColors);
   _contourShader =  new sg_ContourShader(_heightfield);
   _contourShader->setContourInterval(100);
   _headingShader =  new sg_HeadingShader(_heightfield);
   _gradientShader = new sg_GradientShader(_heightfield);
   _lakeShader = new sg_LakeShader(_heightfield);


   _curShader = _hsvShader;

   if (_daylightDlg != 0)
      _daylightDlg->setShader(_daylightShader);
   if (_hsvDlg != 0)
      _hsvDlg->setShader(_hsvShader);
   if (_slopeDlg != 0)
      _slopeDlg->setShader(_slopeShader);
   if (_terraceDlg != 0)
      _terraceDlg->setShader(_contourShader);
   if (_gradientDlg != 0)
      _gradientDlg->setShader(_gradientShader);

   _statBar->setVertExag(getVertExag());
   if (_vertExagDlg != 0)
      _vertExagDlg->update();

   _demHiRes = 1.0;
   _statBar->setResolution(_demHiRes);
   if (_demResDlg != 0)
      _demResDlg->update();

   _demShape->setShader(_curShader);
   delete _demNode;
   _demNode = new sg_ShapeNode(_demShape);
   _demNode->setLighting(true);
   _demNode->setShininess(80.0);
   _demNode->setSpecularColor(sg_RGBA(0.4, 0.4, 0.4));
   if (_demSurfDlg != 0)
      {
         _demSurfDlg->setDEMNode(_demNode);
         _demSurfDlg->setDEMShape(_demShape);
         _demSurfDlg->UpdateData(FALSE);
      }



   getSceneRoot()->addNode(_demNode);

   // Calculate a nice viewing point, and adjust the camera.
   sg_MinBoundBox mbb;
   _demShape->getBoundingBox(mbb);

   sg_CameraNode* camera = getCameraNode();
   camera->setFrontClipPlane(mbb.max_y * 0.1);
   camera->setBackClipPlane(mbb.max_y * 10.0);
   _cameraController->setMinRadius(mbb.max_y * 0.1);
   _cameraController->setMaxRadius(mbb.max_y * 4.0);
   _cameraController->setCurrentRadius(mbb.max_y * 2.5);
   _cameraController->setCurrentAzimuth(0);
   _cameraController->setCurrentAltitude(30);

   _lightController->setRadius(mbb.max_y);
   _lightController->setCurrentAzimuth(270);
   _lightController->setCurrentAltitude(45);



   if (_headerInfoDlg != 0)
      _headerInfoDlg->updateHeaderInfo(_heightfield);

   // Get rid of the opening title.
   if (_introTextNode != 0)
      {
         KillTimer(2);
         getSceneRoot()->removeNode(_introTextNode);
         delete _introText;
         delete _introTextNode;
         _introText = 0;
         _introTextNode = 0;

         // Also turn off the grid.
         _gridNode->setActive(false);
      }

   // DEM Name text.
   _demNameText->setText(_heightfield->getQuadName());
   sg_MinBoundBox demNameMbb;
   _demNameNode->getBoundingBox(demNameMbb);

   double s = (mbb.max_y - mbb.min_y) / (demNameMbb.max_x - demNameMbb.min_x);
   _demNameNode->identity();
   _demNameNode->rotate(90,1,0,0);
   _demNameNode->rotate(90,0,1,0);
   _demNameNode->translate(0,mbb.max_z * 1.2, 0);
   _demNameNode->translate(0, 0, mbb.min_x);
   _demNameNode->scale(s,s,s);
   _demNameNode->translate(-1 * (demNameMbb.max_x - demNameMbb.min_x) / 2.0,0,0);
   _demNameNode->setActive(false);

   // Compass
   if (_compassNode != 0)
      getSceneRoot()->removeNode(_compassNode);
   delete _compass;
   delete _compassNode;
   double tickWidth = mbb.max_y / 50.0;
   _compass = new sg_Compass(_defaultFont,mbb.max_y * 1.5, 8, tickWidth, tickWidth * 5.0);
   _compassNode = new sg_ShapeNode(_compass);
   _compassNode->setColor(sg_Color_Yellow);
   _compassNode->setLighting(true);
   _compassNode->setPolygonMode(sg_Appearance::FILL);
   _compassNode->setActive(false);
   getSceneRoot()->addNode(_compassNode);


   // Setup Grid Size

   _gridShape->setWidth(((mbb.max_x - mbb.min_x) * 0.25) + (mbb.max_x - mbb.min_x));
   _gridShape->setHeight(((mbb.max_y - mbb.min_y) * 0.25) + (mbb.max_y - mbb.min_y));

   setVertExag(1.0);
   setRenderMode(LORES);
   enableLoResTimer();
   startRender();
}

void CDem3dView::OnShadersSlope() 
{
	// TODO: Add your command handler code here
   setRenderMode(LORES);
   _curShader = _slopeShader;
   _demShape->setShader(_curShader);
   startRender();
	
}

void CDem3dView::OnShadersTerrace() 
{
	// TODO: Add your command handler code here
	setRenderMode(LORES);
   _curShader = _contourShader;
   _demShape->setShader(_curShader);
   startRender();
}

void
CDem3dView::setController(ControllerType cntrlr)
{
   setRenderMode(LORES);
   if (_curController != 0)
      _curController->deactivate();

   switch(cntrlr)
   {
      case(Camera):
      case(PoleCamera):
         _curController = _cameraController;
         break;
      case(MagicCamera):
         _curController = _magicController;
         break;
      case(Light):
         _curController = _lightController;
         break;
      case(Water):
         _curController = _waterController;
         break;
   }
   _curController->activate();

   startRender();
}

void CDem3dView::OnToolsMoveCamera() 
{
   setController(Camera);
}

void CDem3dView::OnToolsMovelLights() 
{
   setController(Light);
}

void CDem3dView::OnShadersDaylight() 
{
   setRenderMode(LORES);
   _curShader = _daylightShader;
   _demShape->setShader(_curShader);
   startRender();
}

void CDem3dView::OnShadersHeading() 
{
   setRenderMode(LORES);
   _curShader = _headingShader;
   _demShape->setShader(_curShader);
   startRender();	
}

void CDem3dView::OnShadersHSV() 
{
   setRenderMode(LORES);
   _curShader = _hsvShader;
   _demShape->setShader(_curShader);
   startRender();	
}

void CDem3dView::OnShadersInterval() 
{
   setRenderMode(LORES);
   _curShader = _rangeShader;
   _demShape->setShader(_curShader);
   startRender();	
}

int CDem3dView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (COpenGLView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
   CDem3dApp* oglApp;
   oglApp = dynamic_cast<CDem3dApp*>(AfxGetApp());
   oglApp->setDem3dView(this);

   setRenderMode(LORES);

   sg_CompositeNode* sceneRoot;
   sceneRoot = new sg_CompositeNode;

   sg_CameraNode* camera = new sg_CameraNode;
   camera->look(0, 0, 0,
                0, 1, 0,
                0, 0, 1);
   camera->setFieldOfView(40.0);
   camera->setFrontClipPlane(500);
   camera->setBackClipPlane(40000);
   sceneRoot->addNode(camera);
   setCameraNode(camera);

   glEnable(GL_LIGHT1);
   _lightIconShape = new sg_LightBox(300);
   _lightIconNode = new sg_ShapeNode(_lightIconShape);
   _light = new sg_DistantLight(GL_LIGHT1);
   _light->setIcon(_lightIconNode);
   _light->setDiffuse(sg_Color_White);
   _light->setPosition(sg_Point(0,0,8000));
   _light->shineOn(sg_Point(0,0,0));
   sceneRoot->addNode(_light);

   _cameraController = new sg_ControlLineController(camera);
   _cameraController->setMinRadius(100);
   _cameraController->setMaxRadius(20000);
   _cameraController->setNode(camera);
   _cameraController->sync();
   _cameraController->setCurrentRadius(3000);
   _cameraController->setCurrentAzimuth(345);
   _cameraController->setCurrentAltitude(20);

   _curController = _cameraController;

   // XXX Just for testing...
   _magicController = new sg_MagicCarpetController(camera);
   _magicController->setWorldUnitsPerPixel(10);

   _lightController = new sg_AltAzLightController(camera);
   _lightController->setLight(_light);
   _lightController->sync();
   _lightController->setRadius(4800);
   _lightController->setCurrentAzimuth(270);
   _lightController->setCurrentAltitude(30);

   _gridShape = new sg_Grid();
   _gridNode = new sg_ShapeNode(_gridShape);
   _gridNode->setColor(sg_Color_LightGreen);
   _gridNode->translate(0,0,-500);
   _gridNode->setLighting(false);
   _gridNode->setPolygonMode(sg_Appearance::FILL);
   sceneRoot->addNode(_gridNode);


   LOGFONT logfont;
   logfont.lfHeight = -12;
   logfont.lfWidth = 0;
   logfont.lfEscapement = 0;
   logfont.lfWeight = FW_NORMAL;
   logfont.lfItalic = FALSE;
   logfont.lfUnderline = FALSE;
   logfont.lfStrikeOut = FALSE;
   logfont.lfCharSet = ANSI_CHARSET;
   logfont.lfOutPrecision = OUT_DEFAULT_PRECIS;
   logfont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
   logfont.lfQuality = DEFAULT_QUALITY;
   logfont.lfPitchAndFamily = FF_DONTCARE | DEFAULT_PITCH;

   lstrcpy(logfont.lfFaceName, "");

   CFont newfont;
   newfont.CreateFontIndirect(&logfont);
   CFont* oldfont = m_pDC->SelectObject(&newfont);
   GLYPHMETRICSFLOAT agmf[256];
   glColor3d(0.8, 0.8, 0.8);

   // create display lists for glyphs 0 through 255 with 0.1 extrusion 
   // and default deviation. The display list numbering starts at 1000 
   // (it could be any number) 
   wglUseFontOutlines(m_pDC->GetSafeHdc(), 0, 255, 1000, 0.0f, 0.3f,  
                      WGL_FONT_POLYGONS, agmf); 
   m_pDC->SelectObject(oldfont);

   _defaultFont = new sg_Win32Font(1000, agmf, 0.3);
    
   sg_MinBoundBox mbb;
   char* titlestr = "dem3d 2.0a";
   _introText = new sg_Win32Text(_defaultFont, titlestr);
   _introTextNode = new sg_ShapeNode(_introText);
   _introTextNode->setColor(sg_Color_LightBlue);
   _introTextNode->getBoundingBox(mbb);

   _introTextNode->rotate(90, 1, 0, 0);
   _introTextNode->rotate(45, 0, 1, 0);
   _introTextNode->scale(500,500,500);
   _introTextNode->translate(-1 * (mbb.max_x / 2.0),0,0);
   _introTextNode->setLighting(true);
   _introTextNode->setPolygonMode(sg_Appearance::FILL);
   sceneRoot->addNode(_introTextNode);


   // DEM caption.
   _demNameText = new sg_Win32Text(_defaultFont, "Testing...");
   _demNameNode = new sg_ShapeNode(_demNameText);
   _demNameNode->setColor(sg_Color_LightBlue);
   _demNameNode->setActive(false);
   _demNameNode->setLighting(true);
   _demNameNode->setPolygonMode(sg_Appearance::FILL);
   sceneRoot->addNode(_demNameNode);
   
   setSceneRoot(sceneRoot);
   setRenderMode(LORES);

	return 0;
}

void CDem3dView::OnLButtonDown(UINT nFlags, CPoint point) 
{
   ::SetCapture(*this);
	
	COpenGLView::OnLButtonDown(nFlags, point);

   disableLoResTimer();
   setRenderMode(LORES);
   _curController->sync();
   startRender();
}

void CDem3dView::OnLButtonUp(UINT nFlags, CPoint point) 
{
   ::ReleaseCapture();

	COpenGLView::OnLButtonUp(nFlags, point);

   _curController->button1Up(point.x, point.y);
   enableLoResTimer();
   startRender();
}

void CDem3dView::OnMouseMove(UINT nFlags, CPoint point) 
{
	COpenGLView::OnMouseMove(nFlags, point);

   // Store the point coords here so we can have them
   // in OnIdle(). (See my rant about this in CDem3dView.h -- jamie)
   _mouseX = point.x;
   _mouseY = point.y;

   long keystat = sg_Controller::noKeysPressed;
   if (nFlags & MK_LBUTTON)
      keystat = keystat | sg_Controller::button1;
   if (nFlags & MK_RBUTTON)
      keystat = keystat | sg_Controller::button2;
   if (nFlags & MK_SHIFT)
      keystat = keystat | sg_Controller::shiftKey;

   if ((keystat & sg_Controller::button1) ||
       (keystat & sg_Controller::button2))
   {
      _curController->mouseMove(point.x, point.y, keystat);
      startRender();
   }

#if 0
	if (nFlags == MK_LBUTTON) {
		_curController->mouseMove(point.x, point.y, sg_Controller::button1);
      startRender();
	}
	if (nFlags == MK_RBUTTON) {
		_curController->mouseMove(point.x, point.y, sg_Controller::button2);
		startRender();
	}
#endif

}

void CDem3dView::OnRButtonDown(UINT nFlags, CPoint point) 
{
   ::SetCapture(*this);
	
	COpenGLView::OnRButtonDown(nFlags, point);
   disableLoResTimer();
   setRenderMode(LORES);
   _curController->sync();
   startRender();
}

void CDem3dView::OnRButtonUp(UINT nFlags, CPoint point) 
{
   ::ReleaseCapture();
	
	COpenGLView::OnRButtonUp(nFlags, point);

   _curController->button1Up(point.x, point.y);
   enableLoResTimer();
   startRender();
}

void CDem3dView::OnSceneGrid() 
{
   _gridNode->setActive(!(_gridNode->isActive()));
   startRender();
}

void CDem3dView::OnUpdateSceneGrid(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(_gridNode->isActive());	
}

void CDem3dView::OnUpdateToolsMoveCamera(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(_curController == _cameraController);	
}

void CDem3dView::OnUpdateToolsMoveLights(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(_curController == _lightController);	
}

void
CDem3dView::setRenderMode(RenderMode mode)
{
   switch(mode)
      {
         case LORES:
            // In case we're interrupting HIRES
            if (_statBar != 0)
               _statBar->setProgress(0.0);

            if (_demShape != 0)
            {
               _demShape->setResolution(_demLoRes);
               _demShape->setDrawBoundingBox(_demBoundBox);
               _demShape->setDrawDEM(_loResApprox);
            }
            break;
         case HIRES:
            if (_demShape != 0)
            {
               _demShape->setResolution(1.0);
               _demShape->setDrawBoundingBox(false);
               _demShape->setDrawDEM(true);
            }
            break;
      }

   COpenGLView::setRenderMode(mode);
}

void
CDem3dView::setResolutionDEM(double res)
{
   disableLoResTimer();

   _demHiRes = res;
   _statBar->setResolution(res);

   setRenderMode(LORES);
   enableLoResTimer();
   startRender();
}

double
CDem3dView::getResolutionDEM()
{
   return _demHiRes;
}

void
CDem3dView::setVertExag(double exag)
{
   disableLoResTimer();
   if (_demShape != 0)
      {
         _demShape->setVerticalExageration(exag);
         _statBar->setVertExag(exag);
         if (_gridNode != 0)
            {
               sg_MinBoundBox mbb;
               _demShape->getBoundingBox(mbb);
               _gridNode->identity();
               if (mbb.min_z < mbb.max_z)
                  _gridNode->translate(0,0,mbb.min_z * 1.2 - 100.0);
               else
                  _gridNode->translate(0,0,mbb.max_z * 1.2 - 100.0);
            }
      }
   setRenderMode(LORES);
   enableLoResTimer();
   startRender();
}

double
CDem3dView::getVertExag()
{
   if (_demShape != 0)
      return _demShape->getVerticalExageration();

   return 1.0;
}

void
CDem3dView::setLightIconAlwaysOn(bool flag)
{
   _lightIconAlwaysOn = flag;

   if (_lightIconAlwaysOn)
      _light->showIcon(true);
   else
      if (_curController != _lightController)
         _light->showIcon(false);

}

bool
CDem3dView::lightIconAlwaysOn() const
{
   return _lightIconAlwaysOn;
}

bool
CDem3dView::demBoundBox() const
{
   return _demBoundBox;
}

void
CDem3dView::setDEMBoundBox(bool flag)
{
   _demBoundBox = flag;
}

bool
CDem3dView::loResApprox() const
{
   return _loResApprox;
}

void
CDem3dView::setLoResApprox(bool flag)
{
   _loResApprox = flag;
}

bool
CDem3dView::getDrawWater() const
{
   return _drawWater;
}

void
CDem3dView::setDrawWater(bool flag)
{
   _drawWater = flag;
}

void
CDem3dView::startRender()
{
   if (_curController == _lightController)
      _light->showIcon(true);
   else
      _light->showIcon(_lightIconAlwaysOn);

   if (_demShape != 0)
   {
      if (_curController == _waterController)
         _demShape->setDrawWater(true);
      else
         _demShape->setDrawWater(_drawWater);
   }

   // Dialog updates.
   if (_curController == _lightController)
   {
      if (_lightDlg != 0)
         _lightDlg->update();
   }

   if (_curController == _cameraController)
   {
      if (_cameraDlg != 0)
         _cameraDlg->update();
   }

   if (_curController == _waterController)
   {
     if (_waterDlg != 0)
     {
         _waterDlg->update();
         _waterDlg->Invalidate();
     }
   }

   COpenGLView::startRender();
}

BOOL
CDem3dView::OnIdle(LONG lCount)
{
   if (_curController == _cameraController)
      _statBar->setCameraAltAz(_cameraController->getCurrentAltitude(),
                               _cameraController->getGeoAzimuth());
   else
   if (_curController == _lightController)
      _statBar->setLightAltAz(_lightController->getCurrentAltitude(),
                              _lightController->getGeoAzimuth());
   else
   if ((_curController == _waterController) &&
      (_demShape != 0))
      _statBar->setWaterLevel(_demShape->getWaterLevel());

   if (_demShape != 0)
   {
      // Determine the object space coordinates of the cursor.
      GLdouble modelview[16];
      GLdouble projection[16];
      _demShape->getModelView(modelview);
      _demShape->getProjection(projection);

      GLint viewport[4];
      glGetIntegerv(GL_VIEWPORT, viewport);

      RECT rc;
      GetClientRect(&rc);
      GLfloat zval;
      glReadPixels(_mouseX, rc.bottom - _mouseY, 1, 1,
                   GL_DEPTH_COMPONENT,
                   GL_FLOAT, &zval);
      GLdouble winx, winy, winz;
      winx = _mouseX;
      winy = rc.bottom - _mouseY;
      winz = zval;
      GLdouble objx, objy, objz;
      if (gluUnProject(winx, winy, winz,
                       modelview, projection, viewport,
                       &objx, &objy, &objz) == GL_TRUE)
      {
         sg_MinBoundBox mbb;
         _demShape->getBoundingBox(mbb);

         // If the point isn't in the DEM, we aren't interested.
         if ((mbb.min_x <= objx) && (mbb.max_x >= objx) &&
             (mbb.min_y <= objy) && (mbb.max_y >= objy) &&
             (fabs(mbb.max_z - mbb.min_z) > 0.0001))
         {
            // Convert the object space coords to map
            // coords.
            double minElev = _demShape->getHeightField()->getMinElev();
            double maxElev = _demShape->getHeightField()->getMaxElev();
   
            double elev = ((objz - mbb.min_z) / (mbb.max_z - mbb.min_z)) * 
                          (maxElev - minElev) + minElev;
            // XXX This'll have to be changed when the heightfield stuff
            // XXX is redone. 'WorldUnits'? horizontal or verical? -- jamie
            if (_demShape->getHeightField()->getWorldUnits() == hf_DEM::FEET)
               _statBar->setElevationUnits(Dem3dStatus::FEET);
            else
               _statBar->setElevationUnits(Dem3dStatus::METERS);
            _statBar->setCoords(objx, objy, elev);
         }
         else
            _statBar->setCoords(0.0, 0.0, 0.0, false);
      }

   }

   return COpenGLView::OnIdle(lCount);
}

void CDem3dView::OnUpdateShadersDaylight(CCmdUI* pCmdUI) 
{
   if (_demNode == 0)
      {
         pCmdUI->Enable(false);
         return;
      }
   pCmdUI->SetCheck(_curShader == _daylightShader);	
}

void CDem3dView::OnUpdateShadersHeading(CCmdUI* pCmdUI) 
{
   if (_demNode == 0)
      {
         pCmdUI->Enable(false);
         return;
      }
   pCmdUI->SetCheck(_curShader == _headingShader);	
}

void CDem3dView::OnUpdateShadersHSV(CCmdUI* pCmdUI) 
{
   if (_demNode == 0)
      {
         pCmdUI->Enable(false);
         return;
      }
   pCmdUI->SetCheck(_curShader == _hsvShader);	
}

void CDem3dView::OnUpdateShadersInterval(CCmdUI* pCmdUI) 
{
   if (_demNode == 0)
      {
         pCmdUI->Enable(false);
         return;
      }
   pCmdUI->SetCheck(_curShader == _rangeShader);	
}

void CDem3dView::OnUpdateShadersSlope(CCmdUI* pCmdUI) 
{
   if (_demNode == 0)
      {
         pCmdUI->Enable(false);
         return;
      }
   pCmdUI->SetCheck(_curShader == _slopeShader);	
}

void CDem3dView::OnUpdateShadersTerrace(CCmdUI* pCmdUI) 
{
   if (_demNode == 0)
      {
         pCmdUI->Enable(false);
         return;
      }
   pCmdUI->SetCheck(_curShader == _contourShader);	
}

void CDem3dView::OnSceneCaption() 
{
   if (_demNameNode == 0)
      return;

   if (_demNameNode->isActive())
      _demNameNode->setActive(false);
   else
      _demNameNode->setActive(true);

   startRender();
}

void CDem3dView::OnUpdateSceneCaption(CCmdUI* pCmdUI) 
{
   if (_demNode == 0)
      {
         pCmdUI->Enable(false);
         return;
      }

   pCmdUI->SetCheck(_demNameNode->isActive());	
}

void CDem3dView::OnSceneCompass() 
{
   if (_compassNode == 0)
      return;
      
   if (_compassNode->isActive())
      _compassNode->setActive(false);
   else
      _compassNode->setActive(true);
      
   startRender();	
}

void CDem3dView::OnUpdateSceneCompass(CCmdUI* pCmdUI) 
{
   if (_compassNode == 0)
      {
         pCmdUI->Enable(false);
         return;
      }

   pCmdUI->SetCheck(_compassNode->isActive());
}


void
CDem3dView::preRenderLoRes()
{
   COpenGLView::preRenderLoRes();

   if (_loResApprox)
   {
      if (_demNode != 0)
         {
            _demShape->setResolution(_demLoRes);
            _demShadeStyle = _demNode->getShadingStyle();
            _demNode->setShadingStyle(sg_Appearance::FLAT);
            _drawTransWater = _demShape->getTransparentWater();
            _demShape->setTransparentWater(false);

         }
	   _startTime = timeGetTime();
   }
}

void CDem3dView::postRenderLoRes()
{

	COpenGLView::postRenderLoRes();

   if (_loResApprox)
   {
      _frameCounter->frameCompleted(timeGetTime() - _startTime);
      double rate = _frameCounter->getFramesPerSec();

      double delta = 1 / (rate*20);
	   if (rate > 8)
   		_demLoRes = _demLoRes + delta;
   	if (rate < 5)
		   if ((_demLoRes - delta) > 0.02)
			   _demLoRes = _demLoRes - delta;

      if (_demNode != 0)
         _demNode->setShadingStyle(_demShadeStyle);
   }

   if (_demShape != 0)
   {
      _demShape->setTransparentWater(_drawTransWater);
   }
}

long
CDem3dView::render()
{
   if (_renderMode == HIRES)
   {
      double percent = (_totalHiResRenderSteps - _hiResRenderSteps)
                       * 100.00 / _totalHiResRenderSteps;
      _statBar->setProgress(percent);
   }
   return COpenGLView::render();
}

void
CDem3dView::preRenderHiRes()
{
   COpenGLView::preRenderHiRes();

   if (_demNode != 0)
      {
         _demShape->setResolution(_demHiRes);
         _totalHiResRenderSteps = _hiResRenderSteps;
      }
   else
      _totalHiResRenderSteps = 0;
}

void
CDem3dView::postRenderHiRes()
{
   _statBar->setProgress(0.0);
}
void CDem3dView::OnDemResolution() 
{
   if (_demResDlg == 0)
      {
         _demResDlg = new DemResDlg(this);
         _demResDlg->Create(IDD_DEMRES);
      }

   _demResDlg->ShowWindow(SW_SHOW);	
}

void CDem3dView::OnDemVerticalExageration() 
{
   if (_vertExagDlg == 0)
      {
         _vertExagDlg = new VertExagDlg(this);
         _vertExagDlg->Create(IDD_VERTEXAG);
      }

   _vertExagDlg->ShowWindow(SW_SHOW);
}

void CDem3dView::OnUpdateDemResolution(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(_demShape != 0);	
}

void CDem3dView::OnUpdateDemVerticalExag(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(_demShape != 0);	
}

void CDem3dView::OnUpdateDemViewHeaderInfo(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(_heightfield != 0);	
}

void CDem3dView::OnShadersOptionsTerrace() 
{
   if (_terraceDlg == 0)
      {
         _terraceDlg = new TerraceConfigDlg(this);
         _terraceDlg->Create(IDD_TERRACEDLG);
      }

   _terraceDlg->setShader(_contourShader);
   _terraceDlg->ShowWindow(SW_SHOW);
}

void CDem3dView::OnUpdateShadersOptionsTerrace(CCmdUI* pCmdUI) 
{
   if (_contourShader == 0)
      pCmdUI->Enable(false);	
}

void CDem3dView::OnTimer(UINT nIDEvent) 
{
   COpenGLView::OnTimer(nIDEvent);
}

void CDem3dView::OnShadersOptionsDaylight() 
{
   if (_daylightDlg == 0)
      {
         _daylightDlg = new DaylightConfigDlg(this);
         _daylightDlg->Create(IDD_DAYLIGHTDLG);
      }

   _daylightDlg->setShader(_daylightShader);
   _daylightDlg->ShowWindow(SW_SHOW);	
}

void CDem3dView::OnUpdateShadersOptionsDaylight(CCmdUI* pCmdUI) 
{
   if (_daylightShader == 0)
      pCmdUI->Enable(false);	
}

void CDem3dView::OnShadersOptionsSlope() 
{
   if (_slopeDlg == 0)
      {
         _slopeDlg = new SlopeConfigDlg(this);
         _slopeDlg->Create(IDD_SLOPEDLG);
      }

   _slopeDlg->setShader(_slopeShader);
   _slopeDlg->ShowWindow(SW_SHOW);	
}

void CDem3dView::OnUpdateShadersOptionsSlope(CCmdUI* pCmdUI) 
{
   if (_slopeShader == 0)
      pCmdUI->Enable(false);	
}

void CDem3dView::OnSceneOptionsGrid() 
{
}

void CDem3dView::OnUpdateSceneOptionsGrid(CCmdUI* pCmdUI) 
{
   // XXX Disable it for now.
   pCmdUI->Enable(false);	
}

void CDem3dView::OnSceneBackGroundColor() 
{
   if (_backgroundDlg == 0)
      {
         _backgroundDlg = new BackgroundConfigDlg(this);
         _backgroundDlg->Create(IDD_BKGRNDDLG);
      }	
   _backgroundDlg->ShowWindow(SW_SHOW);
}

void CDem3dView::OnUpdateSceneBackgroundColor(CCmdUI* pCmdUI) 
{
	
}

void CDem3dView::OnShadersGradient() 
{
   setRenderMode(LORES);
   _curShader = _gradientShader;
   _demShape->setShader(_curShader);
   startRender();	
}

void CDem3dView::OnUpdateShadersGradient(CCmdUI* pCmdUI) 
{
   if (_demNode == 0)
      {
         pCmdUI->Enable(false);
         return;
      }
   pCmdUI->SetCheck(_curShader == _gradientShader);		
}


void CDem3dView::OnShadersOptionsGradient() 
{
   if (_gradientDlg == 0)
      {
         _gradientDlg = new GradientDlg(this);
         _gradientDlg->Create(IDD_GRADIENTDLG);
      }

   _gradientDlg->setShader(_gradientShader);
   _gradientDlg->ShowWindow(SW_SHOW);		
}

void CDem3dView::OnUpdateShadersOptionsGradient(CCmdUI* pCmdUI) 
{
   if (_gradientShader == 0)
      pCmdUI->Enable(false);	
}


void CDem3dView::OnShadersLake() 
{
   setRenderMode(LORES);
   _curShader = _lakeShader;
   _demShape->setShader(_curShader);
   startRender();		
}

void CDem3dView::OnUpdateShadersLake(CCmdUI* pCmdUI) 
{
   if (_demNode == 0)
      {
         pCmdUI->Enable(false);
         return;
      }
   pCmdUI->SetCheck(_curShader == _lakeShader);			
}

void CDem3dView::OnShadersOptionsHSV() 
{
   if (_hsvDlg == 0)
      {
         _hsvDlg = new HSVConfigDlg(this);
         _hsvDlg->Create(IDD_HSVDLG);
      }

   _hsvDlg->setShader(_hsvShader);
   _hsvDlg->ShowWindow(SW_SHOW);
}

void CDem3dView::OnUpdateShadersOptionsHSV(CCmdUI* pCmdUI) 
{
   if (_hsvShader == 0)
      pCmdUI->Enable(false);	
}

void CDem3dView::OnDemSurfProp() 
{
   if (_demSurfDlg == 0)
      {
         _demSurfDlg = new DEMSurfacePropDlg(this,_demNode,_demShape);
         _demSurfDlg->Create(IDD_DEMSURFDLG);
      }

   _demSurfDlg->ShowWindow(SW_SHOW);	
}

void CDem3dView::OnUpdateDemSurfProp(CCmdUI* pCmdUI) 
{
   if (_demNode == 0)
      pCmdUI->Enable(false);	
}



void CDem3dView::OnSceneLight() 
{
   if (_lightDlg == 0)
   {
      _lightDlg = new LightDlg(this, _lightController);
      _lightDlg->Create(IDD_LIGHTDLG);
   }	

   _lightDlg->update();
   _lightDlg->ShowWindow(SW_SHOW);
}

void CDem3dView::OnUpdateSceneLight(CCmdUI* pCmdUI) 
{
	
}

void CDem3dView::OnSize(UINT nType, int cx, int cy) 
{
	COpenGLView::OnSize(nType, cx, cy);
	
   if (_statBar != 0)
      _statBar->resize();	
}

void CDem3dView::OnSceneCamera() 
{
   if (_cameraDlg == 0)
   {
      _cameraDlg = new CameraDlg(this, _cameraController, getCameraNode());
      _cameraDlg->Create(IDD_CAMERADLG);
   }	

   _cameraDlg->update();
   _cameraDlg->ShowWindow(SW_SHOW);	
}

void CDem3dView::OnToolsWaterLevel() 
{
   setController(Water);
}

void CDem3dView::OnUpdateToolsWaterLevel(CCmdUI* pCmdUI) 
{
   if (_demShape == 0)
   {
      pCmdUI->Enable(FALSE);
      return;
   }

   pCmdUI->SetCheck(_curController == _waterController);
}

void CDem3dView::OnSceneWaterLevel() 
{
   _drawWater = !_drawWater;
   _demShape->setDrawWater(_drawWater);
   startRender();
}

void CDem3dView::OnUpdateSceneWaterLevel(CCmdUI* pCmdUI) 
{
   if (_demShape == 0)
   {
      pCmdUI->Enable(FALSE);
      return;
   }
   pCmdUI->SetCheck(_drawWater);
}

void CDem3dView::OnSceneOptionsWater() 
{
   if (_waterDlg == 0)
   {
      _waterDlg = new WaterDlg(this,_demShape);
      _waterDlg->Create(IDD_WATERDLG);
   }

   _waterDlg->ShowWindow(SW_SHOW);	
}

void CDem3dView::OnUpdateSceneOptionsWater(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(_demShape != 0);	
}

void CDem3dView::OnToolsMagicTest() 
{
//   setController(MagicCamera);	
}
