// dem3dView.h : interface of the CDem3dView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_DEM3DVIEW_H__E0AB4213_ABDE_11D0_80AB_02608CDA2E56__INCLUDED_)
#define AFX_DEM3DVIEW_H__E0AB4213_ABDE_11D0_80AB_02608CDA2E56__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "OpenGLView.h"

class HeaderInfoDlg;
class DemResDlg;
class VertExagDlg;
class Dem3dStatus;
class hf_DEM;
class sg_CompositeNode;
class sg_Controller;
class sg_ControlLineController;
class sg_MagicCarpetController;
class sg_AltAzLightController;
class sg_WaterLevelController;
class sg_PointLight;
class sg_DistantLight;
class sg_LightBox;
class sg_ShapeNode;
class sg_DEM;
class sg_DEMVertexShader;
class sg_HSVShader;
class sg_DaylightShader;
class sg_SlopeShader;
class sg_RangeShader;
class sg_ContourShader;
class sg_HeadingShader;
class sg_GradientShader;
class sg_LakeShader;
class sg_Win32Font;
class sg_Win32Text;
class sg_Compass;
class sg_Grid;
class FrameCounter;
class sg_Plane;

class LightDlg;
class CameraDlg;
class BackgroundConfigDlg;
class DaylightConfigDlg;
class HSVConfigDlg;
class SlopeConfigDlg;
class TerraceConfigDlg;
class GradientDlg;
class DEMSurfacePropDlg;
class WaterDlg;



#ifndef INCLUDED_SG_APPEARANCE_HXX
#include "scenegraph/sg_Appearance.hxx"
#endif

class CDem3dView : public COpenGLView
{
protected: // create from serialization only
	CDem3dView();
	DECLARE_DYNCREATE(CDem3dView)

// Attributes
public:
	CDem3dDoc* GetDocument();

// Operations
public:

   void setRenderMode(RenderMode mode);

   void setResolutionDEM(double res);
      // 0.0 <= res <= 1.0
   double getResolutionDEM();

   void setVertExag(double exag);
   double getVertExag();

   Dem3dStatus* getDem3dStatus();

   void setLightIconAlwaysOn(bool flag);
   bool lightIconAlwaysOn() const;

   bool demBoundBox() const;
   void setDEMBoundBox(bool flag);

   bool loResApprox() const;
   void setLoResApprox(bool flag);

   bool getDrawWater() const;
   void setDrawWater(bool flag);

   void startRender();
   BOOL OnIdle(LONG lCount);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDem3dView)
	public:
	virtual void OnInitialUpdate();
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CDem3dView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

   hf_DEM* _heightfield;
   sg_Appearance::ShadingStyle _demShadeStyle;
   sg_Controller* _curController;
   sg_AltAzLightController* _lightController;
   sg_ControlLineController* _cameraController;
   sg_MagicCarpetController* _magicController;
   sg_WaterLevelController* _waterController;
   sg_DistantLight* _light;
   sg_ShapeNode* _lightIconNode;
   sg_LightBox* _lightIconShape;
   bool _lightIconAlwaysOn;
   bool _demBoundBox;
   bool _loResApprox;
   bool _drawWater;

   sg_Win32Text* _aboutBoxText;
   sg_ShapeNode* _aboutBoxNode;
   sg_CompositeNode* _oldSceneRoot;
   sg_CameraNode* _oldCamera;
   sg_Controller* _oldController;

   sg_ShapeNode* _compassNode;
   sg_Compass* _compass;
   sg_ShapeNode* _gridNode;
   sg_Grid* _gridShape;
   sg_ShapeNode* _demNode;
   sg_DEM* _demShape;
   double _demLoRes;  // Default 0.05;
   double _demHiRes;

   sg_Win32Font* _defaultFont;
   sg_Win32Text* _introText;
   sg_ShapeNode* _introTextNode;
   sg_Win32Text* _demNameText;
   sg_ShapeNode* _demNameNode;

   sg_DEMVertexShader* _curShader;
   sg_HSVShader* _hsvShader;
   sg_DaylightShader* _daylightShader;
   sg_SlopeShader* _slopeShader;
   sg_RangeShader* _rangeShader;
   sg_ContourShader* _contourShader;
   sg_HeadingShader* _headingShader;
   sg_GradientShader* _gradientShader;
   sg_LakeShader* _lakeShader;

   // Shader configuration dialogs.
   DaylightConfigDlg* _daylightDlg;
   HSVConfigDlg* _hsvDlg;
   SlopeConfigDlg* _slopeDlg;
   TerraceConfigDlg* _terraceDlg;
   GradientDlg* _gradientDlg;

   DEMSurfacePropDlg* _demSurfDlg;
   WaterDlg* _waterDlg;

   // Scene configuration dialogs
   CameraDlg* _cameraDlg;
   LightDlg* _lightDlg;
   BackgroundConfigDlg* _backgroundDlg;

   // XXX I can't _believe_ I can't just query the current x,y
   // XXX of the mouse. The best I could come up with is to
   // XXX have OnMouseMove() store the coordinates here so I
   // XXX can get to them from on OnIdle(). 
   // XXX If you know how to just ask for the current position
   // XXX relative to the view window, please get rid of this! -- jamie
   long _mouseX, _mouseY;

   long render();
   void preRenderHiRes();
   void postRenderHiRes();
   void preRenderLoRes();
 	void postRenderLoRes();

   enum ControllerType {Camera, MagicCamera, PoleCamera, Light, Water};
   void setController(ControllerType cntrlr);


// Generated message map functions
protected:

	//{{AFX_MSG(CDem3dView)
	afx_msg void OnDemViewHeaderInfo();
	afx_msg void OnShadersSlope();
	afx_msg void OnShadersTerrace();
	afx_msg void OnToolsMoveCamera();
	afx_msg void OnToolsMovelLights();
	afx_msg void OnShadersDaylight();
	afx_msg void OnShadersHeading();
	afx_msg void OnShadersHSV();
	afx_msg void OnShadersInterval();
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnSceneGrid();
	afx_msg void OnUpdateSceneGrid(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolsMoveCamera(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolsMoveLights(CCmdUI* pCmdUI);
	afx_msg void OnUpdateShadersDaylight(CCmdUI* pCmdUI);
	afx_msg void OnUpdateShadersHeading(CCmdUI* pCmdUI);
	afx_msg void OnUpdateShadersHSV(CCmdUI* pCmdUI);
	afx_msg void OnUpdateShadersInterval(CCmdUI* pCmdUI);
	afx_msg void OnUpdateShadersSlope(CCmdUI* pCmdUI);
	afx_msg void OnUpdateShadersTerrace(CCmdUI* pCmdUI);
	afx_msg void OnSceneCaption();
	afx_msg void OnUpdateSceneCaption(CCmdUI* pCmdUI);
	afx_msg void OnSceneCompass();
	afx_msg void OnUpdateSceneCompass(CCmdUI* pCmdUI);
	afx_msg void OnDemResolution();
	afx_msg void OnDemVerticalExageration();
	afx_msg void OnUpdateDemResolution(CCmdUI* pCmdUI);
	afx_msg void OnUpdateDemVerticalExag(CCmdUI* pCmdUI);
	afx_msg void OnUpdateDemViewHeaderInfo(CCmdUI* pCmdUI);
	afx_msg void OnShadersOptionsTerrace();
	afx_msg void OnUpdateShadersOptionsTerrace(CCmdUI* pCmdUI);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnShadersOptionsDaylight();
	afx_msg void OnUpdateShadersOptionsDaylight(CCmdUI* pCmdUI);
	afx_msg void OnShadersOptionsSlope();
	afx_msg void OnUpdateShadersOptionsSlope(CCmdUI* pCmdUI);
	afx_msg void OnSceneOptionsGrid();
	afx_msg void OnUpdateSceneOptionsGrid(CCmdUI* pCmdUI);
	afx_msg void OnSceneBackGroundColor();
	afx_msg void OnUpdateSceneBackgroundColor(CCmdUI* pCmdUI);
	afx_msg void OnShadersGradient();
	afx_msg void OnUpdateShadersGradient(CCmdUI* pCmdUI);
	afx_msg void OnShadersOptionsGradient();
	afx_msg void OnUpdateShadersOptionsGradient(CCmdUI* pCmdUI);
	afx_msg void OnShadersLake();
	afx_msg void OnUpdateShadersLake(CCmdUI* pCmdUI);
	afx_msg void OnShadersOptionsHSV();
	afx_msg void OnUpdateShadersOptionsHSV(CCmdUI* pCmdUI);
	afx_msg void OnDemSurfProp();
	afx_msg void OnUpdateDemSurfProp(CCmdUI* pCmdUI);
	afx_msg void OnSceneLight();
	afx_msg void OnUpdateSceneLight(CCmdUI* pCmdUI);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnSceneCamera();
	afx_msg void OnToolsWaterLevel();
	afx_msg void OnUpdateToolsWaterLevel(CCmdUI* pCmdUI);
	afx_msg void OnSceneWaterLevel();
	afx_msg void OnUpdateSceneWaterLevel(CCmdUI* pCmdUI);
	afx_msg void OnSceneOptionsWater();
	afx_msg void OnUpdateSceneOptionsWater(CCmdUI* pCmdUI);
	afx_msg void OnToolsMagicTest();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	long           _startTime;
	FrameCounter*  _frameCounter;
	HeaderInfoDlg* _headerInfoDlg;
   DemResDlg*     _demResDlg;
   VertExagDlg*   _vertExagDlg;
   Dem3dStatus*   _statBar;
   long           _totalHiResRenderSteps;
   bool           _drawTransWater;
};

#ifndef _DEBUG  // debug version in dem3dView.cpp
inline CDem3dDoc* CDem3dView::GetDocument()
   { return (CDem3dDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DEM3DVIEW_H__E0AB4213_ABDE_11D0_80AB_02608CDA2E56__INCLUDED_)
