
#ifndef INCLUDED_SG_COMPASS_HXX
#include "sg_Compass.hxx"
#endif

#ifndef INCLUDED_SG_SCENENODE_HXX
#include "scenegraph/sg_SceneNode.hxx"
#endif

#ifndef INCLUDED_SG_SCENEITERATOR_HXX
#include "scenegraph/sg_SceneIterator.hxx"
#endif

#ifndef INCLUDED_SG_SCENEVISITOR_HXX
#include "scenegraph/sg_SceneVisitor.hxx"
#endif

#include "sg_Win32Font.hxx"
#include "sg_Win32Text.hxx"

sg_Compass::sg_Compass(sg_Win32Font const* font,
                       long radius,
                       long numTicks,
                       long tickWidth,
                       long tickHeight)
{
   // Create a mini scenegraph modelling the compass.

   _compassRoot = new sg_CompositeNode();

   double tickincr = 360.0 / numTicks;
   double theta = 0;
   sg_Point corner1(-tickWidth / 2.0, -tickWidth / 2.0, -tickHeight / 2.0);
   sg_Point corner2(tickWidth / 2.0, tickWidth / 2.0, tickHeight / 2.0);
   long i;
   for (i = 0; i < numTicks; i++)
      {
         sg_ShapeNode* tick = new sg_ShapeNode(new sg_Box(corner1, corner2));
         _compassRoot -> addNode(tick);
         tick->rotate(theta, 0, 0, 1);
         tick->translate(radius, 0, 0);
         theta = theta + tickincr;
      }

   // Add the labels;
   for (i = 0; i < 8; i++)
      {
         sg_Win32Text* label = new sg_Win32Text(font,"Testing...");
         switch(i)
            {
               case(0): label->setText("E");  break;
               case(1): label->setText("NE"); break;
               case(2): label->setText("N"); break;
               case(3): label->setText("NW"); break;
               case(4): label->setText("W");  break;
               case(5): label->setText("SW"); break;
               case(6): label->setText("S");  break;
               case(7): label->setText("SE"); break;
            }

         sg_ShapeNode* labelNode = new sg_ShapeNode(label);
         _compassRoot->addNode(labelNode);
         sg_MinBoundBox mbb;
         label->getBoundingBox(mbb);
         labelNode->rotate(90, 1, 0, 0);
         labelNode->rotate(-90, 0, 1, 0);
         labelNode->rotate(i * 45.0, 0, 1, 0);
         labelNode->translate(0, 0, -1.0 * radius);
         labelNode->scale(tickHeight, tickHeight, tickHeight);
         labelNode->translate(-1.0 * mbb.max_x / 2.0, 0, 0);
         labelNode->translate(0, 1.2, 0);
      }
}

sg_Compass::~sg_Compass()
{
}

long
sg_Compass::render(long previousStep)
{
   // XXX This is just a hack until COpenGLView::render() ignores the
   // XXX the children of an inactive sg_CompositeNode.

   sg_RenderVisitor rv;
   sg_RenderIter itr(_compassRoot);

   itr.first();
   while(!itr.isDone())
      {
         itr.currentItem()->accept(rv);
         while (!rv.finishedWithNode())
            {
               itr.currentItem()->accept(rv);
            }
         itr.next();
      }

   return 0;
}

