#ifndef INCLUDED_HF_DEM_HXX
#define INCLUDED_HF_DEM_HXX

#ifndef INCLUDED_DEMTYPES_HXX
#include "dem/DEMTypes.hxx"
#endif

#ifndef INCLUDED_DEMPOINT_HXX
#include "dem/DEMPoint.hxx"
#endif

#include <string>

#ifdef _WIN32
   using namespace std;
#else
#endif


class hf_DEM
{
  // This is the abstract base class for the elevation datatypes to use

	public:
		enum WorldUnits { FEET, METERS };
    enum HEIGHTFIELDTYPE { USGS_DEM, SDTS_DEM, UNKNOWN_HEIGHTFIELD };

		hf_DEM();
		virtual ~hf_DEM()=0;

		long getWidth() const;							// In pixels (NOT world coords)
		long getHeight() const;							// In pixels (NOT world coords)
		long getMaxElev() const;
		long getMinElev() const;
		double getXRes() const;							// Pixel width
		double getYRes() const;							// Pixel height
		double getZRes() const;
		double getMissingData() const;

    // XXX These return SDTS specific information, I couldn't get dynamic 
    // casts to work to stick them in hf_sdtsDEM
    string const& getPositionalComment() const;
    string const& getLineageComment() const;
    string const& getLogicalComment() const;

    string const& getMappingCenterCode() const;
    long getLevelCode() const;
    long getElevPattern() const;
    long getGroundRefSysCode() const;
    long getGroundRefSysZone() const;
    long getGroundRefSysUnits() const;
    DEMPointVector const& getDEMCorners() const;
    long getProfileColumns() const;      

    HEIGHTFIELDTYPE getHeightfieldType() const;
		WorldUnits getWorldUnits() const;
		
		//  Return a elevation for a given x,y coord in the grid
		//      pure virtual function because some datatypes may
		//      have missing data values (USGS dems) while others
		//      don't, and this must be checked for.
		virtual
		double getElev(double x, double y) const =0;	

		// Pure Virtual Function used to fill the grid with data
		virtual						
		long fill(const string& filename)=0;				// pass in file prefix

		virtual
		void getWorldCoords(long x, long y,
                        double& worldX, double& worldY)=0;

		virtual
		string getQuadName() const =0;

    virtual
    double* getBoundingRect() const =0;

	protected:
		long _minElev;
		long _maxElev;
		long _height;
		long _width;
		double _xRes;
		double _yRes;
		double _zRes;
    string _mapCenterCode;
    long _levelCode;
    long _elevPattern;
    long _groundRefSysCode;
    long _groundRefSysZone;
    long _groundRefSysUnits;
    DEMPointVector _points;
    long _numProfiles;
		double _missingData;
		string _quadName;
		WorldUnits _units;
		bool _firstTime;
    HEIGHTFIELDTYPE _type;
    string _lineageComment;
    string _positionalComment;
    string _logicalComment;

};
#endif INCLUDED_HF_DEM_HXX