// NOTE:  This class is compliant ONLY for the USGS/NMD DEMs in SDTS
//		  using parts 2 & 3 of the Mapping of the USGS/NMD DEM into
//		  the Raster Profile of the Spatial Data Transfer Standard.
//
//									-- paul



#ifndef INCLUDED_HF_SDTSDEM_HXX
#include "hf_sdtsDEM.hxx"
#endif

sio_Converter_BI32 bi32_converter;
sio_Converter_BI16 bi16_converter;

hf_sdtsDEM::hf_sdtsDEM()
: _grid(0), _curHeight(0), _gridLocation(0)
{
   _type = SDTS_DEM;
}

hf_sdtsDEM::~hf_sdtsDEM()
{
	delete _grid;
}

void
hf_sdtsDEM::loadIDEN()
{
	sb_Ident* _iden;

	string _nameIDEN = _modules["IDEN"];
	ifstream ddf(_nameIDEN.c_str(), ios::binary);
	sio_8211Reader reader(ddf, &_converters);
   sio_8211ForwardIterator i( reader );
   i.get( _record );
	_iden = new sb_Ident( _record );
   
}

void
hf_sdtsDEM::loadDQHL()
{
	sb_Lineage* _dqhl;

	string _nameDQHL = _modules["DQHL"];
	ifstream ddf(_nameDQHL.c_str(), ios::binary);
	sio_8211Reader reader(ddf, &_converters);
   sio_8211ForwardIterator i( reader );
   i.get( _record );
	_dqhl = new sb_Lineage( _record );
   _lineageComment = _dqhl->getComment();
}

void
hf_sdtsDEM::loadDQPA()
{
	sb_PosAccuracy* _dqpa;

	string _nameDQPA = _modules["DQPA"];
	ifstream ddf(_nameDQPA.c_str(), ios::binary);
	sio_8211Reader reader(ddf, &_converters);
   sio_8211ForwardIterator i( reader );
   i.get( _record );
	_dqpa = new sb_PosAccuracy( _record );
   _positionalComment = _dqpa->getComment();
}
void
hf_sdtsDEM::loadDQLC()
{
	sb_LogicalConsis* _dqlc;

	string _nameDQLC = _modules["DQLC"];
	ifstream ddf(_nameDQLC.c_str(), ios::binary);
	sio_8211Reader reader(ddf, &_converters);
   sio_8211ForwardIterator i( reader );
   i.get( _record );
	_dqlc = new sb_LogicalConsis( _record );
   _logicalComment = _dqlc->getComment();
}

void
hf_sdtsDEM::loadCATD()
{
	sb_CatalogDir* _catd;
	
	// get module names
	string _nameCATD = _filePrefix + "CATD.DDF";
	ifstream ddf(_nameCATD.c_str(), ios::binary);
	sio_8211Reader reader(ddf, &_converters);
    for( sio_8211ForwardIterator i( reader );
        ! i.done();
        ++i )
	{
		i.get( _record );
		_catd = new sb_CatalogDir( _record );
		_modules[_catd->getName()] = _filePath + _catd->getFile();
		delete _catd;
	}
}

void
hf_sdtsDEM::loadLDEF()
{
	// Get important layer info
	string _nameLDEF = _modules["LDEF"];
	ifstream ddf(_nameLDEF.c_str(), ios::binary);
	sio_8211Reader reader(ddf, &_converters);
	for ( sio_8211ForwardIterator i( reader ); ! i.done(); ++i )
	{
		i.get ( _record );
		for ( _field_itr = _record.begin();
			_field_itr != _record.end();
			_field_itr++ )
		{
			for ( _subfield_itr = _field_itr->begin();
					_subfield_itr != _field_itr->end();
					_subfield_itr++ )
			{
					if (_subfield_itr->getMnemonic() == "CMNM") 
					{
						_subfield_itr->getA( _cellModuleName );
					}
					else if (_subfield_itr->getMnemonic() == "LLBL")
					{
						_subfield_itr->getA( _label );
					}
					else if (_subfield_itr->getMnemonic() == "NROW")
					{
						_subfield_itr->getI( _height );
					}
					else if (_subfield_itr->getMnemonic() == "NCOL")
					{
						_subfield_itr->getI( _width );
					}
			}
		}
	}
}

void
hf_sdtsDEM::readNextRecord()
{
   _curRecord->get ( _record );
	for ( _field_itr = _record.begin();
		_field_itr != _record.end();
		_field_itr++ )
	{
		for ( _subfield_itr = _field_itr->begin();
				_subfield_itr != _field_itr->end();
				_subfield_itr++ )
		{
			if (_subfield_itr->getMnemonic() == _label)
			{
				_subfield_itr->getBI16(_grid[_gridLocation]);
				_gridLocation++;
			}
		}
	}
	++(*_curRecord);
	_curHeight++;
}

long
hf_sdtsDEM::loadCELL(bool incrementalRead)
{
   if (_firstTime)
	{
   	string _nameCELL = _modules["CEL0"];
      _ddf.open(_nameCELL.c_str(),ios::binary);
	   _reader = new sio_8211Reader(_ddf, &_converters);
		_curRecord = new sio_8211ForwardIterator( *_reader );
		_firstTime = false;
	}

	while (! _curRecord->done() )
	{
      readNextRecord();
      if (incrementalRead)
	      return _height - _curHeight;
	}
   _ddf.close();
	return 0;
}

void
hf_sdtsDEM::loadDDOM()
{
	sb_DataDictDom* _ddom;

	string _nameDDOM = _modules["DDOM"];
	ifstream ddf(_nameDDOM.c_str(), ios::binary);
	sio_8211Reader reader(ddf, &_converters);
    for( sio_8211ForwardIterator i( reader );
        ! i.done();
        ++i )
	{
		i.get( _record );
		_ddom = new sb_DataDictDom( _record );
		if (_ddom->getRangeOrValue() == "MIN")
		{
			_ddom->getDomainValue().getI(_minElev);
		}
		else if (_ddom->getRangeOrValue() == "MAX")
		{
			_ddom->getDomainValue().getI(_maxElev);
		}
		delete _ddom;
	}
}

void
hf_sdtsDEM::loadIREF()
{
	sb_IntSpatialRef* _iref;

	// Get x and y resolution
	string _nameIREF = _modules["IREF"];
	ifstream ddf(_nameIREF.c_str(), ios::binary);
	sio_8211Reader reader(ddf, &_converters);
    for( sio_8211ForwardIterator i( reader );
        ! i.done();
        ++i )
	{
		i.get( _record );
		_iref = new sb_IntSpatialRef( _record );
		_xRes = _iref->getXCompHorizRes();
		_yRes = _iref->getYCompHorizRes();
		delete _iref;
	}
}

void
hf_sdtsDEM::loadDDSH()
{
	sb_DataDictSchema* _ddsh;

	// get data format and precision, precision is _zRes
	string _nameDDSH = _modules["DDSH"];
	ifstream ddf(_nameDDSH.c_str(), ios::binary);
	sio_8211Reader reader(ddf, &_converters);
    for( sio_8211ForwardIterator i( reader );
        ! i.done();
        ++i )
	{
		i.get( _record );
		_ddsh = new sb_DataDictSchema( _record );
		if (_ddsh->getName() == _cellModuleName)
		{
			_format = _ddsh->getFormat();
			if (_ddsh->getUnit() == "1")
			{
            _units = hf_DEM::FEET;
			}
			else if (_ddsh->getUnit() == "2")
			{
            _units = hf_DEM::METERS;
			}
			// should be a GetPrec here too
		}
	}
}


long
hf_sdtsDEM::fill(const string& filename)
{
	if (_firstTime)
	{
		_filePrefix = filename.substr(0,filename.length() - string("CATD.DDF").length());
		_filePath = _filePrefix.substr(0,
			(_filePrefix.length() - ((filename.length() - filename.find_last_of("\\")) - string("CATD.DDF").length()))+1);

		_converters["X"] = &bi32_converter;
		_converters["Y"] = &bi32_converter;
		_converters["ELEVATION"] = &bi16_converter;		// This should be changed...

		loadCATD();
		loadLDEF();
		loadDDOM();
		loadIREF();
		loadDDSH();
      loadDQLC();
      loadDQPA();
      loadDQHL();

		_grid = new long[_width * _height];
// XXX need to change this:
		_missingData = -32766;
	}	

	return loadCELL(true);
}
