#ifndef INCLUDED_HF_SDTSDEM_HXX
#define INCLUDED_HF_SDTSDEM_HXX

#ifndef INCLUDED_HF_DEM_HXX
#include "heightfield/hf_DEM.hxx"
#endif

#include <iostream>
#include <fstream>


#ifdef WIN32
   using namespace std;
#endif

#include "io/sio_Reader.h"
#include "io/sio_8211DirEntry.h"
#include "io/sio_Converter.h"

#include "container/sc_Subfield.h"
#include "container/sc_Field.h"
#include "container/sc_Record.h"

#include "builder/sb_CatalogDir.hxx"
#include "builder/sb_RasterDefn.hxx"
#include "builder/sb_DataDictDom.hxx"
#include "builder/sb_IntSpatialRef.hxx"
#include "builder/sb_DataDictSchema.hxx"
#include "builder/sb_Ident.hxx"
#include "builder/sb_LogicalConsis.hxx"
#include "builder/sb_PosAccuracy.hxx"
#include "builder/sb_Lineage.hxx"

class hf_sdtsDEM : public hf_DEM
{
public:
	hf_sdtsDEM();
	~hf_sdtsDEM();

	long 
	fill(const string& filename);				// pass in file prefix

	double
	getElev(double x,double y) const;

	void
	getWorldCoords(long, long, double&, double&);

	string
	getQuadName() const;

   double*                                // fill in later
   getBoundingRect() const;

private:

	string _filePrefix;
	string _filePath;
	map<string, sio_Converter*> _converters;
	map<string, string> _modules;

	string _format;
	long* _grid;
	long _gridLocation;
	sio_8211ForwardIterator* _curRecord;
   sio_8211Reader* _reader;
   ifstream _ddf;
	long _curHeight;
	string _label;
	string _cellModuleName;
   sc_Record _record;
   sc_FieldCntr::const_iterator _field_itr;
   sc_SubfieldCntr::const_iterator _subfield_itr;

	void loadCATD();
	void loadLDEF();
	void loadDDOM();
	void loadIREF();
	void loadDDSH();
   void loadIDEN();
   void loadDQHL();
   void loadDQLC();
   void loadDQPA();
	long loadCELL(bool incrementalRead);
   void readNextRecord();
};

inline
double
hf_sdtsDEM::getElev(double x, double y) const
{
   if (_grid == 0)
      return _missingData;

   if ((x < 0) || (y < 0) || (x >= _width) || (y >= _height))
      return _missingData;

   return _grid[(long) ((y * _width) + x) ];
}

inline
double*
hf_sdtsDEM::getBoundingRect() const
{
   return 0;
}

inline
void
hf_sdtsDEM::getWorldCoords(long, long, double&, double&) 
{
}

inline
string
hf_sdtsDEM::getQuadName() const
{
	return _quadName;
}

#endif INCLUDED_HF_SDTSDEM_HXX