#ifndef INCLUDED_HF_USGSDEM_HXX
#include "heightfield/hf_usgsDEM.hxx"
#endif

#ifndef INCLUDED_DEMHEADER_HXX
#include "dem/DEMHeader.hxx"
#endif

#ifndef INCLUDED_DEMGRID_HXX
#include "dem/DEMGrid.hxx"
#endif

hf_usgsDEM::hf_usgsDEM()
: _demGrid(0)
{
   _type = USGS_DEM;
}

hf_usgsDEM::~hf_usgsDEM()
{
	delete _demGrid;
   delete _boundingRect;
}

long
hf_usgsDEM::fill(const string& filename)
{
   int i,j;
   if (_firstTime)
      {
         _firstTime = false;
         _demfile.open(filename.c_str(),ios::binary);
         _demGrid = new DEMGrid();
         return _demGrid->read(_demfile, true);
      }

	long retval;
   retval = _demGrid->read(_demfile,true);
   if (retval == 0)
	   {
         _demfile.close();
		   _demHeader = _demGrid->getHeader();

		   _maxElev = _demHeader.getMaximumElev();
		   _minElev = _demHeader.getMinimumElev();
		   _xRes = _demHeader.getSpatialResX();
		   _yRes = _demHeader.getSpatialResY();
		   if (_demHeader.getElevationUnits() == 1)
   			_units = hf_DEM::FEET;
		   if (_demHeader.getElevationUnits() == 2)
   			_units = hf_DEM::METERS;
   
	   	_quadName = _demHeader.getQuadName();

         _mapCenterCode = _demHeader.getMappingCenterCode();
         _levelCode = _demHeader.getLevelCode();
         _elevPattern = _demHeader.getElevPattern();
         _groundRefSysCode = _demHeader.getGroundRefSysCode();
         _groundRefSysZone = _demHeader.getGroundRefSysZone();
         _groundRefSysUnits = _demHeader.getGroundRefSysUnits();
         _zRes = _demHeader.getSpatialResZ();
         _numProfiles = _demHeader.getProfileColumns();
         _points = _demHeader.getDEMCorners();

		   _missingData = _demGrid->getMissingDataValue();
		   _width = _demGrid->getWidth();
   		_height = _demGrid->getHeight();

         _boundingRect = new double[8];
         for (i = 0, j=0; i < 4; i++, j+=2)
            {
               _boundingRect[j] = _points[i].getX();
               _boundingRect[j+1] = _points[i].getY();
            }
     }
	return retval;
}

DEMHeader
hf_usgsDEM::getHeader() const 
{
   return _demHeader;
}