// $ID$

#pragma warning(once:4244)

#ifdef WIN32
#include <afxwin.h>
#include <afxext.h>
#include <afxtempl.h>
#endif

#include <gl/gl.h>

#ifndef INCLUDED_SG_TYPES_HXX
#include "scenegraph/sg_Types.hxx"
#endif

#ifndef INCLUDED_SG_APPEARANCE
#include "scenegraph/sg_Appearance.hxx"
#endif

sg_Appearance::sg_Appearance()
             : _color(1.0, 1.0, 1.0, 1.0),  // White
               _colorShadow(false),
               _style(FLAT),
               _styleShadow(false),
               _lighting(true),
               _lightingShadow(false),
               _polyMode(FILL),
               _polyModeShadow(false),
               _shininess(0.0),
               _specularColor(0.0, 0.0, 0.0, 1.0)
{
}

sg_Appearance::~sg_Appearance()
{
}

void
sg_Appearance::setColor(sg_RGBA const& rgba)
{
   _color = rgba;
   _colorShadow = true;
}

void
sg_Appearance::setColor(double r, double g, double b, double a)
{
   setColor(sg_RGBA(r, g, b, a));
}

void
sg_Appearance::setShadingStyle(sg_Appearance::ShadingStyle style)
{
   _style = style;
   _styleShadow = true;
}

sg_Appearance::ShadingStyle
sg_Appearance::getShadingStyle() const
{
   return _style;
}

void
sg_Appearance::setLighting(bool flag)
{
   _lighting = flag;
   _lightingShadow = true;
}

bool
sg_Appearance::isLit() const
{
   return _lighting;
}

void
sg_Appearance::setPolygonMode(PolygonMode mode)
{
   _polyMode = mode;
   _polyModeShadow = true;
}

sg_Appearance::PolygonMode 
sg_Appearance::getPolygonMode() const
{
   return _polyMode;
}

void
sg_Appearance::setShininess(double shininess)
{
   if (shininess < 0.0)
      shininess = 0.0;
   else
      if (shininess > 128.0)
         shininess = 128.0;

   _shininess = shininess;
}

double
sg_Appearance::getShininess() const
{
   return _shininess;
}

void
sg_Appearance::setSpecularColor(sg_RGBA const& specColor)
{
   _specularColor = specColor;
}

sg_RGBA const&
sg_Appearance::getSpecularColor() const
{
   return _specularColor;
}


void
sg_Appearance::apply()
{
   if (_colorShadow)
      glColor4f(_color.red, _color.green, _color.blue, _color.alpha);
      
   if (_styleShadow)
      {
         switch(_style)
            {
               case(FLAT):   glShadeModel(GL_FLAT);     break;
               case(SMOOTH): glShadeModel(GL_SMOOTH);   break;
            }
      }      

   if (_lightingShadow)
      {
         if (_lighting)
            glEnable(GL_LIGHTING);
         else
            glDisable(GL_LIGHTING);
      }

   if (_polyModeShadow)
      {
         if (_polyMode == LINE)
            glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
         else
            glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
      }

   // XXX No shadow variables for these. I'm probably going to do away
   // with shadow variables all together. -- jamie 06/17/97

   glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, _shininess);

   GLfloat specColor[4] = {_specularColor.red,
                           _specularColor.green,
                           _specularColor.blue,
                           _specularColor.alpha};
   glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specColor);
}

