
#include <math.h>

#ifndef INCLUDED_SG_CONTROLLER_HXX
#include "scenegraph/sg_Controller.hxx"
#endif

#ifndef INCLUDED_SG_SCENENODE_HXX
#include "scenegraph/sg_SceneNode.hxx"
#endif

#ifndef INCLUDED_SG_UTILS_HXX
#include "scenegraph/sg_Utils.hxx"
#endif

#ifndef INCLUDED_SG_SHAPE_HXX
#include "scenegraph/sg_Shape.hxx"
#endif

#ifndef INCLUDED_SG_DEM_HXX
#include "scenegraph/sg_DEM.hxx"
#endif

// $Id: sg_Controller.cxx,v 1.4 1997/02/07 00:58:53 moyers Exp $

#ifdef WIN32
#define M_PI 3.14159265358979323846
#else
#include <iostream.h>
#endif

static inline
double
normalizeAngle(double theta, double minTheta)
{
   // Returns an angle equivelant to theta, but in the 
   // range [minTheta, minTheta + 360.0)
   theta = theta - minTheta;
   double sign = (theta >= 0.0 ? 1.0 : -1.0);
   double t = fabs(theta);
   double result = t - (floor(t / 360.0) * 360.0);
   if (sign == -1.0)
      result = 360.0 - result;
   return result + minTheta;
}

sg_Controller::sg_Controller()
{
}

void
sg_Controller::sync()
{
}

void
sg_Controller::finish()
{
}

void
sg_Controller::activate()
{
}

void
sg_Controller::deactivate()
{
}

void
sg_Controller::mouseMove(long x, long y, long status)
{
}

void
sg_Controller::button1Down(long x, long y, long status)
{
}

void
sg_Controller::button1Up(long x, long y, long status)
{
}

void
sg_Controller::button2Down(long x, long y, long status)
{
}

void
sg_Controller::button2Up(long x, long y, long status)
{
}

void
sg_Controller::button3Down(long x, long y, long status)
{
}

void
sg_Controller::button3Up(long x, long y, long status)
{
}

void
sg_Controller::keyPressed(char charRep, long status)
{
}

void
sg_Controller::keyReleased(char charRep, long status)
{
}


// sg_ControlLineController

sg_ControlLineController::sg_ControlLineController(sg_CameraNode const* camera)
                              : _camera(camera),
                                _node(0),
                                _origin(0.0, 0.0, 0.0),
                                _minRadius(0.0),
                                _maxRadius(100.0),
                                _curRadius(100.0),
                                _curAzimuth(180.0),
                                _curAltitude(0.0),
                                _first_time(true),
                                _old_x(0), _old_y(0)
{
}

void
sg_ControlLineController::setNode(sg_SceneNode* node)
{
   _node = node;
}

void
sg_ControlLineController::setOrigin(double x, double y, double z)
{
   _origin.x = x;
   _origin.y = y;
   _origin.z = z;
}

void
sg_ControlLineController::setMinRadius(double minRadius)
{
   _minRadius = minRadius;
   if (_curRadius < _minRadius)
      _curRadius = _minRadius;
}

void
sg_ControlLineController::setMaxRadius(double maxRadius)
{
   _maxRadius = maxRadius;
   if (_curRadius > _maxRadius)
      _curRadius = _maxRadius;
}

void
sg_ControlLineController::setCurrentRadius(double curRadius)
{
   _curRadius = curRadius;
   if (_curRadius < _minRadius)
      _curRadius = _minRadius;
   if (_curRadius > _maxRadius)
      _curRadius = _maxRadius;

   positionNode();
}

void
sg_ControlLineController::setCurrentAzimuth(double azimuth)
{
   _curAzimuth = normalizeAngle(azimuth,0);
   positionNode();
}

void
sg_ControlLineController::setCurrentAltitude(double altitude)
{
   _curAltitude = normalizeAngle(altitude,-180.0);
   if (_curAltitude < -90.0)
      _curAltitude = -90.0;
   if (_curAltitude > 90.0)
      _curAltitude = 90.0;
   positionNode();
}

double
sg_ControlLineController::getCurrentAzimuth() const
{
   return _curAzimuth;
}

void
sg_ControlLineController::sync()
{
   _first_time = true;

   if (_node == 0)
      return;

   // Determine the node's current position in terms of radius, altitude and azimuth.
   sg_Point orig(0,0,0);
   sg_Point pos;
   sg_MatrixUtils::transformPoint(_node->getTransformation().getMatrix(),orig,pos);

   double dist;
   double alti;
   double azim;
   sg_Utils::calcSpherical(pos, dist, alti, azim);

   if (dist < _minRadius)
      {
         dist = _minRadius;
         setCurrentRadius(dist);
         setCurrentAltitude(0.0);
         setCurrentAzimuth(0.0);
         positionNode();
         return;
      }

   setCurrentAzimuth(azim);
   setCurrentAltitude(alti);
   setCurrentRadius(dist);

   // positionNode();
}

void
sg_ControlLineController::mouseMove(long x, long y, long status)
{
   if (_camera == 0)
      return;

   if (_first_time)
      {
         _old_x = x;
         _old_y = y;
         _first_time = false;
         return;
      }

   if (!((status & button1) | (status & button2)))
      return;

   long ll_x, ll_y;
   long width, height;
   _camera->getViewPort(ll_x, ll_y, width, height);
   long dx = x - _old_x;
   long dy = y - _old_y;
   _old_x = x;
   _old_y = y;

   if (status & button1)
      {
         // Move camera up,down,left,right.
         double d_azimuth = (180.0 * dx) / width;
         double d_altitude = (180.0 * dy) / height;

         double t_azim;
         double t_alt;
         if (static_cast<sg_SceneNode const*>(_camera) == _node)
            {
               // This is a special case. We reverse the sense of the mouse
               // if we're "looking out of" the object being moved.
               t_azim = _curAzimuth - d_azimuth;
               t_alt = _curAltitude + d_altitude;
            }
         else
            {
               t_azim = _curAzimuth + d_azimuth;
               t_alt = _curAltitude - d_altitude;
            }

         setCurrentAzimuth(t_azim);
         setCurrentAltitude(t_alt);

         // positionNode();
      }

   if (status & button2)
      {
         // Move camera in and out.
         double sensitivity = 0.5;   // Between 0.0 and 1.0
         double minDist = (_maxRadius - _minRadius) * sensitivity;
         double dist = _curRadius - _minRadius;
         if (dist < minDist)
            dist = minDist;

         double d_radius = (dist / height) * dy;
         setCurrentRadius(_curRadius + d_radius);

         // positionNode();
      }
}

void
sg_ControlLineController::button1Up(long x, long y, long status)
{
   _first_time = true;
}

void
sg_ControlLineController::button2Up(long x, long y, long status)
{
   _first_time = true;
}

void
sg_ControlLineController::positionNode()
{
   if (_node == 0)
      return;

   _node->identity();
   _node->translate(_origin.x, _origin.y, _origin.z);
//   _node->rotate(_azimuth - 180.0, 0,0,1);
   _node->rotate(_curAzimuth + 90.0, 0,0,1);
   _node->rotate(-1.0 * _curAltitude, 1,0,0);
   _node->translate(0.0, -1.0 * _curRadius, 0.0);
}

void 
sg_ControlLineController::setCamera(sg_CameraNode const * camera)
{
	_camera = camera;
}

double sg_ControlLineController::getCurrentRadius()
{
	return _curRadius;
}

double sg_ControlLineController::getCurrentAltitude()
{
	return _curAltitude;
}

void
sg_ControlLineController::setGeoAzimuth(double geoAzim)
{
   normalizeAngle(geoAzim, 0.0);
   geoAzim = 450.0 - geoAzim;
   if (geoAzim > 360.0)
      geoAzim = geoAzim - 360.0;
   setCurrentAzimuth(geoAzim);
}

double
sg_ControlLineController::getGeoAzimuth() const
{
   double geoAzim = 450.0 - _curAzimuth;
   if (geoAzim > 360.0)
      geoAzim = geoAzim - 360.0;
   return geoAzim;
}

// sg_AltAzLightController

sg_AltAzLightController::sg_AltAzLightController(sg_CameraNode const* camera)
                              : _camera(camera),
                                _light(0),
                                _origin(0.0, 0.0, 0.0),
                                _radius(1.0),
                                _curAzimuth(180.0),
                                _curAltitude(0.0),
                                _first_time(true),
                                _old_x(0), _old_y(0),
                                _lightCompParent(0)
{
   _ourComposite = new sg_CompositeNode();
   _altAzInd = new sg_AltAzIndicator(_radius);
   _altAzInd->setAltColor(sg_RGBA(0.0, 1.0, 1.0));
   _altAzInd->setAzimColor(sg_RGBA(0.0, 1.0, 0.0));
   _altAzIndNode = new sg_ShapeNode(_altAzInd);
   _altAzIndNode->setLighting(false);
   _ourComposite->addNode(_altAzIndNode);
}

void
sg_AltAzLightController::setLight(sg_LightNode* light)
{
   _light = light;
}

void
sg_AltAzLightController::setOrigin(double x, double y, double z)
{
   _origin.x = x;
   _origin.y = y;
   _origin.z = z;
}

void
sg_AltAzLightController::setRadius(double radius)
{
   _radius = radius;
   _altAzInd->setRadius(_radius);
   positionNode();
}

void
sg_AltAzLightController::setCurrentAzimuth(double azimuth)
{  
   _curAzimuth = normalizeAngle(azimuth,0);

   _altAzInd->setAzimuth(_curAzimuth);
   positionNode();
}

void
sg_AltAzLightController::setCurrentAltitude(double altitude)
{
   _curAltitude = normalizeAngle(altitude,-180.0);
   if (_curAltitude < -90.0)
      _curAltitude = -90.0;
   if (_curAltitude > 90.0)
      _curAltitude = 90.0;

   _altAzInd->setAltitude(_curAltitude);
   positionNode();
}

void
sg_AltAzLightController::setGeoAzimuth(double geoAzim)
{
   normalizeAngle(geoAzim, 0.0);
   geoAzim = 450.0 - geoAzim;
   if (geoAzim > 360.0)
      geoAzim = geoAzim - 360.0;
   setCurrentAzimuth(geoAzim);
}

double
sg_AltAzLightController::getGeoAzimuth()
{
   double geoAzim = 450.0 - _curAzimuth;
   if (geoAzim > 360.0)
      geoAzim = geoAzim - 360.0;
   return geoAzim;
}

double
sg_AltAzLightController::getCurrentAzimuth() const
{
   return _curAzimuth;
}

void
sg_AltAzLightController::sync()
{
   _first_time = true;

   if (_light == 0)
      return;

   // Determine the light's current position in terms of radius, altitude and azimuth.
   sg_Point orig(0,0,0);
   sg_Point pos;
   sg_MatrixUtils::transformPoint(_light->getTransformation().getMatrix(),orig,pos);

   double dist;
   double alti;
   double azim;
   sg_Utils::calcSpherical(pos, dist, alti, azim);

   setCurrentAzimuth(azim);
   setCurrentAltitude(alti);
   setRadius(_radius);

}

void
sg_AltAzLightController::mouseMove(long x, long y, long status)
{
   if (_camera == 0)
      return;

   if (_first_time)
      {
         _old_x = x;
         _old_y = y;
         _first_time = false;
         return;
      }

   if (!((status & button1) | (status & button2)))
      return;

   long ll_x, ll_y;
   long width, height;
   _camera->getViewPort(ll_x, ll_y, width, height);
   long dx = x - _old_x;
   long dy = y - _old_y;
   _old_x = x;
   _old_y = y;

   if (status & button1)
      {
         // Move camera up,down,left,right.
         double d_azimuth = (180.0 * dx) / width;
         double d_altitude = (180.0 * dy) / height;

         setCurrentAzimuth(_curAzimuth + d_azimuth);
         setCurrentAltitude(_curAltitude - d_altitude);
      }
}

void
sg_AltAzLightController::button1Up(long x, long y, long status)
{
   _first_time = true;
}

void
sg_AltAzLightController::positionNode()
{
   if (_light == 0)
      return;

   _light->identity();
   _light->translate(_origin.x, _origin.y, _origin.z);
   _light->rotate(_curAzimuth + 90.0, 0,0,1);
   _light->rotate(-1.0 * _curAltitude, 1,0,0);
   _light->translate(0.0, -1.0 * _radius, 0.0);
}

void 
sg_AltAzLightController::setCamera(sg_CameraNode const * camera)
{
	_camera = camera;
}

double
sg_AltAzLightController::getCurrentAltitude() const
{
	return _curAltitude;
}

void
sg_AltAzLightController::activate()
{
   if (_light == 0)
      return;

   sg_SceneNode* lightParent = const_cast<sg_SceneNode*>(_light->getParent());
   _lightCompParent = lightParent->getComposite();
   if (_lightCompParent == 0)
   {
      // This should always be a composite!
      return;
   }

   _lightCompParent->replaceNode(_light, _ourComposite);
   _ourComposite->addNode(_light);
}

void
sg_AltAzLightController::deactivate()
{
   _ourComposite->removeNode(_light);
   _lightCompParent->replaceNode(_ourComposite, _light);
}



sg_WaterLevelController::sg_WaterLevelController(sg_CameraNode const* camera,
                         sg_DEM* demShape)
                       : _camera(camera),
                         _demShape(demShape)
{
   if (_demShape == 0)
      return;

   _dElev = _demShape->getMaxElev() - _demShape->getMinElev();
}

void
sg_WaterLevelController::sync()
{
   _first_time = true;
}

void
sg_WaterLevelController::activate()
{
   _prevWaterState = _demShape->getDrawWater();
   _demShape->setDrawWater(true);
}

void
sg_WaterLevelController::deactivate()
{
   _demShape->setDrawWater(_prevWaterState);
}

void
sg_WaterLevelController::mouseMove(long x, long y, long status)
{
   if (_camera == 0)
      return;

   if (_first_time)
      {
         _old_x = x;
         _old_y = y;
         _first_time = false;
         return;
      }

   if (!((status & button1) | (status & button2)))
      return;

   if (status & button1)
      {
         long ll_x, ll_y;
         long width, height;
         _camera->getViewPort(ll_x, ll_y, width, height);

         long dy = y - _old_y;
         _old_y = y;

         double wz = _demShape->getWaterLevel();
         wz = wz + (_dElev * (dy * -1.0 / height));
         _demShape->setWaterLevel(wz);
      }
}

void
sg_WaterLevelController::button1Up(long x, long y, long status)
{
   _first_time = true;
}