#ifndef INCLUDED_SG_DEM_HXX
#define INCLUDED_SG_DEM_HXX

#ifndef INCLUDED_SG_SHAPE_HXX
#include "scenegraph/sg_Shape.hxx"
#endif

class hf_DEM;
class sg_DEMVertexShader;
struct sg_DEMVertexShaderInfo;

class sg_DEM : public sg_Shape
{
   public:

      sg_DEM(hf_DEM const* heightfield);

      hf_DEM const* getHeightField() const;

      long render(long previousStep = 0);

      void getBoundingBox(double& min_x, double& min_y, double& min_z,
                          double& max_x, double& max_y, double& max_z) const;

      void getBoundingBox(sg_MinBoundBox& mbb) const;

      void setShader(sg_DEMVertexShader* shader); 
         // sg_DEM doesn't own the shader.

      void setResolution(double res);
         // 0.0 <= res <= 1.0
         // Default: 1.0

      void setVerticalExageration(double ve);
         // 1.0 = normal
         // Default: 1.0

      double getVerticalExageration() const;

      double getMinElev() const;
      double getMaxElev() const;

      void setGouraud(bool flag);
         // If flag == true, calculates Gouraud surface normals.

      bool isGouraud() const;

      // Methods for accessing the ModelView and Projection Matrices that
      // exist when rendering starts (previousStep == 0). These matrices
      // can be used to "unproject" a location on the display to obtain
      // the object space coordinates for that point.
      // XXX These return garbage until render() has been called at least once.
      void getModelView(GLdouble* modelView);
      void getProjection(GLdouble* projection);

      // XXX This should probably be somewhere else (like sg_Shape)
      // XXX but here is fine for now.
      void setDrawBoundingBox(bool flag);
         // Draws a wireframe bounding box around DEM.
         // Default: false
      void setBoundingBoxColor(sg_RGBA const& bbColor);
      void drawBoundingBox();
      void setDrawDEM(bool flag);
         // If this is false, only the bounding box is drawn.
         // Default: true

      // XXX This, too, should probably really be somewhere else.
      // XXX But this was so _easy_. :)
      void setWaterColor(sg_RGBA const& waterColor);
      sg_RGBA const& getWaterColor() const;

      void drawWater();

      void setDrawWater(bool flag);
      bool getDrawWater() const;

      void setWaterLevel(double waterZ);
      double getWaterLevel() const;

      void setTransparentWater(bool flag);
      bool getTransparentWater() const;

      void setTransAlpha(double transAlpha);
      double getTransAlpha() const;

   private:

	   double _convFactor;
	   long   _curRow;
	   long   _chunkSize;
	   double _missingDataValue;
	   double _dx;
	   double _dy;
	   long   _virtRows;
	   long   _virtColumns;

      hf_DEM const* _heightfield;
      sg_DEMVertexShader* _shader;
      double        _res;
      double        _vertExag;
      bool          _gouraudFlag;

      double _x_min, _y_min;  // Extent of DEM in World coordinates.
      double _x_max, _y_max;
      double _z_min, _z_max;
      double _minElev, _maxElev;   // Min and max elev values from _demGrid.
      double _numRows, _numColumns;  // in demGrid.
      double _width, _height;  // X and Y extent of DEM in World coordinates.

      GLdouble _modelViewMat[16];
      GLdouble _projMat[16];

      bool    _drawDEM;
      bool    _drawBoundingBox;
      sg_RGBA _boundBoxColor;

      sg_RGBA _waterColor;
      bool    _drawWater;
      double  _waterLevel;
      bool    _transparentWater;
      double  _transAlpha;

      void getDEMPoint(long virtRow, long virtCol, sg_Point& p);
         // Returns the world coordinates of the point at the "virtual" 
         // row and column position.

      void setupShaderInfo(long row, long column,  // virtual row and column of 'point'.
                           sg_Point const& point,
                           sg_Point const& q,
                           sg_Point const& r,
                           sg_DEMVertexShaderInfo& aVertexInfo);
         // Initializes the shader structure with adjacent points
         // and a normal.

};

#endif  // INCLUDED_SG_DEM_HXX
