
// $Id: sg_DEMVertexShader.cxx,v 1.5 1997/01/29 19:25:12 pjgray Exp $

#include <math.h>
#ifdef WIN32
#define M_PI_2 1.57079632679489661923
#define M_PI 3.14159265358979323846
#include <afxwin.h>
#include <afxext.h>
#include <afxtempl.h>
#endif


#include "gl/gl.h"

#ifndef INCLUDED_SG_DEMVERTEXSHADER_HXX
#include "scenegraph/sg_DEMVertexShader.hxx"
#endif

#ifndef INCLUDED_SG_COLORS_HXX
#include "scenegraph/sg_Colors.hxx"
#endif

#ifndef INCLUDED_SG_HSV_H
#include "scenegraph/sg_hsv.h"
#endif

sg_DEMVertexShader::sg_DEMVertexShader(hf_DEM const* heightField)
                  : _heightfield(heightField),
                    _x_min(0.0), _x_max(0.0),
                    _y_min(0.0), _y_max(0.0),
                    _z_min(0.0), _z_max(0.0)
{
}

void
sg_DEMVertexShader::setExtent(double x_min, double x_max,
                              double y_min, double y_max,
                              double z_min, double z_max)
{
   _x_min = x_min;
   _x_max = x_max;
   _y_min = y_min;
   _y_max = y_max;
   _z_min = z_min;
   _z_max = z_max;
}

void
sg_DEMVertexShader::init()
{
}

void
sg_DEMVertexShader::cleanup()
{
}

void
sg_DEMVertexShader::shade(sg_DEMVertexShaderInfo& info)
{
}


// sg_DaylightShader

sg_DaylightShader::sg_DaylightShader(hf_DEM const* heightField)
                : sg_DEMVertexShader(heightField),
                  _demColor(0.8, 0.8, 0.8, 1.0)
{
}

void
sg_DaylightShader::shade(sg_DEMVertexShaderInfo& info)
{
   double i,j,k;
   info.normal.getComponents(i,j,k);
   glNormal3d(i,j,k);
   glColor3d(_demColor.red, _demColor.green, _demColor.blue);
}

void
sg_DaylightShader::setColor(sg_RGBA const& color)
{
   _demColor = color;
}

sg_RGBA const&
sg_DaylightShader::getColor() const
{
   return _demColor;
}


// sg_HSVShader

sg_HSVShader::sg_HSVShader(hf_DEM const* heightField)
                : sg_DEMVertexShader(heightField),
                  _lowColorStart(0.0),
                  _saturation(1.0),
                  _value(1.0),
                  _range(360.0)
{
}

void
sg_HSVShader::init()
{
   glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
   glEnable(GL_COLOR_MATERIAL);
}

void
sg_HSVShader::shade(sg_DEMVertexShaderInfo& info)
{
   double i,j,k;
   info.normal.getComponents(i,j,k);
   glNormal3d(i,j,k);

   double r,g,b;
   double dz = _z_max - _z_min;
   if (fabs(dz) < 0.0000001)
      dz = 1.0;
   double hue = _range * ((info.vertex.z - _z_min) / dz) + _lowColorStart;
   // XXX This needs to be fixed... if _range is big enough, this won't
   // XXX make hue < 360.0. -- jamie 06/16/97
   if (hue > 360.0)
      hue = hue - 360.0;
   sg_hsv2rgb(hue, _saturation, _value,r,g,b);
   glColor3d(r,g,b);
}

void
sg_HSVShader::setLowColorStart(double deg)
{
   if (deg > 360.0)
      deg = 360.0;
   else
      if (deg < 0.0)
         deg = 0.0;

   _lowColorStart = deg;
}

void
sg_HSVShader::setSaturation(double sat)
{
   if (sat < 0.0)
      sat = 0.0;
   if (sat > 1.0)
      sat = 1.0;
   _saturation = sat;
}

void
sg_HSVShader::setValue(double val)
{
   if (val < 0.0)
      val = 0.0;
   if (val > 1.0)
      val = 1.0;
   _value = val;
}

void
sg_HSVShader::setRange(double range)
{
   if (range < 1.0)
      range = 1.0;
   _range = range;
}

double
sg_HSVShader::getLowColorStart() const
{
   return _lowColorStart;
}

double
sg_HSVShader::getSaturation() const
{
   return _saturation;
}

double
sg_HSVShader::getValue() const
{
   return _value;
}

double
sg_HSVShader::getRange() const
{
   return _range;
}

// sg_RangeShader

sg_RangeShader::sg_RangeShader(hf_DEM const* heightField)
              : sg_DEMVertexShader(heightField)
{
}

void
sg_RangeShader::shade(sg_DEMVertexShaderInfo& info)
{
   double i,j,k;
   info.normal.getComponents(i,j,k);
   glNormal3d(i,j,k);
   
   if (_colors.size() == 0)
      glColor3d(1.0, 1.0, 1.0);

   double curz;
   double dz = _z_max - _z_min;
   if (fabs(dz) < 0.0000001)
      curz = 0.0;
   else
      curz = (info.vertex.z - _z_min) / dz;

   long index = _colors.size() * curz;
   if (index == _colors.size())
      index = index - 1;
   glColor3f(_colors[index].red, _colors[index].green, _colors[index].blue);
}

void
sg_RangeShader::setColors(vector<sg_RGBA> const& colorlist)
{
   _colors = colorlist;
}


// sg_SlopeShader

sg_SlopeShader::sg_SlopeShader(hf_DEM const* heightField)
              : sg_DEMVertexShader(heightField)
{
   _flatColor = sg_Color_Red;
   _steepColor = sg_Color_Yellow;
}

void
sg_SlopeShader::setFlatColor(sg_RGBA const & flatColor)
{
	_flatColor = flatColor;
}

void
sg_SlopeShader::setSteepColor(sg_RGBA const & steepColor)
{
	_steepColor = steepColor;
}

sg_RGBA const&
sg_SlopeShader::getFlatColor() const
{
   return _flatColor;
}

sg_RGBA const&
sg_SlopeShader::getSteepColor() const
{
   return _steepColor;
}

void
sg_SlopeShader::init()
{
   _red_diff = _flatColor.red - _steepColor.red;
   _green_diff = _flatColor.green - _steepColor.green;
   _blue_diff = _flatColor.blue - _steepColor.blue;
}

void
sg_SlopeShader::shade(sg_DEMVertexShaderInfo& info)
{
   double i,j,k;
   info.normal.getComponents(i,j,k);
   glNormal3d(i,j,k);
   
   double slope = info.normal.dot(sg_Vector(0,0,1));

   if (slope < 0.0)
      slope = 0.0;

   // Take the inverse cosine to get a linear distribution.
   double val = 1.0 - (acos(slope) / M_PI_2);


   float red = _steepColor.red + (val * _red_diff);
   float green = _steepColor.green + (val * _green_diff);
   float blue = _steepColor.blue + (val * _blue_diff);

   glColor3f(red,green,blue);
}


// sg_HeadingShader

sg_HeadingShader::sg_HeadingShader(hf_DEM const* heightField)
                : sg_DEMVertexShader(heightField)
{
}

void
sg_HeadingShader::shade(sg_DEMVertexShaderInfo& info)
{
   double i,j,k;
   info.normal.getComponents(i,j,k);
   glNormal3d(i,j,k);

   // Pick a really saturated color in HSV based on the heading of the normal.
   double heading;
   heading = (asin(j) * 180.0) / M_PI;
   if (i > 0.0)
      {
         if (j < 0.0)
            heading = heading + 360.0;
      }
   else
      heading = 180.0 - heading;

   double slope = info.normal.dot(sg_Vector(0,0,1));
   if (slope < 0.0)
      slope = 0.0;
   double val = acos(slope) / M_PI_2;

   double r,g,b;
   sg_hsv2rgb(heading, 1.0 - val, 1.0, r, g, b);
   glColor3d(r,g,b);
}


// sg_ContourShader

sg_ContourShader::sg_ContourShader(hf_DEM const* heightField)
                : sg_DEMVertexShader(heightField),
                  _interval(10)
{
	_color1 = sg_Color_Blue;
	_color2 = sg_Color_Yellow;
   _edgeColor = sg_Color_Red;
}


void
sg_ContourShader::shade(sg_DEMVertexShaderInfo& info)
{

   double fudge = fabs(_z_max - _z_min) + 100;

   info.vertex.z = info.vertex.z + fudge;
   info.vertex.z = floor(info.vertex.z / _interval) * _interval;
   info.vertex.z = info.vertex.z - fudge;
   if (info.vertex.z < _z_min)
      info.vertex.z = _z_min;

   info.q.z = info.q.z + fudge;
   info.q.z = floor(info.q.z / _interval) * _interval;
   info.q.z = info.q.z - fudge;
   if (info.q.z < _z_min)
      info.q.z = _z_min;

   info.r.z = info.r.z + fudge;
   info.r.z = floor(info.r.z / _interval) * _interval;
   info.r.z = info.r.z - fudge;
   if (info.r.z < _z_min)
      info.r.z = _z_min;

   // recalulate surface normal
   sg_Vector a(info.q.x - info.vertex.x,
              info.q.y - info.vertex.y,
              info.q.z - info.vertex.z);

   sg_Vector b(info.r.x - info.vertex.x,
              info.r.y - info.vertex.y,
              info.r.z - info.vertex.z);
   info.normal = a.cross(b);
   info.normal.normalize();


   double i,j,k;
   info.normal.getComponents(i,j,k);
   glNormal3d(i,j,k);

  // If normal isn't vertical, color it the "edge" color.
   if (fabs(info.normal.dot(sg_Vector(0,0,1))) < 0.99999)
      glColor3d(_edgeColor.red, _edgeColor.green, _edgeColor.blue);
   else
      {
         long val = static_cast<long>(info.vertex.z / _interval);
         if ((val % 2) == 0)
            glColor3d(_color1.red, _color1.green, _color1.blue);
         else
            glColor3d(_color2.red, _color2.green, _color2.blue);
      }
}

void
sg_ContourShader::setColor1(sg_RGBA const& color1)
{
   _color1 = color1;
}

void
sg_ContourShader::setColor2(sg_RGBA const& color2)
{
   _color2 = color2;
}

void
sg_ContourShader::setEdgeColor(sg_RGBA const& edgeColor)
{
   _edgeColor = edgeColor;
}

sg_RGBA const&
sg_ContourShader::getColor1()
{
   return _color1;
}

sg_RGBA const&
sg_ContourShader::getColor2()
{
   return _color2;
}

sg_RGBA const&
sg_ContourShader::getEdgeColor()
{
   return _edgeColor;
}

void
sg_ContourShader::setContourInterval(double interval)
{
   _interval = interval;
}

double
sg_ContourShader::getContourInterval() const
{
   return _interval;
}


//
// sg_GradientShader
//

sg_GradientShader::sg_GradientShader(hf_DEM const* heightField)
                 : sg_DEMVertexShader(heightField),
                   _lowColor(0,0,0),
                   _highColor(1,1,1)
{
}

void
sg_GradientShader::shade(sg_DEMVertexShaderInfo& info)
{
   double i,j,k;
   info.normal.getComponents(i,j,k);
   glNormal3d(i,j,k);

   float t = (info.vertex.z - _z_min) / (_z_max - _z_min);

   float red = _lowColor.red + (_highColor.red - _lowColor.red) * t;
   float green = _lowColor.green + (_highColor.green - _lowColor.green) * t;
   float blue = _lowColor.blue + (_highColor.blue - _lowColor.blue) * t;

   glColor3f(red,green,blue);
}

void
sg_GradientShader::setLowColor(sg_RGBA const& lowColor)
{
   _lowColor = lowColor;
}

void
sg_GradientShader::setHighColor(sg_RGBA const& highColor)
{
   _highColor = highColor;
}

sg_RGBA const&
sg_GradientShader::getLowColor() const
{
   return _lowColor;
}

sg_RGBA const&
sg_GradientShader::getHighColor() const
{
   return _highColor;
}

//
// sg_LakeShader
//

sg_LakeShader::sg_LakeShader(hf_DEM const* heightField)
                 : sg_DEMVertexShader(heightField),
                   _lakeColor(0,1,1),
                   _landColor(0.2,0.8,0.4)
{
}

void
sg_LakeShader::shade(sg_DEMVertexShaderInfo& info)
{
   double i,j,k;
   info.normal.getComponents(i,j,k);
   glNormal3d(i,j,k);

   // Color flat areas the lake color, everything else the land color.

   if (fabs(info.normal.dot(sg_Vector(0,0,1)) - 1.0) < 0.0001)
      {
         glColor3d(_lakeColor.red, _lakeColor.green, _lakeColor.blue);
      }
   else
      glColor3d(_landColor.red, _landColor.green, _landColor.blue);
}


//
// sg_TextureShader
//

sg_TextureShader::sg_TextureShader(hf_DEM const* heightField)
               : sg_DEMVertexShader(heightField)
{
}

void
sg_TextureShader::init()
{
   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

   glTexImage2D(GL_TEXTURE_2D, 0, 3, _imageWidth,
      _imageHeight, 0, GL_RGB, GL_UNSIGNED_BYTE,
      _pixels);

   glEnable(GL_TEXTURE_2D);

   glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}

void
sg_TextureShader::cleanup()
{
	glDisable(GL_TEXTURE_2D);
}

void
sg_TextureShader::setImageWidth(long width)
{
	_imageWidth = (double) width;
}

void
sg_TextureShader::setImageHeight(long height)
{
	_imageHeight = (double) height;
}

void
sg_TextureShader::setDRGHeight(long drgHeight)
{
	_drgHeight = (double) drgHeight;
}

void
sg_TextureShader::setDRGWidth(long drgWidth)
{
	_drgWidth = (double) drgWidth;
}

void
sg_TextureShader::setPixels(GLuint* rgb)
{
	_pixels = rgb;
}

void
sg_TextureShader::shade(sg_DEMVertexShaderInfo& info)
{

   double i,j,k;
   info.normal.getComponents(i,j,k);
   glNormal3f(i,j,k);

   double tx, ty;

   tx = (_drgWidth / _imageWidth) * ( (info.vertex.x - _x_min) / (_x_max - _x_min) );
   ty = (_drgHeight / _imageHeight) * ( (info.vertex.y - _y_min) / (_y_max - _y_min) );

   glTexCoord2f((float) tx, (float) 1.0 - ty);
   glColor3d(1.0,1.0,1.0);
}

