
#include <math.h>

#ifndef INCLUDED_SG_MAGICCARPETCONTROLLER_H
#include "scenegraph/sg_MagicCarpetController.h"
#endif

#ifndef INCLUDED_SG_SCENENODE_HXX
#include "scenegraph/sg_SceneNode.hxx"
#endif

#ifndef INCLUDED_SG_MATRIXUTILS_HXX
#include "scenegraph/sg_MatrixUtils.hxx"
#endif

#ifndef INCLUDED_SG_TYPES_HXX
#include "scenegraph/sg_Types.hxx"
#endif

#ifndef INCLUDED_SG_VECTOR_HXX
#include "scenegraph/sg_Vector.hxx"
#endif

#ifdef WIN32
#define M_PI            3.14159265358979323846
#define M_PI_2          1.57079632679489661923
#endif

sg_MagicCarpetController::sg_MagicCarpetController(sg_CameraNode* camera)
                        : _camera(camera),
                          _first_time(true),
                          _old_x(0),
                          _old_y(0),
                          _moveRate(1.0),
                          _accelThreshold(5.0),
                          _accel(2.0),
                          _autoLevel(true)
{

}

sg_MagicCarpetController::~sg_MagicCarpetController()
{

}

void
sg_MagicCarpetController::sync()
{
   _first_time = true;

   if (_camera == 0)
      return;

   // XXX Determining the orientation of the camera and converting
   // XXX to quaternion is too much trouble right now. Let's just
   // XXX go with whatever the quaternion was the last time we
   // XXX used the controller. 
   // XXX Later, we'll insert the code to determine current camera
   // XXX position and orientation.

   return;

}

void
sg_MagicCarpetController::mouseMove(long x, long y, long status)
{
   if (_camera == 0)
      return;

   if (!((status & button1) | (status & button2)))
      return;

   if (_first_time)
      {
         _old_x = x;
         _old_y = y;
         _first_time = false;
         return;
      }

   long ll_x, ll_y;
   long width, height;
   _camera->getViewPort(ll_x, ll_y, width, height);
   long dx = x - _old_x;
   long dy = y - _old_y;
   _old_x = x;
   _old_y = y;

   sg_Matrix m(_quat.getMatrix());

   sg_Point look;
   sg_Point up;
   sg_Point right;

   sg_MatrixUtils::transformPoint(m, sg_Point(0,1,0), look);
   sg_MatrixUtils::transformPoint(m, sg_Point(0,0,1), up);
   sg_MatrixUtils::transformPoint(m, sg_Point(1,0,0), right);

   if (status & button1)
      {
         double d_theta = (M_PI * dx) / width;
         double d_phi   = (M_PI * dy) / height;

         if ((status & shiftKey) && !_autoLevel)
            _quat = _quat + sg_Quaternion(d_theta,
                                          sg_Vector(look.x, look.y, look.z));
         else
         {
            _quat = _quat + sg_Quaternion(d_theta * -1.0,
                                          sg_Vector(up.x, up.y, up.z));

            if (_autoLevel)
            {
               m = _quat.getMatrix();
               sg_MatrixUtils::transformPoint(m, sg_Point(0,1,0), look);
               sg_Vector lookVec(look.x, look.y, look.z);
               if (fabs(1.0 - lookVec.dot(sg_Vector(0,0,1))) > 0.0001)
               {
                  sg_MatrixUtils::transformPoint(m, sg_Point(0,0,1), up);
                  sg_Vector upVec(up.x, up.y, up.z);

                  sg_Vector levelVec = lookVec.cross(sg_Vector(0,0,1));
                  levelVec.normalize();

                  sg_MatrixUtils::transformPoint(m, sg_Point(1,0,0), right);
                  sg_Vector rightVec(right.x, right.y, right.z);
                  rightVec.normalize();

                  double dotprod = rightVec.dot(levelVec);
                  if (fabs(dotprod) < 0.001)
                     dotprod = 0.0;
                  if (dotprod > 1.0)
                     dotprod = 1.0;
                  if (dotprod < -1.0)
                     dotprod = -1.0;
                  double rollAngle = acos(dotprod);
                  _quat = _quat + sg_Quaternion(rollAngle, lookVec);
               }
            }
         }

         sg_MatrixUtils::transformPoint(m, sg_Point(1,0,0), right);
         _quat = _quat + sg_Quaternion(d_phi,
                                       sg_Vector(right.x, right.y, right.z));

         m = _quat.getMatrix();
         sg_MatrixUtils::transformPoint(m, sg_Point(0,1,0), look);
         sg_MatrixUtils::transformPoint(m, sg_Point(0,0,1), up);
      }
   else
   if (status & button2)
      {
         // Translate camera. 
         // +/- mouseY => Forward,back along look vector
         // +/- mouseX => Right,left (perpendicular to look and up vectors)
         // shft and +/-mouseY => Up,down along up vector.

         double wx = dx * _moveRate;
         double wy = dy * _moveRate * -1.0;
         double mouseDist = sqrt(dx * dx + dy * dy);
         if (mouseDist > _accelThreshold)
         {
            wx = wx * _accel;
            wy = wy * _accel;
         }
         _position = _position + sg_Vector(right.x * wx, right.y * wx, right.z * wx);

         if (status & shiftKey)
            _position = _position + sg_Vector(up.x * wy, up.y * wy, up.z * wy);
         else
            _position = _position + sg_Vector(look.x * wy, look.y * wy, look.z * wy);
      }

   double ex,ey,ez;
   _position.getComponents(ex,ey,ez);
   _camera->look(ex,ey,ez, ex + look.x, ey + look.y, ez + look.z,
                 up.x, up.y, up.z);


}

void
sg_MagicCarpetController::button1Up(long x, long y, long status)
{
   _first_time = true;
}

void
sg_MagicCarpetController::button2Up(long x, long y, long status)
{
   _first_time = true;
}

void
sg_MagicCarpetController::setWorldUnitsPerPixel(double moveRate)
{
   if (moveRate < 0.0)
      moveRate = 0.0;

   _moveRate = moveRate;
}

void
sg_MagicCarpetController::setAccelThreshold(double accelThresh)
{
   if (accelThresh < 0.0)
      accelThresh = 0.0;

   _accelThreshold = accelThresh;
}

void
sg_MagicCarpetController::setAcceleration(double accel)
{
   if (accel < 0.0)
      accel = 0.0;

   _accel = accel;
}
