
#ifndef INCLUDED_SG_MAGICCARPETCONTROLLER_H
#define INCLUDED_SG_MAGICCARPETCONTROLLER_H

#ifndef INCLUDED_SG_CONTROLLER_HXX
#include "scenegraph/sg_Controller.hxx"
#endif

#ifndef INCLUDED_SG_QUATERNION_H
#include "scenegraph/sg_Quaternion.h"
#endif

#ifndef INCLUDED_SG_VECTOR_HXX
#include "scenegraph/sg_Vector.hxx"
#endif



class sg_CameraNode;

class sg_MagicCarpetController : public sg_Controller
{
   public:
	   sg_MagicCarpetController(sg_CameraNode* camera);
	  ~sg_MagicCarpetController();

      void sync();

      // Mouse events.
      void mouseMove(long x, long y, long status = noKeysPressed);
      void button1Up(long x, long y, long status = noKeysPressed);
      void button2Up(long x, long y, long status = noKeysPressed);

      void setWorldUnitsPerPixel(double moveRate);
         // moveRate > 0
         // Default: 1.0
         // The camera translates in a given direction 'moveRate' 
         // world units for every pixel the cursor moves.

      void setAccelThreshold(double accelThresh);
         // accelThresh > 0
         // Default: 5.0
         // If the distance (in pixels) between two successive
         // mouse positions is greater than or equal to accelThresh,
         // the camera movement rate is multiplied by the acceleration.

      void setAcceleration(double accel);
         // accel > 0
         // Default: 2.0

      void setAutoLevel(bool flag);
         // Autoleveling forces the camera to be level at all times.
         // Here, 'level' means that the camera's x axis is parallel
         // to the World Coordinate System xy plane whenever possible.
         // Default: false

   private:

      sg_CameraNode* _camera;
      sg_Quaternion  _quat;
      sg_Vector      _position;
      bool           _first_time;
      long           _old_x, _old_y;

      double _moveRate;
      double _accelThreshold;
      double _accel;
      bool   _autoLevel;
};

#endif  // INCLUDED_SG_MAGICCARPETCONTROLLER_H
