#ifndef INCLUDED_SG_MATRIXUTILS_HXX
#define INCLUDED_SG_MATRIXUTILS_HXX

// $Id: sg_MatrixUtils.hxx,v 1.2 1997/01/10 08:07:55 moyers Exp $

class sg_Matrix;
class sg_Vector;
class sg_Point;

class sg_MatrixUtils
{
   // Class utility that provides functions for use with sg_Matrix.

   public:
      static
      void rotateMatrix(sg_Matrix& matrix,
                        double angle, double x, double y, double z);
         // Rotation about a line from the origin to the point (x,y,z) by
         // angle 'angle'.

      static
      void translateMatrix(sg_Matrix& matrix,
                           double dx, double dy, double dz);
      static
      void scaleMatrix(sg_Matrix& matrix,
                       double sx, double sy, double sz);

      static
      void transformPoint(sg_Matrix const& matrix,
                             sg_Point const& p,
                             sg_Point& q);
         // Transforms point p by transformation matrix giving point q.

      static
      void invertMatrix(sg_Matrix const& b,
                        sg_Matrix& a);
         // Finds the inverse of b and returns it in a.

      static
      void calcCameraMatrix(sg_Vector& up,          // In / out
                              sg_Vector& look,      // In / out
                              sg_Point const& p,    // In
                              sg_Vector& right,     // Out
                              sg_Matrix& m);        // Out
         // A "Camera" has a local X, Y, and Z axis. The origin of this local
         // coordinate system is at point p. This function calculates a matrix
         // which will move an object at the World Coordinate system origin
         // to point p, and align the object's axes with the camera's.
         // The look vector is parallel to the camera's +Y axis.
         // The up vector together with the look vector defines a plane. The 
         // actual up vector will be orthogonal to 'look' and lie in this plane.
         // The calculated up vector will be parallel to the camera's positive X axis.
         // 'right' is calculated as look cross up, and is parallel to the camera's
         // positive X axis.
         

   private:

      sg_MatrixUtils();  // This class isn't meant to be instantiated or 
                         // inherited from.

      static
      void genRotateMatrix(double angle, double x, double y, double z,
                           sg_Matrix& matrix);
};

#endif  // INCLUDED_SG_MATRIXUTILS_HXX
