
#include <math.h>

#ifndef INCLUDED_SG_QUATERNION_H
#include "sg_Quaternion.h"
#endif

sg_Quaternion::sg_Quaternion()
             : _v(0.0, 0.0, 0.0),
               _s(1.0)
{

}

sg_Quaternion::sg_Quaternion(double theta, sg_Vector const& axis)
{
   _v = axis;
   _v.normalize();
   _v = _v * sin(theta / 2.0);
   _s = cos(theta / 2.0);
}

sg_Quaternion const&
sg_Quaternion::operator=(sg_Quaternion const& rhs)
{
   if (&rhs == this)
      return *this;

   _s = rhs._s;
   _v = rhs._v;

   return *this;
}

sg_Quaternion::sg_Quaternion(sg_Quaternion const& rhs)
{
   operator=(rhs);
}

void
sg_Quaternion::normalize()
{
   double i,j,k;
   _v.getComponents(i,j,k);

   double mag = sqrt(i * i + j * j + k * k + _s * _s);
   _v = _v * (1.0 / mag);
}

sg_Matrix
sg_Quaternion::getMatrix() const
{
   double i,j,k;
   _v.getComponents(i,j,k);

   sg_Matrix m;

   m(0,0) = 1.0 - 2.0 * (j * j + k * k);
   m(0,1) = 2.0 * (i * j - k * _s);
   m(0,2) = 2.0 * (k * i + j * _s);
   m(0,3) = 0.0;

   m(1,0) = 2.0 * (i * j + k * _s);
   m(1,1) = 1.0 - 2.0 * (k * k + i * i);
   m(1,2) = 2.0 * (j * k - i * _s);
   m(1,3) = 0.0;

   m(2,0) = 2.0 * (k * i - j * _s);
   m(2,1) = 2.0 * (j * k + i * _s);
   m(2,2) = 1.0 - 2.0 * (j * j + i * i);
   m(2,3) = 0.0;

   m(3,0) = 0.0;
   m(3,1) = 0.0;
   m(3,2) = 0.0;
   m(3,3) = 1.0;

   return m;
}

sg_Quaternion::~sg_Quaternion()
{

}

sg_Quaternion const&
operator+(sg_Quaternion const& left,
          sg_Quaternion const& right)
{
   sg_Vector t1(left._v);
   t1 = t1 * right._s;

   sg_Vector t2(right._v);
   t2 = t2 * left._s;

   sg_Vector t3(right._v);
   t3 = t3.cross(left._v);
   sg_Vector tf(t1 + t2);
   tf = tf + t3;

   double tf_s = left._s * right._s - left._v.dot(right._v);

   sg_Quaternion result;
   result._v = tf;
   result._s = tf_s;

   result.normalize();

   return result;
}
