
// $Id: sg_SceneIterator.cxx,v 1.1.1.1 1997/01/08 21:54:13 pjgray Exp $
#ifdef WIN32
#include <afxwin.h>
#include <afxext.h>
#include <afxtempl.h>
#include <iostream>
#else
#include <iostream.h>
#endif

#include "GL/gl.h"

#ifndef INCLUDED_SG_SCENEITERATOR_HXX
#include "scenegraph/sg_SceneIterator.hxx"
#endif

#ifndef INCLUDED_SG_SCENENODE_HXX
#include "scenegraph/sg_SceneNode.hxx"
#endif

#if 0
// Temporary debugging aid.
#include <string>
#include <fstream>
class sg_Logfile
{
   public:
      sg_Logfile(string const& filename) { _ofstr.open(filename.c_str()); }
      ~sg_Logfile() { _ofstr.close(); }
      ofstream& operator()() {return _ofstr;}
   private:
      ofstream _ofstr;
};


static sg_Logfile logfile("logfile.txt");
#endif

sg_SceneIterator::sg_SceneIterator()
{
}

void
sg_SceneIterator::preVisit()
{
}

void
sg_SceneIterator::postVisit()
{
}

vector<sg_SceneNodePtr>&
sg_SceneIterator::getCompositeChildren(sg_CompositeNode* compnode) const
{
   return compnode->_children;
}


// sg_CompositeIter

sg_CompositeIter::sg_CompositeIter()
                : _myNode(0)
{
}

sg_CompositeIter::sg_CompositeIter(sg_CompositeIter const& right)
                : _myNode(0)
{
   operator=(right);
}

sg_CompositeIter::sg_CompositeIter(sg_CompositeNode* node)
                : _myNode(node)
{
}

sg_CompositeIter const&
sg_CompositeIter::operator=(sg_CompositeIter const& right)
{
   if (this == &right)
      return *this;

   _myNode = right._myNode;
   _curChild = right._curChild;

   return *this;
}

void
sg_CompositeIter::first()
{
   if (_myNode == 0)
      return;

   vector<sg_SceneNodePtr>& children = getCompositeChildren(_myNode);
   _curChild = children.begin();
}

void
sg_CompositeIter::next()
{
   if (_myNode == 0)
      return;

   vector<sg_SceneNodePtr>& children = getCompositeChildren(_myNode);
   if (_curChild != children.end())
      _curChild++;
}

bool
sg_CompositeIter::isDone() const
{
   if (_myNode == 0)
      return false;

   vector<sg_SceneNodePtr>& children = getCompositeChildren(_myNode);
   return _curChild == children.end();
}

sg_SceneNode*
sg_CompositeIter::currentItem() const
{
   return &(**_curChild);
}


// sg_PreOrderIter

sg_PreOrderIter::sg_PreOrderIter(sg_SceneNode* node)
                    : _myNode(node),
                      _currentNode(0),
                      _isDoneFlag(true)
{
}

void
sg_PreOrderIter::first()
{
   _currentNode = _myNode;

   if (_currentNode != 0)
      {
         preVisit();
         _isDoneFlag = false;
      }
   else
      _isDoneFlag = true;
}

void
sg_PreOrderIter::next()
{
   if (_currentNode == 0)
      {
         _isDoneFlag = true;
         return;
      }

   sg_CompositeNode* compnode;
   compnode = _currentNode->getComposite();
   if (compnode != 0)
      {
         _nodeStack.push_back(sg_CompositeIter(compnode));
         _nodeStack.back().first();
      }
   else
      postVisit();

   bool found = false;
   _currentNode = 0;
   while (!_nodeStack.empty() && (!found))
      {
         if (!(_nodeStack.back().isDone()))
            {
               preVisit();
               _currentNode = _nodeStack.back().currentItem();
               _nodeStack.back().next();
               found = true;
            }
         else
            {
               _nodeStack.pop_back();
               postVisit();
            }
      }

   if (!found)
      {
         _isDoneFlag = true;
         return;
      }
}

bool
sg_PreOrderIter::isDone() const
{
   return _isDoneFlag;
}

sg_SceneNode*
sg_PreOrderIter::currentItem() const
{
   return _currentNode;
}

// sg_RenderIter

sg_RenderIter::sg_RenderIter(sg_SceneNode* nodeptr)
             : sg_PreOrderIter(nodeptr)
{
}

void
sg_RenderIter::preVisit()
{
   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();
}

void
sg_RenderIter::postVisit()
{
   glMatrixMode(GL_MODELVIEW);
   glPopMatrix();
}


// sg_PostOrderIter

sg_PostOrderIter::sg_PostOrderIter(sg_SceneNode* node)
                    : _myNode(node),
                      _currentNode(0),
                      _isDoneFlag(true)
{
}

void
sg_PostOrderIter::first()
{
   _currentNode = 0;
   if (_myNode == 0)
      {
         _isDoneFlag = true;
         return;
      }

   _isDoneFlag = false;

   // Find the first non-composite child. 
   sg_SceneNode* curnode = _myNode;
   sg_CompositeNode* curcomp;
   bool found = false;
   while ((!found) && ((curcomp = curnode->getComposite()) != NULL))
      {
         _stack.push_back(StackEntry(sg_SceneNodePtr(curcomp),sg_CompositeIter(curcomp)));
         sg_CompositeIter& curiter = _stack.back().second;
         curiter.first();
         if (!curiter.isDone())
            curnode = curiter.currentItem();
         else
            {
               _stack.pop_back();
               found = true;
            }
      }

   preVisit();
   _currentNode = curnode;
}

void
sg_PostOrderIter::next()
{
   // If the stack's empty, we're done.
   if (_stack.empty())
      {
         if (_currentNode != 0)
            {
               postVisit();
               _currentNode = 0;
            }
         _isDoneFlag = true;
         return;
      }

   // Get the next node from the Composite Iterator on the top of the stack.
   sg_SceneNode* curnode;
   sg_CompositeIter& curiter = _stack.back().second;
   curiter.next();


   if (curiter.isDone())
      {
         // If the topmost composite node has no more children, it becomes
         // the current node and we pop its entry off the stack.
         curnode = &(*(_stack.back().first));
         _stack.pop_back();
      }
   else
      {
         curnode = curiter.currentItem();

         // The current node may be another Composite node. If so, keep
         // pushing them on the stack until we run out or find a non composite node.
         sg_CompositeNode* curcomp;
         bool found = false;  // True if we find a composite node which is a leaf.
         while (((curcomp = curnode->getComposite()) != NULL) && (!found))
            {
               _stack.push_back(StackEntry(sg_SceneNodePtr(curcomp),
                           sg_CompositeIter(curcomp)));
               sg_CompositeIter& curiter = _stack.back().second;
               curiter.first();
               if (!curiter.isDone())
                  curnode = curiter.currentItem();
               else
                  found = true;
            }
      }

   postVisit();

   preVisit();
   _currentNode = curnode;
   return;
}

bool
sg_PostOrderIter::isDone() const
{
   return _isDoneFlag;
}

sg_SceneNode*
sg_PostOrderIter::currentItem() const
{
   return _currentNode;
}

void
sg_PostOrderIter::preVisit()
{
   cout << "sg_PostOrderIter::preVisit()\n";
}

void
sg_PostOrderIter::postVisit()
{
   cout << "sg_PostOrderIter::postVisit()\n";
}


