#ifndef INCLUDED_SG_SCENEITERATOR_HXX
#define INCLUDED_SG_SCENEITERATOR_HXX

// $Id: sg_SceneIterator.hxx,v 1.1.1.1 1997/01/08 21:54:13 pjgray Exp $

#ifdef WIN32
#include <afxwin.h>
#include <afxext.h>
#include <afxtempl.h>
#include <vector>
#include <stack>
using namespace std;
#else
#include <vector.h>
#include <stack.h>
#endif

#ifndef INCLUDED_SG_SCENENODEPTR_HXX
#include "scenegraph/sg_SceneNodePtr.hxx"
#endif

class sg_SceneNode;
class sg_CompositeNode;





class sg_SceneIterator
{
   // Abstract Base Class for scene iterators.

   public:

      virtual
      void first() = 0;
         // Position the interator on the first item.

      virtual
      void next() = 0;
         // Move the iterator to the next item.

      virtual
      bool isDone() const = 0;
         // returns true if we've moved beyond the last item.

      virtual
      sg_SceneNode* currentItem() const = 0;



   protected:

      sg_SceneIterator();
 
      vector<sg_SceneNodePtr>& getCompositeChildren(sg_CompositeNode* compnode) const;

      virtual
      void preVisit();
         // Called before iterator "visits" the node that will become the
         // new "currentItem".
 
      virtual
      void postVisit();
        // Called after iterator "visits" the currentItem.

};

class sg_CompositeIter : public sg_SceneIterator
{
   // Iterator for a single sg_CompositeNode.

   public:

      sg_CompositeIter();
      sg_CompositeIter(sg_CompositeIter const& right);
      explicit sg_CompositeIter(sg_CompositeNode* node);
      sg_CompositeIter const& operator=(sg_CompositeIter const& right);

      void first();
      void next();
      bool isDone() const;
      sg_SceneNode* currentItem() const;

	  bool operator<(sg_CompositeIter const& rhs) const {return false;}
	  bool operator==(sg_CompositeIter const& rhs) const {return false;}

   private:
      sg_CompositeNode* _myNode;
      vector<sg_SceneNodePtr>::iterator _curChild;

};


class sg_PreOrderIter : public sg_SceneIterator
{
   // A Pre Order (Visits parent before children) iterator.

   public:

      explicit sg_PreOrderIter(sg_SceneNode* nodeptr);

      void first();
      void next();
      bool isDone() const;
      sg_SceneNode* currentItem() const;

   private:

      sg_SceneNode* _myNode;
      sg_SceneNode* _currentNode;
      bool _isDoneFlag;
      vector<sg_CompositeIter> _nodeStack;

};


class sg_RenderIter : public sg_PreOrderIter
{
   // Renders a scenegraph.

   public:

      explicit sg_RenderIter(sg_SceneNode* nodeptr);

   protected:

      void preVisit();
      void postVisit();
};


class sg_PostOrderIter : public sg_SceneIterator
{
   // A Post Order (Visits children before parent) iterator.

   public:

      explicit sg_PostOrderIter(sg_SceneNode* nodeptr);

      void first();
      void next();
      bool isDone() const;
      sg_SceneNode* currentItem() const;

   protected:

      void preVisit();
      void postVisit();

   private:

      sg_SceneNode* _myNode;
      sg_SceneNode* _currentNode;
      bool _isDoneFlag;
      
      // Each entry on the stack will be a composite node and its iterator.
      typedef pair<sg_SceneNodePtr, sg_CompositeIter> StackEntry;
      vector<StackEntry> _stack;

};


#endif  // INCLUDED_SG_SCENEITERATOR_HXX
