
// $Id: sg_SceneNode.cxx,v 1.8 1997/02/01 01:03:11 moyers Exp $

#include <math.h>

#ifdef WIN32
#include <afxwin.h>
#include <afxext.h>
#include <afxtempl.h>
#include <algorithm>
using namespace std;
#else
#include <algo.h>
#endif

#include "GL/gl.h"
#include "GL/glu.h"

#ifndef INCLUDED_SG_SCENENODE_HXX
#include "scenegraph/sg_SceneNode.hxx"
#endif

#ifndef INCLUDED_SG_SCENEVISITOR_HXX
#include "scenegraph/sg_SceneVisitor.hxx"
#endif

#ifndef INCLUDED_SG_TRANSFORMATION_HXX
#include "scenegraph/sg_Transformation.hxx"
#endif

#ifndef INCLUDED_SG_SHAPE_HXX
#include "scenegraph/sg_Shape.hxx"
#endif

#ifndef INCLUDED_SG_MATRIX_HXX
#include "scenegraph/sg_Matrix.hxx"
#endif

#ifndef INCLUDED_SG_VECTOR_HXX
#include "scenegraph/sg_Vector.hxx"
#endif

sg_SceneNode::sg_SceneNode()
            : _active(true),
              _parent(0),
              _shape(0)
{
}

sg_SceneNode::~sg_SceneNode()
{
}

bool
sg_SceneNode::isActive() const
{
   return _active;
}

void
sg_SceneNode::render()
{
   glMatrixMode(GL_MODELVIEW);
   glMultMatrixd(_transformation.getMatrix().getArray());
   _appearance.apply(); 
}

void
sg_SceneNode::renderLoRes()
{
   render();
}

sg_Transformation&
sg_SceneNode::getTransformation()
{
   return _transformation;
}

sg_Shape*
sg_SceneNode::getShape()
{
   return _shape;
}

sg_Shape const*
sg_SceneNode::getShape() const
{
   return _shape;
}

bool
sg_SceneNode::getBoundingBox(double& min_x, double& min_y, double& min_z,
                             double& max_x, double& max_y, double& max_z) const
{
   return false;
}

bool
sg_SceneNode::getBoundingBox(sg_MinBoundBox& mbb) const
{
   return false;
}

sg_CompositeNode*
sg_SceneNode::getComposite()
{
   // sg_CompositeNode will override this to return the correct value.
   return 0;
}

sg_Appearance::ShadingStyle
sg_SceneNode::getShadingStyle() const
{
   return _appearance.getShadingStyle();
}

void
sg_SceneNode::setActive(bool activeFlag)
{
   _active = activeFlag;
}

void
sg_SceneNode::identity()
{
   _transformation.identity();
}

void
sg_SceneNode::rotate(double angle, double x, double y, double z)
{
   _transformation.rotate(angle,x,y,z);
}

void
sg_SceneNode::translate(double dx, double dy, double dz)
{
   _transformation.translate(dx,dy,dz);
}

void
sg_SceneNode::scale(double sx, double sy, double sz)
{
   _transformation.scale(sx,sy,sz);
}


void
sg_SceneNode::setShape(sg_Shape* shape)
{
   delete _shape;
   _shape = shape;
}

void
sg_SceneNode::setParent(sg_SceneNode* parent)
{
   _parent = parent;
}

sg_SceneNode const*
sg_SceneNode::getParent() const
{
   return _parent;
}


void
sg_SceneNode::setColor(sg_RGBA const& color)
{
   _appearance.setColor(color);
}

void
sg_SceneNode::setShadingStyle(sg_Appearance::ShadingStyle style)
{
   _appearance.setShadingStyle(style);
}

void
sg_SceneNode::setLighting(bool flag)
{
   _appearance.setLighting(flag);
}

bool
sg_SceneNode::isLit()
{
   return _appearance.isLit();
}

void
sg_SceneNode::setPolygonMode(sg_Appearance::PolygonMode mode)
{
   _appearance.setPolygonMode(mode);
}

sg_Appearance::PolygonMode
sg_SceneNode::getPolygonMode()
{
   return _appearance.getPolygonMode();
}

void
sg_SceneNode::setShininess(double shininess)
{
   _appearance.setShininess(shininess);
}

double
sg_SceneNode::getShininess() const
{
   return _appearance.getShininess();
}

void
sg_SceneNode::setSpecularColor(sg_RGBA const& specColor)
{
   _appearance.setSpecularColor(specColor);
}

sg_RGBA const&
sg_SceneNode::getSpecularColor() const
{
   return _appearance.getSpecularColor();
}



void
sg_SceneNode::accept(sg_SceneVisitor& visitor)
{
}

// sg_CompositeNode

sg_CompositeNode::sg_CompositeNode()
{
}

sg_CompositeNode*
sg_CompositeNode::getComposite()
{
   return this;
}

void
sg_CompositeNode::accept(sg_SceneVisitor& visitor)
{
   visitor.visitCompositeNode(this);
}

void
sg_CompositeNode::addNode(sg_SceneNode* node)
{
   // Add the node to the composite node.
   // Note that while nodeptr is const, the node it points to is not.

    _children.push_back(sg_SceneNodePtr(node));
    (*_children.back()).setParent(this);
 }

void
sg_CompositeNode::removeNode(sg_SceneNode* node)
{
   // See the node is one of our children
   NodeVector::iterator pos = find(_children.begin(), _children.end(), 
                                   sg_SceneNodePtr(node));
   if (pos != _children.end())
      {
         _children.erase(pos);
         node->setParent(0);
      }
}

void
sg_CompositeNode::replaceNode(sg_SceneNode* oldnode,
                              sg_SceneNode* newnode)
{
   NodeVector::iterator pos = find(_children.begin(), _children.end(),
                                   sg_SceneNodePtr(oldnode));
   if (pos != _children.end())
      {
         *pos = sg_SceneNodePtr(newnode);
         newnode->setParent(this);
         oldnode->setParent(0);
      }
}


// sg_ShapeNode

sg_ShapeNode::sg_ShapeNode(sg_Shape* myshape)
{
   setShape(myshape);
}

void
sg_ShapeNode::render()
{
   sg_SceneNode::render();

   if (getShape() != 0)
      {
         getShape()->render();
      }
}

void
sg_ShapeNode::renderLoRes()
{
   sg_SceneNode::render();

   if (getShape() != 0)
      {
         getShape()->renderLoRes();
      }
}

void
sg_ShapeNode::accept(sg_SceneVisitor& visitor)
{
   visitor.visitShapeNode(this);
}

bool
sg_ShapeNode::getBoundingBox(double& min_x, double& min_y, double& min_z,
                             double& max_x, double& max_y, double& max_z) const
{
   sg_Shape const* shape = getShape();

   if (shape != 0)
      {
         shape->getBoundingBox(min_x, min_y, min_z, max_x, max_y, max_z);
         return true;
      }
   return false;
}                             

bool
sg_ShapeNode::getBoundingBox(sg_MinBoundBox& mbb) const
{
   if (getShape() != 0)
      {
         getShape()->getBoundingBox(mbb);
         return true;
      }
   return false;
}

// sg_LightNode

sg_LightNode::sg_LightNode(GLenum lightID)
            : _lightID(lightID),
              _on(true),
              _icon(0),
              _showIcon(false),
              _pos(0.0,0.0,0.0),
              _dir(0.0,1.0,0.0),
              _ambient(0.0,0.0,0.0,1.0),
              _diffuse(1.0,1.0,1.0,1.0),
              _specular(1.0,1.0,1.0,1.0),
              _constantAtten(1.0),
              _linearAtten(0.0),
              _quadAtten(0.0)
{
   sg_Vector up(0,0,1);   // Light thinks +Z is up.
   sg_Vector right;
   sg_Matrix m;
   sg_MatrixUtils::calcCameraMatrix(up, _dir, _pos, right, m);
   getTransformation().setMatrix(m);
}

sg_LightNode::~sg_LightNode()
{
	glDisable(_lightID);
}

GLenum
sg_LightNode::getLightID() const
{
   return _lightID;
}

bool
sg_LightNode::isOn() const
{
   return _on;
}

void
sg_LightNode::setIcon(sg_ShapeNode* icon)
{
   _icon = icon;
}

void
sg_LightNode::showIcon(bool flag)
{
   _showIcon = flag;
}

bool
sg_LightNode::iconOn() const
{
   return _showIcon;
}

void
sg_LightNode::render()
{
   if (!_on)
      {
         glDisable(_lightID);
         return;
      }
   glEnable(_lightID);

   GLfloat params[4];

   _ambient.getArray(params);
   glLightfv(_lightID, GL_AMBIENT, params);

   _diffuse.getArray(params);
   glLightfv(_lightID, GL_DIFFUSE, params);

   _specular.getArray(params);
   glLightfv(_lightID, GL_SPECULAR, params);

   glLightf(_lightID, GL_CONSTANT_ATTENUATION, _constantAtten);
   glLightf(_lightID, GL_LINEAR_ATTENUATION, _linearAtten);
   glLightf(_lightID, GL_QUADRATIC_ATTENUATION, _quadAtten);
}

void
sg_LightNode::accept(sg_SceneVisitor& visitor)
{
   visitor.visitLightNode(this);
}

void
sg_LightNode::turnOn()
{
   _on = true;
}

void
sg_LightNode::turnOff()
{
   _on = false;
}

void
sg_LightNode::setPosition(sg_Point const& pos)
{
   _pos = pos;
   sg_Vector up(0,0,1);   // Light thinks +Z is up.
   sg_Vector right;
   sg_Matrix m;
   sg_MatrixUtils::calcCameraMatrix(up, _dir, _pos, right, m);
   getTransformation().setMatrix(m);
}

void
sg_LightNode::setPosition(double x, double y, double z)
{
   setPosition(sg_Point(x,y,z));
}

void
sg_LightNode::setDirection(sg_Vector const& dir)
{
   _dir = dir;
   _dir.normalize();
   sg_Vector up(0,0,1);   // Light thinks +Z is up.

   if (fabs(up.dot(_dir)) > (1.0 - 0.0000001))
      {
         // The default 'up' vector and the light direction are nearly colinear.
         // Choose a different up vector.
         up = sg_Vector(1,0,0);
      }

   sg_Vector right;
   sg_Matrix m;
   sg_MatrixUtils::calcCameraMatrix(up, _dir, _pos, right, m);
   getTransformation().setMatrix(m);
}

void
sg_LightNode::setDirection(double i, double j, double k)
{
   setDirection(sg_Vector(i,j,k));
}

void
sg_LightNode::shineOn(sg_Point const& target)
{
   sg_Vector up(0,0,1);
   sg_Vector right;
   sg_Matrix m;
   _dir = sg_Vector(target.x - _pos.x, target.y - _pos.y, target.z - _pos.z);
   _dir.normalize();

   if (fabs(up.dot(_dir)) > (1.0 - 0.0000001))
      {
         // The default 'up' vector and the light direction are nearly colinear.
         // Choose a different up vector.
         up = sg_Vector(1,0,0);
      }

   sg_MatrixUtils::calcCameraMatrix(up, _dir, _pos, right, m);
   getTransformation().setMatrix(m);
}

void
sg_LightNode::setAmbient(sg_RGBA const& rgba)
{
   _ambient = rgba;
}

void
sg_LightNode::setAmbient(double r, double g, double b, double a)
{
   setAmbient(sg_RGBA(r,g,b,a));
}

void
sg_LightNode::setDiffuse(sg_RGBA const& rgba)
{
   _diffuse = rgba;
}

void
sg_LightNode::setDiffuse(double r, double g, double b, double a)
{
   setDiffuse(sg_RGBA(r,g,b,a));
}

void
sg_LightNode::setSpecular(sg_RGBA const& rgba)
{
   _specular = rgba;
}

void
sg_LightNode::setSpecular(double r, double g, double b, double a)
{
   setSpecular(sg_RGBA(r,g,b,a));
}

void
sg_LightNode::setConstantAtten(double factor)
{
   _constantAtten = factor;
}

void
sg_LightNode::setLinearAtten(double factor)
{
   _linearAtten = factor;
}

void
sg_LightNode::setQuadraticAtten(double factor)
{
   _quadAtten = factor;
}


// sg_PointLight

sg_PointLight::sg_PointLight(GLenum lightID)
             : sg_LightNode(lightID)
{
   setShape(new sg_PointLightShape);
}

void
sg_PointLight::render()
{
   sg_LightNode::render();
   if (!isOn())
      return;

   GLenum lightID = getLightID();

   GLfloat params[4];

   // Position light.
   sg_Matrix const& m = getTransformation().getMatrix();
   sg_MatrixUtils::transformPoint(m,sg_Point(0,0,0),_pos);

   params[0] = _pos.x;
   params[1] = _pos.y;
   params[2] = _pos.z;
   params[3] = 1.0;
   glLightfv(lightID, GL_POSITION, params);

   // Calculate its direction.
   sg_Point target;
   sg_MatrixUtils::transformPoint(m,sg_Point(0,1,0),target);
   _dir = sg_Vector(target.x - _pos.x, target.y - _pos.y, target.z - _pos.z);
   _dir.normalize();
   
   double i,j,k;
   _dir.getComponents(i,j,k);
   params[0] = i;
   params[1] = j;
   params[2] = k;
   glLightfv(lightID, GL_SPOT_DIRECTION, params);

   // We're a special case of a spot light. Our cone angle is always 180.0 degrees.
   glLightf(lightID, GL_SPOT_CUTOFF, 180.0);

   // Draw a wireframe version of the spotlight.
   if (_showIcon && (_icon != 0))
   {
      glPushMatrix();
      glMultMatrixd(m.getArray());
      _icon->render();
      glPopMatrix();
   }

}


// sg_SpotLight

sg_SpotLight::sg_SpotLight(GLenum lightID)
            : sg_LightNode(lightID),
              _exponent(0.0),
              _cutoffAngle(45.0)
{
   setShape(new sg_SpotLightShape());
}

void
sg_SpotLight::render()
{
   sg_LightNode::render();
   if (!isOn())
      return;

   GLenum lightID = getLightID();

   GLfloat params[4];

   // Position light.
   sg_Matrix const& m = getTransformation().getMatrix();
   sg_MatrixUtils::transformPoint(m,sg_Point(0,0,0),_pos);

   params[0] = _pos.x;
   params[1] = _pos.y;
   params[2] = _pos.z;
   params[3] = 1.0;
   glLightfv(lightID, GL_POSITION, params);

   // Calculate its direction.
   sg_Point target;
   sg_MatrixUtils::transformPoint(m,sg_Point(0,1,0),target);
   _dir = sg_Vector(target.x - _pos.x, target.y - _pos.y, target.z - _pos.z);
   
   double i,j,k;
   _dir.getComponents(i,j,k);
   params[0] = i;
   params[1] = j;
   params[2] = k;
   glLightfv(lightID, GL_SPOT_DIRECTION, params);

   glLightf(lightID, GL_SPOT_EXPONENT, _exponent);
   glLightf(lightID, GL_SPOT_CUTOFF, _cutoffAngle);

   // Draw an icon representing the light.
   if (_showIcon && (_icon != 0))
   {
      glPushMatrix();
      glMultMatrixd(m.getArray());
      _icon->render();
      glPopMatrix();
   }

}

void
sg_SpotLight::setExponent(double exponent)
{
   _exponent = exponent;
}

void
sg_SpotLight::setCutoffAngle(double cutoffAngle)
{
   _cutoffAngle = cutoffAngle;
}

// sg_DistantLight

sg_DistantLight::sg_DistantLight(GLenum lightID)
               : sg_LightNode(lightID)
{
}

void
sg_DistantLight::render()
{
   sg_LightNode::render();
   if (!isOn())
      return;

   GLenum lightID = getLightID();

   // Calculate its direction.
   sg_Matrix const& m = getTransformation().getMatrix();
   sg_MatrixUtils::transformPoint(m,sg_Point(0,0,0),_pos);
   sg_Point target;
   sg_MatrixUtils::transformPoint(m,sg_Point(0,1,0),target);
   _dir = sg_Vector(target.x - _pos.x, target.y - _pos.y, target.z - _pos.z);
   _dir.normalize();


   // In OpenGL, a directional light is a light whose GL_POSITION 'w' coordinate 
   // is zero. The (x,y,z) components then specify the direction the light is
   // coming from.

   sg_Point temp_pos;
   sg_Vector temp_vec = _dir * -1.0;
   temp_vec.getComponents(temp_pos.x, temp_pos.y, temp_pos.z);

   GLfloat params[4];   
   params[0] = temp_pos.x;
   params[1] = temp_pos.y;
   params[2] = temp_pos.z;
   params[3] = 0.0;      // Directional light
   glLightfv(lightID, GL_POSITION, params);
   glLightf(lightID, GL_SPOT_CUTOFF, 180.0);

   // Draw an icon representing the spotlight.
   if (_showIcon && (_icon != 0))
   {
      glPushMatrix();
      glMultMatrixd(m.getArray());
      _icon->render();
      glPopMatrix();
   }
}

// sg_ManipulatorNode

void
sg_ManipulatorNode::accept(sg_SceneVisitor& visitor)
{
   visitor.visitManipulatorNode(this);
}


// sg_CameraNode

sg_CameraNode::sg_CameraNode()
             : _eye(0.0,0.0,0.0),       // At the origin
               _point(0.0,1.0,0.0),     // Looking at (0,1,0)
               _up(0.0,0.0,1.0),        // Up is toward Z+
               _look(0.0,1.0,0.0),      // Looking toward Y+
               _right(1.0,0.0,0.0),     // +X is to our right
               _fieldOfView(45.0),
               _frontClip(1.0),
               _backClip(0.0),
               _vpX(0), _vpY(0),
               _vpWidth(100), _vpHeight(100)
{
}

void
sg_CameraNode::render()
{
   // XXX Find a way to traverse the scene graph so that we don't have to
   // pop the stack here.

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(_fieldOfView, _aspect, _frontClip, _backClip);
   glViewport(_vpX, _vpY, _vpWidth, _vpHeight);

   sg_Matrix const& m = getTransformation().getMatrix();

   // Figure out where the eye is now.
   sg_Point ep;
   sg_MatrixUtils::transformPoint(m,sg_Point(0,0,0),ep);
   
   // Compute a point to look at.
   sg_Point cp;
   sg_MatrixUtils::transformPoint(m,sg_Point(0,1,0),cp);

   // Find an up vector.
   sg_Point uv;
   sg_MatrixUtils::transformPoint(m,sg_Point(0,0,1),uv);

   glMatrixMode(GL_MODELVIEW);
   glPopMatrix();
   glLoadIdentity();
   gluLookAt(ep.x, ep.y, ep.z,
             cp.x, cp.y, cp.z,
             uv.x - ep.x, uv.y - ep.y, uv.z - ep.z);

   glPushMatrix();
}

void
sg_CameraNode::accept(sg_SceneVisitor& visitor)
{
   visitor.visitCameraNode(this);
}

void
sg_CameraNode::look(sg_Point const& eye,
                    sg_Point const& point,
                    sg_Vector const& up)
{

   // Remember... This camera looks toward Y+ axis, up is Z+ and right is X+

   _up = up;
   _look = sg_Vector(point.x - eye.x, point.y - eye.y, point.z - eye.z);

   sg_Matrix am;
   sg_MatrixUtils::calcCameraMatrix(_up, _look, eye, _right, am);
   getTransformation().setMatrix(am);
}

void
sg_CameraNode::look(double eye_x, double eye_y, double eye_z,
                    double px, double py, double pz,
                    double up_i, double up_j, double up_k)
{
   look(sg_Point(eye_x, eye_y, eye_z),
        sg_Point(px, py, pz),
        sg_Vector(up_i, up_j, up_k));
}

void
sg_CameraNode::setFieldOfView(double degrees)
{
   _fieldOfView = degrees;
}

void
sg_CameraNode::setAspect(double ratio)
{
   _aspect = ratio;
}

void
sg_CameraNode::setFrontClipPlane(double dist)
{
   _frontClip = dist;
}


void
sg_CameraNode::setBackClipPlane(double dist)
{
   _backClip = dist;
}

void
sg_CameraNode::setViewPort(long ll_x, long ll_y,
                           long width, long height)
{
   _vpX = ll_x;
   _vpY = ll_y;
   _vpWidth = width;
   _vpHeight = height;
}
      
void
sg_CameraNode::getViewPort(long& ll_x, long& ll_y,
                           long& width, long& height) const
{
   ll_x = _vpX;
   ll_y = _vpY;
   width = _vpWidth;
   height = _vpHeight;
}

double
sg_CameraNode::getFieldOfView() const
{
   return _fieldOfView;
}

double
sg_CameraNode::getAspect() const
{
   return _aspect;
}

void
sg_CameraNode::getUpVector(sg_Vector& up) const
{
   up = _up;
}

void
sg_CameraNode::getLookVector(sg_Vector& look) const
{
   look = _look;
}

void
sg_CameraNode::getRightVector(sg_Vector& right) const
{
   right = _right;
}
