#ifndef INCLUDED_SG_SCENEVISITOR_HXX
#define INCLUDED_SG_SCENEVISITOR_HXX

#ifndef INCLUDED_SG_TRANSFORMATION
#include "scenegraph/sg_Transformation.hxx"
#endif

class sg_SceneNode;
class sg_ShapeNode;
class sg_CompositeNode;
class sg_LightNode;
class sg_ManipulatorNode;
class sg_CameraNode;

class sg_SceneVisitor
{
   public:

      virtual void visitShapeNode(sg_ShapeNode* node);
      virtual void visitCompositeNode(sg_CompositeNode* node);
      virtual void visitLightNode(sg_LightNode* node);
      virtual void visitManipulatorNode(sg_ManipulatorNode* node);
      virtual void visitCameraNode(sg_CameraNode* node);

   protected:

      sg_SceneVisitor();
};

class sg_TreePrintVisitor : public sg_SceneVisitor
{
   // Mostly for testing. Prints out a scene graph.

   public:
 
      sg_TreePrintVisitor();

      void visitShapeNode(sg_ShapeNode* node);
      void visitCompositeNode(sg_CompositeNode* node);
      void visitLightNode(sg_LightNode* node);
      void visitManipulatorNode(sg_ManipulatorNode* node);

};


class sg_RenderVisitor : public sg_SceneVisitor
{
   // Renders a scene graph. Should be used with an sg_RenderIter.

   public:
	   bool finishedWithNode();
 
      sg_RenderVisitor();

      void visitShapeNode(sg_ShapeNode* node);
      void visitCompositeNode(sg_CompositeNode* node);
      void visitLightNode(sg_LightNode* node);
      void visitManipulatorNode(sg_ManipulatorNode* node);
      void visitCameraNode(sg_CameraNode* node);
      
private:
	sg_ShapeNode* _prevShapeNode;
	long _prevStep;
	bool _finished;
};

class sg_LoResRenderVisitor : public sg_SceneVisitor
{
   // Draws the "Lo Res" version of a scene graph.

   public:
 
      sg_LoResRenderVisitor();

      void visitShapeNode(sg_ShapeNode* node);
      void visitCompositeNode(sg_CompositeNode* node);
      void visitLightNode(sg_LightNode* node);
      void visitManipulatorNode(sg_ManipulatorNode* node);
      void visitCameraNode(sg_CameraNode* node);
};

class sg_BoundingBoxVisitor : public sg_SceneVisitor
{
   // Calculates the bounding box of all sg_ShapeNodes in a  scene graph.

   public:
 
      sg_BoundingBoxVisitor();

      void visitShapeNode(sg_ShapeNode* node);
      void visitCompositeNode(sg_CompositeNode* node);
      void visitLightNode(sg_LightNode* node);
      void visitManipulatorNode(sg_ManipulatorNode* node);

      void getBoundingBox(double& xmin, double& ymin, double& zmin,
                          double& xmax, double& ymax, double& zmax);
         // Default (0,0,0),(0,0,0).

   private:

      double _xmin, _ymin, _zmin;
      double _xmax, _ymax, _zmax;
      sg_Transformation _mat;
};

class sg_StepCountVisitor : public sg_SceneVisitor
{
   // Adds up the render steps required for each Shape in the scene.

   public:
	   void visitCameraNode(sg_CameraNode* node);
 
      sg_StepCountVisitor();

      void visitShapeNode(sg_ShapeNode* node);
      void visitCompositeNode(sg_CompositeNode* node);
      void visitLightNode(sg_LightNode* node);
      void visitManipulatorNode(sg_ManipulatorNode* node);

      long getTotalSteps();

   private:
      long _stepCount;
};

#endif  // INCLUDED_SG_SCENEVISITOR_HXX
