#pragma warning(once:4305)

// $Id: sg_Shape.cxx,v 1.10 1997/02/05 20:18:26 moyers Exp $

#include <math.h>

#ifdef WIN32
#include <afxwin.h>
#include <afxext.h>
#include <afxtempl.h>
#include <algorithm>
#define M_PI            3.14159265358979323846
#define M_PI_2          1.57079632679489661923
using namespace std;
#else
#include <algo.h>
#endif

#include "gl/gl.h"
#include "gl/glu.h"

#ifndef INCLUDED_SG_SHAPE_HXX
#include "scenegraph/sg_Shape.hxx"
#endif

#ifndef INCLUDED_SG_TYPES_HXX
#include "scenegraph/sg_Types.hxx"
#endif

#ifndef INCLUDED_SG_VECTOR_HXX
#include "scenegraph/sg_Vector.hxx"
#endif

#ifndef INCLUDED_SG_COLORS_HXX
#include "scenegraph/sg_Colors.hxx"
#endif

sg_Shape::sg_Shape()
        : _min_x(0.0), _min_y(0.0), _min_z(0.0),
          _max_x(0.0), _max_y(0.0), _max_z(0.0),
          _maxRenderSteps(1),
          _curRenderSteps(1)
{
}

sg_Shape::~sg_Shape()
{
}

long
sg_Shape::render(long previousStep)
{
   return 0;
}

void
sg_Shape::renderLoRes()
{
   // Draw the bounding box.

   double min_x, min_y, min_z;
   double max_x, max_y, max_z;

   getBoundingBox(min_x, min_y, min_z,
                  max_x, max_y, max_z);

   glBegin(GL_LINE_STRIP);
      glVertex3d(min_x, min_y, max_z);
      glVertex3d(max_x, min_y, max_z);
      glVertex3d(max_x, max_y, max_z);
      glVertex3d(min_x, max_y, max_z);
      glVertex3d(min_x, min_y, max_z);

      glVertex3d(min_x, min_y, min_z);
      glVertex3d(max_x, min_y, min_z);
      glVertex3d(max_x, max_y, min_z);
      glVertex3d(min_x, max_y, min_z);
      glVertex3d(min_x, min_y, min_z);
   glEnd();

   glBegin(GL_LINES);
      glVertex3d(max_x, min_y, min_z);
      glVertex3d(max_x, min_y, max_z);

      glVertex3d(max_x, max_y, min_z);
      glVertex3d(max_x, max_y, max_z);

      glVertex3d(min_x, max_y, min_z);
      glVertex3d(min_x, max_y, max_z);
   glEnd();
}

long
sg_Shape::getCurrentRenderSteps()
{
   return _curRenderSteps;
}

long
sg_Shape::getMaxRenderSteps()
{
   return _maxRenderSteps;
}

void
sg_Shape::setCurrentRenderSteps(long steps)
{
   if ((steps < 1) || (steps > _maxRenderSteps))
      return;
   _curRenderSteps = steps;
}

void
sg_Shape::getBoundingBox(double& min_x, double& min_y, double& min_z,
                         double& max_x, double& max_y, double& max_z) const
{
   // Just returns the precomputed bounding box. Subclasses can override if
   // they'd prefer to compute the bounding box.

   min_x = _min_x;
   min_y = _min_y;
   min_z = _min_z;
   max_x = _max_x;
   max_y = _max_y;
   max_z = _max_z;
}

void
sg_Shape::getBoundingBox(sg_MinBoundBox& mbb) const
{
   mbb.min_x = _min_x;
   mbb.max_x = _max_x;
   mbb.min_y = _min_y;
   mbb.max_y = _max_y;
   mbb.min_z = _min_z;
   mbb.max_z = _max_z;
}

// sg_Text

sg_Text::sg_Text(long listbase, 
                 const string& message)
       : _lb(listbase), _mess(message)
{
}

long
sg_Text::render(long previousStep)
{
   glEnable(GL_NORMALIZE);  // Scaling text messes up the normals.
   glListBase(_lb);
   glCallLists(_mess.length(), GL_UNSIGNED_BYTE, _mess.c_str() );
   glDisable(GL_NORMALIZE);
   return 0;
}

// sg_Cylinder

sg_Cylinder::sg_Cylinder(double length, double radius, double faces)
         : _length(length), _radius(radius), _faces(faces)
{
   _myQuad = gluNewQuadric();
   gluQuadricDrawStyle(_myQuad, GLU_FILL);
}

sg_Cylinder::~sg_Cylinder()
{
   gluDeleteQuadric(_myQuad);
}

long
sg_Cylinder::render(long previousStep)
{
   gluCylinder(_myQuad, _radius, _radius, _length, _faces, _faces);
   return 0;
}

// sg_Cone

sg_Cone::sg_Cone(double length, double radius, double faces)
         : _length(length), _radius(radius), _faces(faces)
{
   _myQuad = gluNewQuadric();
   gluQuadricDrawStyle(_myQuad, GLU_FILL);
     // may want to change so second parameter can be specified
}

sg_Cone::~sg_Cone()
{
   gluDeleteQuadric(_myQuad);
}

long
sg_Cone::render(long previousStep)
{
   gluCylinder(_myQuad, _radius, 0, _length, _faces, _faces);
   return 0;
}

// sg_Cube

sg_Cube::sg_Cube(double edgeLength)
       : _edgeLength(edgeLength)
{
   // Set up the bounding box.
   float a = _edgeLength / 2.0;
   float na = -1.0 * a;

   _min_x = na;
   _min_y = na;
   _min_z = na;
   _max_x = a;
   _max_y = a;
   _max_z = a;
}

long
sg_Cube::render(long previousStep)
{
   float a = _edgeLength / 2.0;
   float na = -1.0 * a;

   // All faces drawn CCW around normal.

   glBegin(GL_QUADS);
      // +Z face
      glNormal3f(0.0, 0.0, 1.0);
      glVertex3f(a, a, a);
      glVertex3f(na, a, a);
      glVertex3f(na, na, a);
      glVertex3f(a, na, a);

      // +X face
      glNormal3f(1.0, 0.0, 0.0);
      glVertex3f(a, a, a);
      glVertex3f(a, na, a);
      glVertex3f(a, na , na);
      glVertex3f(a, a, na);

      // +Y face
      glNormal3f(0.0, 1.0, 0.0);
      glVertex3f(na, a, a);
      glVertex3f(a, a, a);
      glVertex3f(a, a, na);
      glVertex3f(na, a, na);

      // -X face
      glNormal3f(-1.0, 0.0, 0.0);
      glVertex3f(na, na, a);
      glVertex3f(na, a, a);
      glVertex3f(na, a, na);
      glVertex3f(na, na, na);

      // -Y face
      glNormal3f(0.0, -1.0, 0.0);
      glVertex3f(a, na, a);
      glVertex3f(na, na, a);
      glVertex3f(na, na, na);
      glVertex3f(a, na, na);

      // -Z face
      glNormal3f(0.0, 0.0, -1.0);
      glVertex3f(a, na, na);
      glVertex3f(na, na, na);
      glVertex3f(na, a, na);
      glVertex3f(a, a, na);
   glEnd();

   return 0;
}

sg_Grid::sg_Grid()
{
	_width = 20000;
	_height = 20000;
	_xsegs = 20;
	_ysegs = 20;
}

void
sg_Grid::setWidth(double width)
{
	_width = fabs(width);
}

void
sg_Grid::setHeight(double height)
{
	_height = fabs(height);
}

void
sg_Grid::setXsegs(double xsegs)
{
   if (xsegs < 1)
      _xsegs = 1;
   else 
	  _xsegs = xsegs;
}

void
sg_Grid::setYsegs(double ysegs)
{
   if (ysegs < 1)
      _ysegs = 1;
   else 
	  _ysegs = ysegs;
}

long
sg_Grid::render(long previousStep)
{
   double xstart = -1.0 * (_width / 2.0);
   double ystart = -1.0 * (_height / 2.0);
   double dx = _width / _xsegs;
   double dy = _height / _ysegs;

   long i;  // Phreaking Visual C++ 4.2 *still* doesn't do new scoping rules....

   glNormal3f(0.0, 0.0, 1.0);

   // Draw horizontal lines
   glBegin(GL_LINES);
   for (i = 0; i < _ysegs + 1; i++)
      {
         glVertex3d(xstart, ystart + (i * dy), 0.0);
         glVertex3d(xstart + _width, ystart + (i * dy), 0.0);
      }
   glEnd();

   // Draw vertical lines
   glBegin(GL_LINES);
   for (i = 0; i < _xsegs + 1; i++)
      {
         glVertex3d(xstart + (i * dx), ystart, 0.0);
         glVertex3d(xstart + (i * dx), ystart + _height, 0.0);
      }
   glEnd();

   return 0;
}

void
sg_Grid::renderLoRes()
{
   render();
}

// sg_Box

sg_Box::sg_Box(sg_Point const& a,
               sg_Point const& b,
               long x_segs,
               long y_segs,
               long z_segs)
      : _a(a),
        _b(b),
        _x_segs(x_segs),
        _y_segs(y_segs),
        _z_segs(z_segs)
{
   // Reorder a and b so that a is "less than" b.

   if (_a.x > _b.x)
      swap(_a.x, _b.x);

   if (_a.y > _b.y)
      swap(_a.y, _b.y);

   if (_a.z > _b.z)
      swap(_a.z, _b.z);
}

void
sg_Box::drawFace(double umin, double umax, double usegs,
                 double vmin, double vmax, double vsegs,
                 sg_Vector const& origin,
                 sg_Vector const& u_axis,
                 sg_Vector const& v_axis)
{
   sg_Vector du = u_axis * ((umax - umin) / usegs);
   sg_Vector dv = v_axis * ((vmax - vmin) / vsegs);

   sg_Vector p1,p2,p3,p4;
   double x,y,z;
   for (long j = 0; j < vsegs; j++)
      {
         glBegin(GL_QUAD_STRIP);
         for (long i = 0; i < usegs + 1; i++)
            {
               p1 = origin + (du * i) + (dv * j);
               p2 = origin + (du * i) + (dv * (j + 1));

               p2.getComponents(x,y,z); glVertex3f(x,y,z);
               p1.getComponents(x,y,z); glVertex3f(x,y,z);

            }
         glEnd();
      }
}


long
sg_Box::render(long previousStep)
{

   double dx = _b.x - _a.x;
   double dy = _b.y - _a.y;
   double dz = _b.z - _a.z;

   // +Z face
   glNormal3f(0,0,1);
   drawFace(0, dx, _x_segs,
            0, dy, _y_segs,
            sg_Vector(_a.x, _a.y, _b.z),
            sg_Vector(1,0,0),
            sg_Vector(0,1,0));

   // +X face
   glNormal3f(1,0,0);
   drawFace(0, dy, _y_segs,
            0, dz, _z_segs,
            sg_Vector(_b.x, _a.y, _a.z),
            sg_Vector(0,1,0),
            sg_Vector(0,0,1));

   // +Y face
   glNormal3f(0,1,0);
   drawFace(0, dx, _x_segs,
            0, dz, _z_segs,
            sg_Vector(_b.x, _b.y, _a.z),
            sg_Vector(-1,0,0),
            sg_Vector(0,0,1));

   // -X face
   glNormal3f(-1,0,0);
   drawFace(0, dy, _y_segs,
            0, dz, _z_segs,
            sg_Vector(_a.x, _b.y, _a.z),
            sg_Vector(0,-1,0),
            sg_Vector(0,0,1));

   // -Y face
   glNormal3f(0,-1,0);
   drawFace(0, dx, _x_segs,
            0, dz, _z_segs,
            sg_Vector(_a.x, _a.y, _a.z),
            sg_Vector(1,0,0),
            sg_Vector(0,0,1));

   // -Z face
   glNormal3f(0,0,-1);
   drawFace(0, dx, _x_segs,
            0, dy, _y_segs,
            sg_Vector(_a.x, _b.y, _a.z),
            sg_Vector(1,0,0),
            sg_Vector(0,-1,0));

   return 0;
}


// sg_PointLightShape

sg_PointLightShape::sg_PointLightShape()
{
}

long
sg_PointLightShape::render(long previousStep)
{
   glPushAttrib(GL_ALL_ATTRIB_BITS);
   glDisable(GL_LIGHTING);

   //used to scale up
   glPushMatrix();
   glScalef(500.0,500.0,500.0);

   glColor3f(0.0, 1.0, 1.0);
   glBegin(GL_LINES);
      glVertex3f(0, 0, 0.3);
      glVertex3f(0, 0, -0.3);

      glVertex3f(0.3, 0, 0);
      glVertex3f(-0.3, 0, 0);

      glVertex3f(0, 0.3, 0);
      glVertex3f(0, -0.3, 0);

      glVertex3f(0.3, 0.3, 0.3);
      glVertex3f(-0.3, -0.3, -0.3);

      glVertex3f(-0.3, 0.3, 0.3);
      glVertex3f(0.3, -0.3, -0.3);

      glVertex3f(-0.3, -0.3, 0.3);
      glVertex3f(0.3, 0.3, -0.3);

      glVertex3f(0.3, -0.3, 0.3);
      glVertex3f(-0.3, 0.3, -0.3);
   glEnd();
   glPopMatrix();

   glEnable(GL_LIGHTING);
   glPopAttrib();

   return 0;
}


// sg_SpotLightShape

sg_SpotLightShape::sg_SpotLightShape()
{
}

long
sg_SpotLightShape::render(long previousStep)
{
   glPushAttrib(GL_ALL_ATTRIB_BITS);
   glDisable(GL_LIGHTING);

   glColor3f(0.0, 1.0, 1.0);

   // Camera body

   // Top
   glBegin(GL_LINE_STRIP);
      glVertex3f(0.3, 0.5, 0.3);
      glVertex3f(-0.3, 0.5, 0.3);
      glVertex3f(-0.3, -0.5, 0.3);
      glVertex3f(0.3, -0.5, 0.3);
      glVertex3f(0.3, 0.5, 0.3);
   glEnd();

   // Bottom
   glBegin(GL_LINE_STRIP);
      glVertex3f(0.3, 0.5, -0.3);
      glVertex3f(-0.3, 0.5, -0.3);
      glVertex3f(-0.3, -0.5, -0.3);
      glVertex3f(0.3, -0.5, -0.3);
      glVertex3f(0.3, 0.5, -0.3);
   glEnd();

   // Sides
   glBegin(GL_LINES);
      glVertex3f(0.3, 0.5, 0.3);
      glVertex3f(0.3, 0.5, -0.3);

      glVertex3f(-0.3, 0.5, 0.3);
      glVertex3f(-0.3, 0.5, -0.3);

      glVertex3f(-0.3, -0.5, 0.3);
      glVertex3f(-0.3, -0.5, -0.3);

      glVertex3f(0.3, -0.5, 0.3);
      glVertex3f(0.3, -0.5, -0.3);
   glEnd();


   // Barn Doors
   glBegin(GL_LINE_STRIP);
      glVertex3f(0.3, 0.5, 0.3);
      glVertex3f(0.55, 0.75, 0.3);
      glVertex3f(0.55, 0.75, -0.3);
      glVertex3f(0.3, 0.5, -0.3);
   glEnd();

   glBegin(GL_LINE_STRIP);
      glVertex3f(-0.3, 0.5, 0.3);
      glVertex3f(-0.55, 0.75, 0.3);
      glVertex3f(-0.55, 0.75, -0.3);
      glVertex3f(-0.3, 0.5, -0.3);
   glEnd();


   glEnable(GL_LIGHTING);
   glPopAttrib();  

   return 0;
}


sg_DistantLightShape::sg_DistantLightShape()
{
}

void
sg_DistantLightShape::drawArrow()
{
	glColor3f(1.0,1.0,1.0);
   glBegin(GL_LINES);
      glVertex3f(0.0, -0.3, 0.0);
      glVertex3f(0.0, 0.3, 0.0);
 
      glVertex3f(0.0, 0.3, 0.0);
      glVertex3f(0.1, 0.1, 0.0);

      glVertex3f(0.0, 0.3, 0.0);
      glVertex3f(-0.1, 0.1, 0.0);
   glEnd();
}

long
sg_DistantLightShape::render(long previousStep)
{
   glPushAttrib(GL_ALL_ATTRIB_BITS);
   glDisable(GL_LIGHTING);

   // Four arrows pointing along the +Y axis.
   // added scaling

   glPushMatrix();
   glScalef(500.0,500.0,500.0);
   glTranslatef(0.15, 0.0, 0.15);
   drawArrow();
   glPopMatrix();

   glPushMatrix();
   glScalef(500.0,500.0,500.0);
   glTranslatef(-0.15, 0.0, 0.15);
   drawArrow();
   glPopMatrix();

   glPushMatrix();
   glScalef(500.0,500.0,500.0);
   glTranslatef(-0.15, 0.0, -0.15);
   drawArrow();
   glPopMatrix();

   glPushMatrix();
   glScalef(500.0,500.0,500.0);
   glTranslatef(0.15, 0.0, -0.15);
   drawArrow();
   glPopMatrix();

   glEnable(GL_LIGHTING);
   glPopAttrib();

   return 0;
}


sg_Axis::sg_Axis(float length, float radius, float faces, long lb)
         : _length(length),       // TOTAL length of an axis bar
           _radius(radius),       // radius of an axis bar
           _faces(faces),         // number of faces of an axis bar
           _lb(lb)               // listbase of font
{
   _myQuad = gluNewQuadric();
   gluQuadricDrawStyle(_myQuad, GLU_FILL);
   _z = "Z";
   _y = "Y";
   _x = "X";
}

sg_Axis::~sg_Axis()
{
   gluDeleteQuadric(_myQuad);
}

long
sg_Axis::render(long previousStep)
{
// Z axis cylinder
   glPushMatrix();
   glTranslatef(0.0,0.0,-(_length/2));
   gluCylinder(_myQuad, _radius, _radius, _length, _faces, _faces);
   glPopMatrix();
// Z axis cone
   glPushMatrix();
   glTranslatef(0.0,0.0,_length/2);
   gluCylinder(_myQuad, _radius+(_radius*0.75), 0, 0.05*_length, _faces, _faces);
   glPopMatrix();
// Z axis text
   glPushMatrix();
   glRotatef(90,1.0,0.0,0.0);
   glTranslatef(0.0,(_length/2)+(0.08*_length),0.0);
   glTranslatef(-0.3,0.0,0.0);
   glListBase(_lb);
   glCallLists(_z.length(), GL_UNSIGNED_BYTE, _z.c_str() );
   glPopMatrix();

// Y axis cylinder
   glPushMatrix();
   glRotatef(270,1.0,0.0,0.0);
   glTranslatef(0.0,0.0,-(_length/2));
   gluCylinder(_myQuad, _radius, _radius, _length, _faces, _faces);
   glPopMatrix();
// Y axis cone
   glPushMatrix();
   glRotatef(270,1.0,0.0,0.0);
   glTranslatef(0.0,0.0,_length/2);
   gluCylinder(_myQuad, _radius+(_radius*0.75), 0, 0.05*_length, _faces, _faces);
   glPopMatrix();
// Y axis text
   glPushMatrix();
   glRotatef(90,1.0,0.0,0.0);
   glTranslatef(0.0,0.0,-((_length/2)+(0.08*_length)));
   glTranslatef(-0.3,0.0,0.0);
   glListBase(_lb);
   glCallLists(_y.length(), GL_UNSIGNED_BYTE, _y.c_str() );
   glPopMatrix();

// X axis cylinder
   glPushMatrix();
   glRotatef(90,0.0,1.0,0.0);
   glTranslatef(0.0,0.0,-(_length/2));
   gluCylinder(_myQuad, _radius, _radius, _length, _faces, _faces);
   glPopMatrix();
// X axis cone
   glPushMatrix();
   glRotatef(90,0.0,1.0,0.0);
   glTranslatef(0.0,0.0,_length/2);
   gluCylinder(_myQuad, _radius+(_radius*0.75), 0, 0.05*_length, _faces, _faces);
   glPopMatrix();
// X axis text
   glPushMatrix();
   glRotatef(90,1.0,0.0,0.0);
   glTranslatef((_length/2)+(0.08*_length),0.0,0.0);
   glListBase(_lb);
   glCallLists(_x.length(), GL_UNSIGNED_BYTE, _x.c_str() );
   glPopMatrix();

   return 0;
}

sg_ColorCube::sg_ColorCube(double edgeLength)
{
   _edge = edgeLength;
}

long
sg_ColorCube::render(long previousStep)
{
   float a = _edge / 2.0;
   float na = -1.0 * a;

   // All faces drawn CCW around normal.

   glShadeModel(GL_SMOOTH);

   glBegin(GL_QUADS);


      // +Z face
      glNormal3f(0.0, 0.0, 1.0);
      glColor3d(1.0, 1.0, 1.0);
      glVertex3f(a, a, a);
      glColor3d(0.0, 1.0, 1.0);
      glVertex3f(na, a, a);
      glColor3d(0.0, 0.0, 1.0);
      glVertex3f(na, na, a);
      glColor3d(1.0, 0.0, 1.0);
      glVertex3f(a, na, a);


      // +X face
      glNormal3f(1.0, 0.0, 0.0);
      glColor3d(1.0, 1.0, 1.0);
      glVertex3f(a, a, a);
      glColor3d(1.0, 0.0, 1.0);
      glVertex3f(a, na, a);
      glColor3d(1.0, 0.0, 0.0);
      glVertex3f(a, na , na);
      glColor3d(1.0, 1.0, 0.0);
      glVertex3f(a, a, na);

      // +Y face
      glNormal3f(0.0, 1.0, 0.0);
      glColor3d(0.0, 1.0, 1.0);
      glVertex3f(na, a, a);
      glColor3d(1.0, 1.0, 1.0);
      glVertex3f(a, a, a);
      glColor3d(1.0, 1.0, 0.0);
      glVertex3f(a, a, na);
      glColor3d(0.0, 1.0, 0.0);
      glVertex3f(na, a, na);

      // -X face
      glNormal3f(-1.0, 0.0, 0.0);
      glColor3d(0.0, 0.0, 1.0);
      glVertex3f(na, na, a);
      glColor3d(0.0, 1.0, 1.0);
      glVertex3f(na, a, a);
      glColor3d(0.0, 1.0, 0.0);
      glVertex3f(na, a, na);
      glColor3d(0.0, 0.0, 0.0);
      glVertex3f(na, na, na);

      // -Y face
      glNormal3f(0.0, -1.0, 0.0);
      glColor3d(1.0, 0.0, 1.0);
      glVertex3f(a, na, a);
      glColor3d(0.0, 0.0, 1.0);
      glVertex3f(na, na, a);
      glColor3d(0.0, 0.0, 0.0);
      glVertex3f(na, na, na);
      glColor3d(1.0, 0.0, 0.0);
      glVertex3f(a, na, na);

      // -Z face
      glNormal3f(0.0, 0.0, -1.0);
      glColor3d(1.0, 0.0, 0.0);
      glVertex3f(a, na, na);
      glColor3d(0.0, 0.0, 0.0);
      glVertex3f(na, na, na);
      glColor3d(0.0, 1.0, 0.0);
      glVertex3f(na, a, na);
      glColor3d(1.0, 1.0, 0.0);
      glVertex3f(a, a, na);
   glEnd();

   return 0;
}


sg_WireSphere::sg_WireSphere(double radius,
                             long numMeridians,
                             long numSlices)
             : _radius(radius),
               _numMeridians(numMeridians),
               _numMeridianSteps(10),
               _numSlices(numSlices),
               _numSliceSteps(20)
{
}

static
inline
void
sphere2cart(double r, double theta, double phi,
            double& x, double& y, double&z)
{
   double r1 = r * cos(phi);
   x = r1 * cos(theta);
   y = r1 * sin(theta);
   z = r * sin(phi);
}

long
sg_WireSphere::render(long previousStep)
{
   if (_numMeridians > 0)
   {
      double x,y,z;
      double dTheta = 2.0 * M_PI / _numMeridians;
      double dPhi = M_PI / _numMeridianSteps;
      for (long i = 0; i < _numMeridians; i++)
      {
         glBegin(GL_LINE_STRIP);
         for (long j = 0; j < _numMeridianSteps + 1; j++)
         {
            sphere2cart(_radius, dTheta * i, M_PI_2 - (j * dPhi),
                        x,y,z);
            glVertex3d(x,y,z);
         }
         glEnd();
      }
   }

   if (_numSlices > 0)
   {
      double dL = (2.0 * _radius) / (_numSlices + 1);
      double dTheta = 2.0 * M_PI / _numSliceSteps;
      double r,x,y,z;
      for (long i = 0; i < _numSlices; i++)
      {
         z = (-1.0 * _radius) + ((i + 1) * dL);
         r = _radius * sin(acos(z / _radius));

         glBegin(GL_LINE_LOOP);
         for (long j = 0; j < _numSliceSteps; j++)
         {
            x = r * cos(dTheta * j);
            y = r * sin(dTheta * j);
            glVertex3d(x,y,z);
         }
         glEnd();
      }
   }
   return 0;
}

void
sg_WireSphere::setRadius(double radius)
{
   if (radius >= 0.0)
      _radius = radius;
}

void
sg_WireSphere::setMeridians(long numMeridians)
{
   if (numMeridians >= 0)
      _numMeridians = numMeridians;
}

void
sg_WireSphere::setNumSlices(long numSlices)
{
   if (numSlices >= 0)
      _numSlices = numSlices;
}


sg_AltAzIndicator::sg_AltAzIndicator(double radius)
                 : _radius(radius),
                   _alt(0.0),
                   _azim(0.0),
                   _altSteps(30),
                   _azimSteps(30),
                   _azimColor(sg_Color_Cyan),
                   _altColor(sg_Color_Green)
{
}

void
sg_AltAzIndicator::azimTickMarks(double angleIncr,
                                 double tickMarkLen)
{
   double dTheta = 2.0 * M_PI / (360.0 / angleIncr);
   double theta = 0.0;
   double x,y;
   double tickStart = (1.0 - tickMarkLen) * _radius;
   glBegin(GL_LINES);
   while (theta < 360.0)
   {
      x = cos(theta);
      y = sin(theta);
      glVertex3d(tickStart * x, tickStart * y, 0.0);
      glVertex3d(_radius * x, _radius * y, 0.0);
      theta = theta + dTheta;
   }
   glEnd();
}

void
sg_AltAzIndicator::altTickMarks(double angleIncr,
                                double tickMarkLen)
{
   double dPhi = M_PI / (180.0 / angleIncr);
   double phi = -1.0 * M_PI / 2.0;
   double x,y,z;
   double tickStart = (1.0 - tickMarkLen) * _radius;
   glBegin(GL_LINES);
   while (phi < M_PI / 2.0)
   {
      sphere2cart(1.0, _azim, phi, x, y, z);
      glVertex3d(tickStart * x, tickStart * y, tickStart * z);
      glVertex3d(_radius * x, _radius * y, _radius * z);
      phi = phi + dPhi;
   }
   glEnd();
}                                

long
sg_AltAzIndicator::render(long previousStep)
{
   glLineWidth(2.0);

   // Draw Azimuth ring.
   glColor3f(_azimColor.red, _azimColor.green, _azimColor.blue);
   double dTheta = 2.0 * M_PI / _azimSteps;
   glBegin(GL_LINE_LOOP);
   for (long i = 0; i < _azimSteps; i++)
   {
      glVertex3d(_radius * cos(i * dTheta),
                 _radius * sin(i * dTheta),
                 0.0);
   }
   glEnd();

   // Draw Azimuth ring tick marks.
   azimTickMarks(30.0, 0.05);
   azimTickMarks(15.0,  0.025);

   // Draw Altitude ring
   glColor3f(_altColor.red, _altColor.green, _altColor.blue);
   dTheta = M_PI / _altSteps;
   double x,y,z;
   glBegin(GL_LINE_STRIP);
   double j = -1.0 * M_PI / 2.0;
   while (j < M_PI / 2.0)
   {
      sphere2cart(_radius, _azim, j, x,y,z);
      glVertex3d(x,y,z);
      j = j + dTheta;
   }
   glEnd();

   // Draw Altitude ring tick marks.
   altTickMarks(30.0, 0.05);
   altTickMarks(15.0, 0.025);

   glLineWidth(1.0);

   return 0;
}

void
sg_AltAzIndicator::setRadius(double radius)
{
   if (radius >= 0.0)
      _radius = radius;
}

void
sg_AltAzIndicator::setAltitude(double altitude)
{
   _alt = (altitude * M_PI) / 180.0;
}
 
void
sg_AltAzIndicator::setAzimuth(double azimuth)
{
   _azim = (azimuth * M_PI) / 180.0;
}

void
sg_AltAzIndicator::setAzimColor(sg_RGBA const& color)
{
   _azimColor = color;
}

void
sg_AltAzIndicator::setAltColor(sg_RGBA const& color)
{
   _altColor = color;
}


sg_LightBox::sg_LightBox(double size)
           : _size(size),
             _boxColor(sg_Color_Gray),
             _lightColor(sg_Color_Yellow)
{
}           

long
sg_LightBox::render(long previousStep)
{
   // Draw a box with barn doors representing a light.
   // Centered at the origin, pointing down the +Y axis.

   glEnable(GL_LIGHTING);

   float a = _size / 2.0;
   float na = -1.0 * a;

   // All faces drawn CCW around normal.

   glBegin(GL_QUADS);

      glColor3f(_boxColor.red, _boxColor.green, _boxColor.blue);
      // +Z face
      glNormal3f(0.0, 0.0, 1.0);
      glVertex3f(a, a, a);
      glVertex3f(na, a, a);
      glVertex3f(na, na, a);
      glVertex3f(a, na, a);

      // +X face
      glNormal3f(1.0, 0.0, 0.0);
      glVertex3f(a, a, a);
      glVertex3f(a, na, a);
      glVertex3f(a, na , na);
      glVertex3f(a, a, na);

      // +Y face
      float new_emiss[4];
      _lightColor.getArray(new_emiss);
      glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, new_emiss);

      glNormal3f(0.0, 1.0, 0.0);
      glVertex3f(na, a, a);
      glVertex3f(a, a, a);
      glVertex3f(a, a, na);
      glVertex3f(na, a, na);

      float defaultEmiss[4];
      sg_RGBA noEmiss(0.0, 0.0, 0.0);
      noEmiss.getArray(defaultEmiss);
      glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, defaultEmiss);

      // -X face
      glNormal3f(-1.0, 0.0, 0.0);
      glVertex3f(na, na, a);
      glVertex3f(na, a, a);
      glVertex3f(na, a, na);
      glVertex3f(na, na, na);

      // -Y face
      glNormal3f(0.0, -1.0, 0.0);
      glVertex3f(a, na, a);
      glVertex3f(na, na, a);
      glVertex3f(na, na, na);
      glVertex3f(a, na, na);

      // -Z face
      glNormal3f(0.0, 0.0, -1.0);
      glVertex3f(a, na, na);
      glVertex3f(na, na, na);
      glVertex3f(na, a, na);
      glVertex3f(a, a, na);

      const double t = .70710678118654752440;  // 1 / sqrt(2)
      glNormal3f(t, -1.0 * t, 0);
      glVertex3f(a, a, na);
      glVertex3f(a + _size * t, a + _size * t, na);
      glVertex3f(a + _size * t, a + _size * t, a);
      glVertex3f(a, a, a);

      glNormal3f(-1.0 * t, -1.0 * t, 0);
      glVertex3f(na, a, a);
      glVertex3f(na - _size * t, a + _size * t, a);
      glVertex3f(na - _size * t, a + _size * t, na);
      glVertex3f(na, a, na);

      glNormal3f(0.0, -1.0 * t, t);
      glVertex3f(a, a, a);
      glVertex3f(a, a + _size * t, a + _size * t);
      glVertex3f(na, a + _size * t, a + _size * t);
      glVertex3f(na, a, a);

      glNormal3f(0.0, -1.0 * t, -1.0 * t);
      glVertex3f(a, a, na);
      glVertex3f(na, a, na);
      glVertex3f(na, a + _size * t, na - _size * t);
      glVertex3f(a, a + _size * t, na - _size * t);

   glEnd();

   return 0;
}

void
sg_LightBox::setBoxColor(sg_RGBA const& color)
{
   _boxColor = color;
}

void
sg_LightBox::setLightColor(sg_RGBA const& color)
{
   _lightColor = color;
}





