
// $Id: sg_Utils.cxx,v 1.1 1997/02/07 00:58:54 moyers Exp $

#ifdef WIN32
#define M_PI 3.14159265358979323846
#endif

#include <math.h>

#ifndef INCLUDED_SG_UTILS_HXX
#include "scenegraph/sg_Utils.hxx"
#endif

#ifndef INCLUDED_SG_TYPES_HXX
#include "scenegraph/sg_Types.hxx"
#endif

#ifndef INCLUDED_SG_SCENENODE_HXX
#include "scenegraph/sg_SceneNode.hxx"
#endif

#ifndef INCLUDED_SG_VECTOR_HXX
#include "scenegraph/sg_Vector.hxx"
#endif


void
sg_Utils::calcSpherical(sg_Point const& p,
                        double& radius,
                        double& altitude,
                        double& azimuth)
{
   radius = sqrt(p.x * p.x + p.y * p.y + p.z * p.z);

   azimuth = (asin(p.y / sqrt(p.x * p.x + p.y * p.y)) * 180.0) / M_PI;
   if (p.x > 0.0)
      {
         if (p.y < 0.0)
            azimuth = azimuth + 360.0;
      }
   else
      azimuth = 180.0 - azimuth;

   altitude = (asin(p.z / radius) * 180.0) / M_PI;
}

void
sg_Utils::calcGoodViewPoint(sg_CameraNode const* camera,
                            sg_Vector const& lookvec,
                            sg_MinBoundBox const& mbb,
                            sg_Point& viewPoint)
{
   double fov = camera->getFieldOfView();
   double ar = camera->getAspect();

   // fov measures the horizontal field of view. Calculate the vertical
   // field of view from fov and the aspect ratio (w/h).
   double phi = fov * (1.0 / ar);

   // Use the maximum of the horizontal and vertical field of view to
   // determine how far away to be.
   double lambda;
   if (fov > phi)
      lambda = fov;
   else
      lambda = phi;

   // Find the center of the mbb
   sg_Point center(mbb.min_x + (mbb.max_x - mbb.min_x),
                   mbb.min_y + (mbb.max_y - mbb.min_y),
                   mbb.min_z + (mbb.max_z - mbb.min_z));

   // Find the distance to one corner.
   double R = sqrt(pow(mbb.max_x - mbb.min_x,2.0) +
                   pow(mbb.max_y - mbb.min_y,2.0) +
                   pow(mbb.max_z - mbb.min_z,2.0)) / 2.0;

   double d = R / tan(lambda);
   sg_Vector p(lookvec);
   p.normalize();
   p = p * (-1.0 * d);
 
   p.getComponents(viewPoint.x, viewPoint.y, viewPoint.z);
}