
// $Id: sg_Vector.cxx,v 1.2 1997/01/10 08:07:59 moyers Exp $

#include <math.h>

#ifndef INCLUDED_SG_VECTOR_HXX
#include "scenegraph/sg_Vector.hxx"
#endif

sg_Vector::sg_Vector()
         : _i(0.0), _j(0.0), _k(0.0)
{
}

sg_Vector::sg_Vector(double i, double j, double k)
         : _i(i), _j(j), _k(k)
{
}

sg_Vector::sg_Vector(sg_Vector const& rhs)
{
   operator=(rhs);
}

sg_Vector const&
sg_Vector::operator=(sg_Vector const& rhs)
{
   if (this == &rhs)
      return *this;

   _i = rhs._i;
   _j = rhs._j;
   _k = rhs._k;

   return *this;
}

sg_Vector const
sg_Vector::operator+(sg_Vector const& rhs) const
{
   return sg_Vector(_i + rhs._i,
                    _j + rhs._j,
                    _k + rhs._k);   
}

sg_Vector const
sg_Vector::operator-(sg_Vector const& rhs) const
{
   return sg_Vector(_i - rhs._i,
                    _j - rhs._j,
                    _k - rhs._k);   
}

sg_Vector const
sg_Vector::operator*(double s) const
{
   return sg_Vector(s * _i,
                    s * _j,
                    s * _k);
}

sg_Vector const
sg_Vector::cross(sg_Vector const& rhs) const
{
   return sg_Vector((_j * rhs._k) - (_k * rhs._j),
                    (_k * rhs._i) - (_i * rhs._k),
                    (_i * rhs._j) - (_j * rhs._i));
}

double
sg_Vector::dot(sg_Vector const& rhs) const
{
   return (_i * rhs._i) + (_j * rhs._j) + (_k * rhs._k);
}

void
sg_Vector::normalize()
{
   double mag;

   mag = sqrt(_i * _i + _j * _j + _k * _k);

   // changed to take out slow fabs() call --paul     let me know if I should put it back in
   if (mag < 0)
   {
      mag = -mag;
   }

   if (mag > 0.00000001)
      {
         _i = _i / mag;
         _j = _j / mag;
         _k = _k / mag;
      }
}

void
sg_Vector::getComponents(double& i, double& j, double& k) const
{
   i = _i;
   j = _j;
   k = _k;
}