/*
 * Common function to preform dump/output of builder and container objects
 * for use in testing objects or as diagonstics
 *
 * Chad Slaughter
 * July 1, 1997
 *
 */

#ifdef _MSC_VER
	#include <iostream>
	using namespace std;
#else
	#include <iostream.h>
#endif

// Defines for testing sb objects
#define SB_OBJECT_TEST // includes ForeignID and SpatialAddr
#define SB_IDENT_TEST
#define SB_LINE_TEST
#define SB_TRANSFERSTATS_TEST
#define SB_ARC_TEST
#define SB_INTSPATIALREF_TEST
#define SB_POLYGON_TEST
#define SB_POINTNODE_TEST
#define SB_DATADICTDOM_TEST
#define SB_CATALOGDIR_TEST 
#define SB_EXTSPATIALREF_TEST
#define SB_CATALOGSPATIALDOM_TEST
#define SB_CATALOGCROSSREF_TEST
#define SB_DATADICTDEFN_TEST
#define SB_DATADICTSCHEMA_TEST
#define SB_SECURITY_TEST
#define SB_REGISTRATION_TEST
#define SB_RASTER_TEST
#define SB_DIMENDEFN_TEST

//#define SB_BLANK_TEST

#include "build_test/t_sb_output.h"

ostream & operator<<( ostream & os, vector<sb_ForeignID> const& aTempVector);
ostream & operator<<( ostream & os, vector<sb_SpatialAddr> const& aTempVector);

void
dumpSubfield( sc_Subfield const & subfield )
{
	cout << "\t" 
		  << subfield.getMnemonic() << " : "; //<< subfield->getName();

	string tmp_string;
	long   tmp_long;
	double tmp_double;

	switch ( subfield.getSubfieldType() )
	{
		case sc_Subfield::is_A : subfield.getA( tmp_string ); cout << tmp_string; break;
      case sc_Subfield::is_I : subfield.getI( tmp_long ); cout << tmp_long; break;
      case sc_Subfield::is_R : subfield.getR( tmp_double ); cout << tmp_double; break;
      case sc_Subfield::is_S : subfield.getS( tmp_double ); cout << tmp_double; break;
      case sc_Subfield::is_C : subfield.getC( tmp_string ); cout << tmp_string; break;
//      case sc_Subfield::is_B : subfield.getI( tmp_long ); cout << tmp_long; break;
      case sc_Subfield::is_BI8 : subfield.getBI8( tmp_long ); cout << tmp_long; break;
      case sc_Subfield::is_BI16 : subfield.getBI16( tmp_long ); cout << tmp_long; break;
      case sc_Subfield::is_BI24 : subfield.getBI24( tmp_long ); cout << tmp_long; break;
      case sc_Subfield::is_BI32 : subfield.getBI32( tmp_long ); cout << /* hex << */ tmp_long /* << dec*/; break;
//      case sc_Subfield::is_BUI : subfield.getBUI( tmp_long ); cout << tmp_long; break;
//      case sc_Subfield::is_BUI8 : subfield.getBUI8( tmp_long ); cout << tmp_long; break;
//      case sc_Subfield::is_BUI16 : subfield.getBUI16( tmp_long ); cout << tmp_long; break;
//      case sc_Subfield::is_BUI24 : subfield.getBUI24( tmp_long ); cout << tmp_long; break;
//      case sc_Subfield::is_BUI32 : subfield.getBUI32( tmp_long ); cout << tmp_long; break;
      //case sc_Subfield::is_BFP32 : subfield.getBFP32( tmp_double ); cout << tmp_long; break;
      //case sc_Subfield::is_BFP64 : subfield.getBFP64( tmp_double ); cout << tmp_long; break;
	}

	cout << endl;

} // dumpSubfield

void
dumpRecord( sc_Record const & record )
{
	for ( sc_FieldCntr::const_iterator field_itr = record.begin();
			field_itr != record.end();
			field_itr++ )
	{
		cout << field_itr->getMnemonic() << " : " << field_itr->getName() << endl;
		for ( sc_SubfieldCntr::const_iterator subfield_itr = field_itr->begin();
				subfield_itr != field_itr->end();
				subfield_itr++ )
		{
			dumpSubfield( (*subfield_itr) );
		}
	}
} // dumpRecord

/*
 * builder object output functions and operator<<
 *
 */

// dump foriegnID
void
dumpFID( sb_ForeignID  const & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_ForeignID const & temp )
{

    os   << "Foreign ID \n"
         << "\t Module name:  " << temp.getModuleName()  << endl
         << "\t Record ID:    " << temp.getRecordID()    << endl
         << "\t Usage Modifer:" << temp.getUsageModifier()  << endl;
    return os;
}

// Vetor<sb_ForeignID> output function
ostream &
operator<<( ostream & os, vector<sb_ForeignID> const& aTempVector)
{
	vector<sb_ForeignID>::const_iterator  fid;
	for( fid = aTempVector.begin();
	     fid != aTempVector.end();
	     fid++ )
	    {
		os << sb_ForeignID(*fid) << endl;
	    }
    return os;
}


// dump Spatial Addr
void
dumpSpatialAddr( sb_SpatialAddr const& temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_SpatialAddr const & temp )
{

    os   << "Spatial Addr \n"
         << "\t X: " << temp.getX() << endl
         << "\t Y: " << temp.getY() << endl
         << "\t Z: " << temp.getZ() << endl;
    return os;
}
// Vetor<sb_SpatialAddr> output function
ostream &
operator<<( ostream & os, vector<sb_SpatialAddr> const& aTempVector)
{
	vector<sb_SpatialAddr>::const_iterator  fid;
	for( fid = aTempVector.begin();
	     fid != aTempVector.end();
	     fid++ )
	    {
		os << sb_SpatialAddr(*fid) << endl;
	    }
	return os;
}

// void dumpSbObject( sb_Ident const& temp )
void dumpSbObject( sb_Ident const& temp ) 
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_Ident const & temp )
{

      os << "Dump Ident object" << endl
	 << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "Stand Idnet: " << temp.getStandardIdent() << endl
	 << "Std Version: " << temp.getStandardVer() << endl
	 << "Std Doc Ref: " << temp.getStandardDocRef() << endl
	 << "Profile idnt " << temp.getProfileIdent()  << endl
	 << "Profile Ver: " << temp.getProfileVer() << endl
	 << "ProfileDocRef" << temp.getProfileDocRef() << endl
	 << "Title:       " << temp.getTitle() << endl
	 << "DataID :     " << temp.getDataID() << endl
	 << "DataStruct:  " << temp.getDataStruct() << endl
	 << "map date :   " << temp.getMapDate() << endl
	 << "datasetcreate" << temp.getDataSetCreationDate() << endl
	 << "scale:       " << temp.getScale() << endl
	 << "Comment      " << temp.getComment() << endl
	 << "Composite:   " << temp.getComposites() << endl
	 << "VectorGeom   " << temp.getVectorGeom() << endl
	 << "VectorTopol  " << temp.getVectorTopol() << endl
	 << "Raster       " << temp.getRaster () << endl
	 << "ExtSpatRef   " << temp.getExternSpatRef() << endl
	 << "Fretures Lvl " << temp.getFeaturesLevel() <<endl
	 << "Coding Level:" << temp.getCodingLevel() << endl
	 << "Nongeo Dem:  " << temp.getNongeoDeminsion() << endl
	 << "Attrib ID for" << endl
	 <<  temp.getAttributeID() 
         << "***\n";
     return os;
}

// sb_TransferStats boejct dump
void dumpSbObject( sb_TransferStats const& temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_TransferStats const & temp )
{
    os  << "Object Dump:   " << "Transfer Stats " << endl
        << "Module Name:   " << temp.getModuleName() << endl
        << "Record ID:     " << temp.getRecordID() << endl
	<< "ModTypeRef:    " << temp.getModuleTypeReferred()
	<< endl
	<< "ModNameRef:    " << temp.getModuleNameReferred()
	<< endl
	<< "ModRecCount:   " << temp.getModuleRecCount()
	<< endl
	<< "SpatialAddrCnt:" << temp.getSpatialAddrCount()
	<< endl

	   << "***\n";
    return os;

}

// sb_Arc object
void dumpSbObject( sb_Arc & temp )
{
    cout << temp;
}
//Stream Function for Arc
ostream &
operator<<( ostream & os, sb_Arc  & temp )
{

    os   << "Object Dump: " << "Arc" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "ObjectRep:   " << temp.getObjectRep() << endl
	 << "Surface:     " << temp.getSurface() << endl
	 << "Order:       " << temp.getOrder() << endl
	 << "CenterAddr:  " << temp.getCenterAddr() << endl
	 << "StartAddr:   " << temp.getStartAddr() << endl
	 << "EndAddr:     " << temp.getEndAddr() << endl
	 << "getMJRA:     " << temp.getMJRA() << endl
	 << "getMNRA:     " << temp.getMNRA() << endl
         << "Vector of Curve Addrs:" << endl
	 << temp.getCurveAddrs()  << endl
	 << "Vector of AttributeIDs" << endl
	 << temp.getAttributeID() << endl
	 << "Vector fo CompositeIDs" << endl
	 << temp.getCompositeID() << endl
	 << "Vector of RepModuleIDs" << endl
	 << temp.getRepModuleID() << endl
	 << "***\n";
    return os;
}

// sb_Line output functions.

void
dumpSbObject( sb_Line & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_Line  & temp )
{
    os   << "Object Dump:   " << "Line" << endl
         << "Module Name:   " << temp.getModuleName() << endl
	 << "Record ID:     " << temp.getRecordID() << endl
	 << "ObjectRep:     " << temp.getObjectRep() << endl
	 << "AttribID:      " << temp.getAttributeID() << endl
	 << "Polygon Left:  " << temp.getPolygonIDLeft() << endl
	 << "Polygon Right: " << temp.getPolygonIDRight() << endl
	 << "StartNodeID:   " << temp.getStartNodeID() << endl
	 << "EndNodeID:     " << temp.getEndNodeID() << endl
	 << "Vector of Spatial Addrs" << endl
	 << temp.getSpatialAddrs() << endl

	 << "***\n";

    return os;
}
// Internal Spatial Reference
void
dumpSbObject( sb_IntSpatialRef & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_IntSpatialRef & temp )
{
    os   << "Object Dump: " << "IntSpatialRef" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "Comment:     " << temp.getComment() << endl
	 << "SA type:     " <<  temp.getSpatialAddType()  << endl
	 << "Xcomplbl:    " << temp.getSpatialAddXCompLbl()  << endl
	 << "Y Comp label:" << temp.getSpatialAddYCompLbl()  << endl
	 << "Horiz. Fmt:  " << temp.getHorizontalCompFmt()  << endl
	 << "Vert. Fmt:   " << temp.getVerticalCompFmt()  << endl
	 << "Scale X:     " << temp.getScaleFactX()  << endl
	 << "Scale Y:     " << temp.getScaleFactY()  << endl
	 << "Scale Z:     " << temp.getScaleFactZ()  << endl
	 << "Origin X:    " << temp.getXOrigin()  << endl
	 << "Origin Y:    " << temp.getYOrigin()  << endl
	 << "Origin Z:    " << temp.getZOrigin()  << endl
	 << "X CompHorRes:" << temp.getXCompHorizRes()  << endl
	 << "Y CompHorRes:" << temp.getYCompHorizRes()  << endl
	 << "Vertical Res:" << temp.getVerticalResComp()  << endl
	 << "DimensionID: " << temp.getDimensionID()  << endl

	 << "***\n";

    return os;
}

// Polygon output functions
void
dumpSbObject( sb_Polygon & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_Polygon & temp )
{
    os   << "Object Dump: " << "Polygon" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "ObjectRep:   " << temp.getObjectRep()  << endl
	 << "AttribID:    " << temp.getAttributeID()  << endl
	 << "RingID:      " << temp.getRingID()  << endl
	 << "ChainID:     " << temp.getChainID()  << endl
	 << "CompoisteID: " << temp.getCompositeID()  << endl
	 << "RepModuleID: " << temp.getRepModuleID()  << endl
	 << "***\n";

    return os;
}

void
dumpSbObject( sb_PointNode & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_PointNode & temp )
{
    os   << "Object Dump: " << "PointNode" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "Object Rep:  " << temp.getObjectRep() << endl
	 << "Spatial Addrs" << endl
	 << temp.getSpatialAddrs() << endl
	 << "AttribID:    " << temp.getAttributeID() << endl
	 << "LineID:      " << temp.getLineID() << endl
	 << "AreaID:      " << temp.getAreaID() << endl
	 << "CompositeID: " << temp.getCompositeID() << endl
	 << "RepModId:    " << temp.getRepModuleID() << endl
	 << "Vector: OrientSA:    " << endl
	 << temp.getOrientSpatialAddrs() << endl
	 << "AttPrimaryID:" << temp.getAttPrimaryID() << endl
	 << "AttLabel:    " << temp.getAttLbl() << endl
	 << "Vector of SymbolOrientSAs:" << endl
	 << temp.getSymbolOrientSpatialAddrs() << endl
	 << "***\n";

    return os;
}
//sb_DataDictDom.cxx
void
dumpSbObject( sb_DataDictDom & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_DataDictDom & temp )
{
    os   << "Object Dump: " << "DataDictDom" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "AttributeLbl:" << temp.getAttLbl() << endl
	 << "Attrib Auth: " << temp.getAttAuth() << endl
	 << "AttDomType:  " << temp.getAttDomType()  << endl
	 << "AttDomValFmt:" << temp.getAttDomValFmt()  << endl
	 << "ADMU:        " << temp.getAttDomValMsmtUnit()  << endl
	 << "Range|value: " << temp.getRangeOrValue()  << endl
	 << "DomainValue: " << endl ;
	 dumpSubfield (temp.getDomainValue() );
	 os << endl
	 << "DomainValDef:" << temp.getDomainValueDefn() << endl
	 << "***\n";

    return os;
}

void
dumpSbObject( sb_CatalogDir & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_CatalogDir & temp )
{
    os   << "Object Dump: " << "CatalogDir" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "Name:        " << temp.getName() << endl
	 << "Type:        " << temp.getType()  << endl
	 << "Volume:      " << temp.getVolume() << endl
	 << "File:        " << temp.getFile()  << endl
	 << " Record:     " << temp.getRecord()  << endl
	 << "External:    " << temp.getExternal() << endl
	 << "Module Ver:  " << temp.getModuleVer() << endl
	 << "Commet:      " << temp.getComment() << endl
	 << "***\n";

    return os;
}

void
dumpSbObject( sb_ExtSpatialRef & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_ExtSpatialRef & temp )
{
    os   << "Object Dump: " << "ExtSpatialRef" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "Comment:     " << temp.getComment()  << endl
	 << "RefDoc:      " << temp.getRefDoc()  << endl
	 << "RefSysname:  " << temp.getRefSysName()  << endl
	 << "VertDatum:   " << temp.getVertDatum()  << endl
	 << "SoundDatum:  " << temp.getSoundDatum()  << endl
	 << "HorizDatum:  " << temp.getHorizDatum() << endl
	 << "ZoneNum:     " << temp.getZoneNum()  << endl
	 << "Projection:  " << temp.getProjection() << endl
	 << "AttribID:    " << endl
	 << temp.getAttID()  << endl
	 << "***\n";

    return os;
}

void
dumpSbObject( sb_CatalogSpatialDom & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_CatalogSpatialDom & temp )
{
    os   << "Object Dump: " << "CatalogSpatialDom" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "Name:        " << temp.getName()  << endl
	 << "Type:        " << temp.getType()  << endl
	 << "Domain:      " << temp.getDomain() << endl
	 << "Map:         " << temp.getMap()  << endl
	 << "Theme:       " << temp.getTheme()  << endl
	 << "AggregateObj:" << temp.getAggregateObj() << endl
	 << "AggobjType:  " << temp.getAggregateObjType()  << endl
	 << "Comment:     " << temp.getComment()  << endl
	 << "***\n";

    return os;
}

void
dumpSbObject( sb_CatalogCrossRef & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_CatalogCrossRef & temp )
{
    os   << "Object Dump: " << "CatalogCrossRef" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "Name1:       " << temp.getName1() << endl
	 << "Type1:       " << temp.getType1()  << endl
	 << "Name2:       " << temp.getName2() << endl
	 << "Type2:       " << temp.getType2() << endl
	 << "Comment:     " << temp.getComment() << endl
	 << "***\n";

    return os;
}

void
dumpSbObject( sb_DataDictDefn & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_DataDictDefn & temp )
{
    os   << "Object Dump: " << "DataDictDefn" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "EntityOrAtt: " << temp.getEntOrAtt() << endl
	 << "EntityAttlbl:" << temp.getEntAttLbl() << endl
	 << "Source:      " << temp.getSource() << endl
	 << "Definition:  " << temp.getDefinition() << endl
	 << "AttAuthority:" << temp.getAttAuthority()  << endl
	 << "AttAuthDesc: " << temp.getAttAuthorityDes() << endl

	 << "***\n";

    return os;
}

void
dumpSbObject( sb_DataDictSchema & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_DataDictSchema & temp )
{
    os   << "Object Dump: " << "DataDictSchema" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "Name:        " << temp.getName()  << endl
	 << "Type:        " << temp.getType()  << endl
	 << "Entitylbl:   " << temp.getEntityLbl()  << endl
	 << "EntityAuth:  " << temp.getEntityAuth()  << endl
	 << "AttribLbl:   " << temp.getAttributeLbl()  << endl
	 << "AttribAuth:  " << temp.getAttributeAuth()  << endl
	 << "Format:      " << temp.getFormat()  << endl
	 << "Unit:        " << temp.getUnit() << endl
	 << "Precision:   " << temp.getPrecision() << endl
	 << "MaxSubfldLen:" << temp.getMaxSubfieldLen() << endl
	 << "Key:         " << temp.getKey() << endl
	 << "***\n";

    return os;
}

void
dumpSbObject( sb_Security & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_Security & temp )
{
    os   << "Object Dump: " << "Security" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "Sec Class:   " << temp.getSecurityClass()  << endl
	 << "Control:     " << temp.getControl()  << endl
	 << "RelInstruct: " << temp.getReleaseInstruct()  << endl
	 << "ReviewDate:  " << temp.getReviewDate()  << endl
	 << "ReviewInst:  " << temp.getReviewInstruct() << endl
	 << "Comment:     " << temp.getComment()  << endl
	 << "Foreign ID:  " << endl
	 << temp.getForeignID()  << endl
	 << "***\n";

    return os;
}

void
dumpSbObject( sb_Registration & temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_Registration & temp )
{
    os   << "Object Dump: " << "Registration" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
	 << "Comment:     " << temp.getComment()  << endl
	 << "ExtRefSpatialAddrs:" << endl
	 << temp.getExtRefSpatialAddrs() << endl
	 << "IntRefSpatialAddrs:" << endl
	 << temp.getIntRefSpatialAddrs() << endl
	 << "***\n";

    return os;
}

void
dumpSbObject( sb_RasterDefn const& temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_RasterDefn const& temp )
{
    os   << "Object Dump:      " << "RasterDefn" << endl
         << "Module Name:      " << temp.getModuleName() << endl
	 << "Record ID:        " << temp.getRecordID() << endl
	 << "Object Rep:       " << temp.getObjectRep() << endl
	 << "Acquisition Dev:  " << temp.getAcqDev()  << endl
	 << "Acquisition Date: " << temp.getAcqDate() << endl
	 << "Comments:         " << temp.getComments ()  << endl
	 << "Cell Module Name: " << temp.getCellModName() << endl
	 << "Default Impl:     " << temp.getDefImpl()  << endl
	 << "DataCompression:  " << temp.getDataCompress() << endl
	 << "Data Comp Method: " << temp.getDataCompressMethod() << endl
	 << "Row Extent:       " << temp.getRowExtent() << endl
	 << "Column Extent:    " << temp.getColExtent() << endl
	 << "Scan Origin:      " << temp.getScanOrigin() << endl
	 << "Scan Pattern:     " << temp.getScanPattern() << endl
	 << "Tesseral Indexing:" << temp.getTessIndex() << endl
	 << "Tess Index Format:" << temp.getTessIndexFormat() << endl
	 << "Tess Index Desc:  " << temp.getTessIndexDesc() << endl
	 << "Lines/Alternation:" << temp.getLinesPerAlternation() << endl
	 << "First Scan Direct:" << temp.getFirstScanDirection() << endl
	 << "Aspect Ratio:     " << temp.getAspectRatio()  << endl
	 << "Number of Layers: " << temp.getNumberLayers() << endl
	 << "Raster Attrib ID: " << endl
	 << temp.getRasterAttributeID() << endl
	 << "Layer Definitions:" << endl
	 << temp.getLayerDefinition() << endl
	 << "Layer Attrib IDs:" << endl
	 << temp.getLayerAttributeID() << endl
	 << "Composite ID:" << endl
	 << temp.getCompositeID() << endl
	 << "***\n";

    return os;
}

void
dumpSbObject( sb_RasterLayer const& temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_RasterLayer const& temp )
{
    os   << "Object Dump:    " << "RasterLayer" << endl
         << "Layer Name:     " << temp.getLayerName() << endl
	 << "Cell Code:      " << temp.getCellCode() << endl
	 << "Format:         " << temp.getFormat() << endl
	 << "Bitmask:        " << temp.getBitmask() << endl
	 << "Number of Rows: " << temp.getNumberRows()  << endl
	 << "Number of Cols: " << temp.getNumberColumns() << endl
	 << "Min Value:      " << temp.getMinValue() << endl
	 << "Max Value:      " << temp.getMaxValue() << endl
	 << "Scan Origin Row:" << temp.getScanOriginRow() << endl
	 << "Scan Origin Col:" << temp.getScanOriginColumn()  << endl
	 << "RowOffsetOrigin:" << temp.getRowOffsetOrigin() << endl
	 << "ColOffsetOrigin:" << temp.getColumnOffsetOrigin() << endl
	 << "IntraCellRefLoc:" << temp.getIntracellReferenceLocation()
	 << endl
	 << "Comment:        " << temp.getComment() << endl
	 << "***\n";

    return os;
}

ostream &
operator<<( ostream & os, vector<sb_RasterLayer> const& aTempVector)
{
	vector<sb_RasterLayer>::const_iterator  fid;
	for( fid = aTempVector.begin();
	     fid != aTempVector.end();
	     fid++ )
	    {
		os << sb_RasterLayer(*fid) << endl;
	    }
	return os;
}

void
dumpSbObject( sb_DimenDefn const& temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_DimenDefn const& temp )
{
    os   << "Object Dump:      " << "DimenDefn" << endl
         << "Module Name:      " << temp.getModuleName() << endl
	 << "Record ID:        " << temp.getRecordID() << endl
	 << "Dimen Label:      " << temp.getDimenLbl()  << endl
	 << "Cmpnt Format:     " << temp.getDimenCompFormat() << endl
	 << "Cmpnt Resolution: " << temp.getDimenCompRes() << endl
	 << "Value Measure Unt:" << temp.getDimenValMeasureUnit() << endl
	 << "Descriptoin:      " << temp.getDimenDescription() << endl
	 << "Attribute ID:     " << endl
	 << temp.getDimenAttributeID() << endl
	 << "***\n";

    return os;
}

/*
 *  Place class name after the sb_ in the two functions
 *  Fill in the Name in the empty string on the first line of output.
 *  Add the list of get function whose output goes to the stream.
 *
 */

/*(yank 16 lines) Blank function template
void
dumpSbObject( sb_XXX const& temp )
{
    cout << temp;
}
ostream &
operator<<( ostream & os, sb_XXX const& temp )
{
    os   << "Object Dump: " << "XXX" << endl
         << "Module Name: " << temp.getModuleName() << endl
	 << "Record ID:   " << temp.getRecordID() << endl
//	 << " " << temp. () << endl
	 << "***\n";

    return os;
}

*/
