#ifndef INCLUDED_SIO_8211DRLEADER_H
#include "io/sio_8211DRLeader.h"
#endif

#ifndef INCLUDED_SIO_8211DIRENTRY_H
#include "io/sio_8211DirEntry.h"
#endif

#ifndef INCLUDED_SIO_8211DIRECTORY_H
#include "io/sio_8211Directory.h"
#endif

#ifndef INCLUDED_SIO_8211DR_H
#include "io/sio_8211DR.h"
#endif

sio_8211DR::sio_8211DR()
          : _reuseLeaderAndDir(false)
{
}

sio_8211DR::sio_8211DR(sio_8211DRLeader const& leader,
                       sio_8211Directory const& dir)
          : _reuseLeaderAndDir(true)
{
   _leader = new sio_8211DRLeader(leader);
   _dir = new sio_8211Directory(dir);
}

void
sio_8211DR::reuseLeaderAndDirectory(bool flag)
{
   _reuseLeaderAndDir = flag;
}

bool
sio_8211DR::isReusingLeaderAndDirectory() const
{
   return _reuseLeaderAndDir;
}          

istream&
sio_8211DR::streamExtract(istream& istr)
{
   // Assume istr is positioned on byte zero of an ISO8211 DR.

   sio_8211DRLeader* drLeader;
   if (!_reuseLeaderAndDir)
      {
         // Get the leader.
         drLeader = new sio_8211DRLeader;
         istr >> *drLeader;

         // Get the directory.
         delete _dir;
         _dir = new sio_8211Directory(*drLeader);
         istr >> *_dir;
      }

   // Erase any previous fields.
   _fieldArea.erase(_fieldArea.begin(), _fieldArea.end());

   // Get the new fields.
   long fieldAreaStart = istr.tellg();
   sio_8211Directory::iterator i;
   for (i = _dir->begin(); i != _dir->end(); i++)
      { 
         istr.seekg(fieldAreaStart + (*i)->getPosition());
         _fieldArea.push_back(sio_8211Field((*i)->getFieldLength() - 1));
         istr >> _fieldArea.back();
         (*i)->setField(&_fieldArea.back());
      }

   if (!_reuseLeaderAndDir)
      {
         delete _leader;
         _leader = drLeader;
      }

   return istr;
}
