
#ifndef INCLUDED_SIO_8211DIRENTRY_H
#include "io/sio_8211DirEntry.h"
#endif

#ifndef INCLUDED_SIO_8211LEADER_H
#include "io/sio_8211Leader.h"
#endif

#ifndef INCLUDED_SIO_UTILS_H
#include "io/sio_Utils.h"
#endif

sio_8211DirEntry::sio_8211DirEntry(sio_8211Leader const& leader)
                : _fieldLength(0),
                  _fieldPos(0),
                  _field(0)
{
   _lengthSize = leader.getSizeOfFieldLengthField();
   _posSize = leader.getSizeOfFieldPosField();
   _tagSize = leader.getSizeOfFieldTagField();
}

sio_8211DirEntry::sio_8211DirEntry(sio_8211DirEntry const& dirEntry)
{
   operator=(dirEntry);
}

sio_8211DirEntry const&
sio_8211DirEntry::operator=(sio_8211DirEntry const& rhs)
{
   if (&rhs == this)
      return *this;

   _lengthSize = rhs._lengthSize;
   _posSize = rhs._posSize;
   _tagSize = rhs._tagSize;
   _fieldTag = rhs._fieldTag;
   _fieldPos = rhs._fieldPos;
   _fieldLength = rhs._fieldLength;

   return *this;
}

sio_8211DirEntry::~sio_8211DirEntry()
{
}

long
sio_8211DirEntry::getFieldLength() const
{
   return _fieldLength;
}

long
sio_8211DirEntry::getPosition() const
{
   return _fieldPos;
}


string const&
sio_8211DirEntry::getTag() const
{
   return _fieldTag;
}

sio_8211Field const*
sio_8211DirEntry::getField() const
{
   return _field;
}

void
sio_8211DirEntry::setField(sio_8211Field const* field)
{
   _field = field;
}

istream& operator>>(istream& istr, sio_8211DirEntry& dirEntry)
{
   // Assumes that istr is positioned on byte 0 of an ISO8211 record directory entry.
   // Reads the directory entry, leaving the stream pointer positioned on the byte
   // after the last byte in the entry.

   long totalLength = dirEntry._lengthSize + dirEntry._posSize + dirEntry._tagSize;
   char* entryBuffer = new char[totalLength];

   istr.read(entryBuffer, totalLength);
   if ((istr.gcount() < totalLength) || (!istr))
      {
         istr.clear(ios::failbit);
         return istr;
      }

   long rp = 0;  // relative position
   dirEntry._fieldTag = sio_Utils::getString(entryBuffer,rp,dirEntry._tagSize);
   rp = rp + dirEntry._tagSize;
   dirEntry._fieldLength = sio_Utils::getLong(entryBuffer,rp,dirEntry._lengthSize);
   rp = rp + dirEntry._lengthSize;
   dirEntry._fieldPos = sio_Utils::getLong(entryBuffer,rp,dirEntry._posSize);

   return istr;
}
