
#ifndef INCLUDED_SIO_8211FIELD_H
#include "io/sio_8211Field.h"
#endif

#ifndef INCLUDED_SIO_8211DIRENTRY_H
#include "io/sio_8211DirEntry.h"
#endif

#ifndef INCLUDED_SIO_8211DDRLEADER_H
#include "io/sio_8211DDRLeader.h"
#endif

sio_8211Field::sio_8211Field()
             : _data(0),
               _dataLength(0)
{
   // XXX Supplied just because the #$%#$% STL stuff needs one. -- jamie
}               

sio_8211Field::sio_8211Field(long fieldSize)
             : _data(0),
               _dataLength(fieldSize)
{

}

sio_8211Field::sio_8211Field(sio_8211Field const& rhs)
             : _data(0),
               _dataLength(0)
{
   operator=(rhs);
}

sio_8211Field const&
sio_8211Field::operator=(sio_8211Field const& rhs)
{
   if (this == &rhs)
      return *this;

   _dataLength = rhs._dataLength;
   
   if ((_dataLength > 0) && (rhs._data != 0))
      {
         _data = new char[_dataLength];
         memcpy(_data,rhs._data,_dataLength);
      }

   return *this;
}

sio_8211Field::~sio_8211Field()
{
   delete _data;
}

long
sio_8211Field::getDataLength() const
{
   return _dataLength;
}

char const*
sio_8211Field::getData() const
{
   return _data;
}


bool
sio_8211Field::getVariableSubfield(char* value,
                                   long& startPos,
                                   char delim) const
{
   // Attempt to extract a character subfield, terminated by delim.
   // XXX This should probably accept a length argument for 'value' so 
   // XXX we don't overfill it.

   char* tmpbuf = new char[_dataLength];  // We know it won't be longer than this.
   long numchars = 0;
   long pos;
   if ((startPos >= 0) && (startPos < _dataLength))
      pos = startPos;
   else
      return false;

   while ((pos < _dataLength) && (_data[pos] != delim))
      {
         tmpbuf[numchars] = _data[pos];
         numchars++;
         pos++;
      }

   tmpbuf[numchars] = '\0';
   strcpy(value,tmpbuf);
   startPos = startPos + numchars + 1;

   delete tmpbuf;
   return true;
}


istream&
sio_8211Field::streamExtract(istream& istr)
{
   // Assume istr is positioned on byte zero of a field.

   // XXX We need some error checking here.
   delete _data;
   _data = new char[_dataLength];
   istr.read(_data,_dataLength);

   if (istr)
      {
         // Gobble up field terminator.
         char tempChar;
         istr.get(tempChar);  // XXX Should check to see if this really _is_ 
                              // XXX a field terminator.
      }

   return istr;  
}

ostream&
sio_8211Field::streamInsert(ostream& ostr) const
{
   return ostr;
}

istream&
operator>>(istream& istr, sio_8211Field& field)
{
   return field.streamExtract(istr);
}


ostream&
operator<<(ostream& ostr, sio_8211Field const& field)
{
   return field.streamInsert(ostr);
}

