
#ifndef INCLUDED_SIO_8211FIELD_H
#define INCLUDED_SIO_8211FIELD_H

#include <iostream>
#include <string>

#ifdef WIN32
   using namespace std;
#endif

#ifndef INCLUDED_SIO_8211UTILS_H
#include "io/sio_8211Utils.h"
#endif

class sio_8211DirEntry;
class sio_8211DDRLeader;

class sio_8211Field;
istream& operator>>(istream& istr, sio_8211Field& field);
ostream& operator<<(ostream& ostr, sio_8211Field const& field);


class sio_8211Field
{
   // This represents a generic ISO 8211 field.

   public:

      sio_8211Field();

      explicit sio_8211Field(long fieldSize);
         // fieldSize is the size of this field, in bytes.

      sio_8211Field(sio_8211Field const& rhs);

      sio_8211Field const& operator=(sio_8211Field const& rhs);

      // XXX *STUPID* VC++ 5.0 templates want these, even though nothing
      // XXX we're doing should require them. -- jamie
      bool operator<(sio_8211Field const& rhs) const { return false; }
      bool operator>(sio_8211Field const& rhs) const { return false; }
      bool operator==(sio_8211Field const& rhs) const { return false; }
      bool operator!=(sio_8211Field const& rhs) const { return false; }


      virtual ~sio_8211Field();

      // All fields have two things we're interested in... the raw data it contains,
      // and how much of that data there is.

      long getDataLength() const;

      char const* getData() const;


      // Subfield operations

      // All getFixedSubfield() operations attempt to extract a fixed width value.
      // They return true if successful and false if an error occurred.

      bool getFixedSubfield(char* value,
                            long length,
                            long startPos) const;

      bool getFixedSubfield(long& value,
                            long length,
                            long startPos) const;

      bool getFixedSubfield(unsigned long& value,
                            long length,
                            long startPos) const;

      bool getFixedSubfield(double& value,
                            long length,
                            long startPos) const;


      // All getVariableSubfield() operations attempt to extract a variable width
      // value. By default, the variable width subfield delimeter is the ISO8211 
      // Unit Terminator. This can be overridden.

      bool getVariableSubfield(char* value,
                               long& startPos,
                               char delim = sio_8211UnitTerminator) const;

      bool getVariableSubfield(long& value,
                               long& startPos,
                               char delim = sio_8211UnitTerminator) const;

      bool getVariableSubfield(unsigned long& value,
                               long& startPos,
                               char delim = sio_8211UnitTerminator) const;

      bool getVariableSubfield(double& value,
                               long& startPos,
                               char delim = sio_8211UnitTerminator) const;


      friend istream& operator>>(istream& istr, sio_8211Field& field);
      friend ostream& operator<<(ostream& ostr, sio_8211Field const& field);

   protected:

      long _dataLength;
      char* _data;

      virtual istream& streamExtract(istream& istr);
      virtual ostream& streamInsert(ostream& ostr) const;
};

//istream&
//operator>>(istream& istr, sio_8211Field& field);


#endif  // INCLUDED_SIO_8211FIELD_H
