
#ifndef INCLUDED_SIO_CONVERTER_H
#define INCLUDED_SIO_CONVERTER_H

#include <string>

#ifdef WIN32
   using namespace std;
#endif

class sc_Subfield;

class sio_Converter
{

   // A Converter converts between an SDTS subfield and 'raw' data.

   public:

		sio_Converter() {}

      virtual ~sio_Converter();


      // NOTE: Niether the fixed or variable make() functions set the subfield
      //       mnemonic or the subfield name.

      // Fixed width subfields
      virtual long makeFixedSubfield(sc_Subfield* subfield,
                                     char const* data,
                                     long length) const = 0;
         // Returns the length of the data actually converted. This may be
         // different than 'length' if an error occured.

      // Variable width subfields
      virtual long makeVarSubfield(sc_Subfield* subfield,
                                char const* data,
                                long maxLength,
                                char delimiter) const;
         // Returns the length of the data converted. This does _not_ included
         // the delimiter (if any).

   protected:

      long findVariableSubfieldLength(char const* data,
                                   long maxLength,
                                   char delimiter) const;
         // Determines the length of a variable length subfield in 'data'
         // delimited by 'delimiter'.

};

class sio_Converter_A : public sio_Converter
{
   public:
 
		sio_Converter_A() {}

      long makeFixedSubfield(sc_Subfield* subfield,
                             char const* data,
                             long length) const;
};

class sio_Converter_I : public sio_Converter
{
   public:
 
      sio_Converter_I() {}

      long makeFixedSubfield(sc_Subfield* subfield,
                             char const* data,
                             long length) const;
};

class sio_Converter_R : public sio_Converter
{
   public:

      sio_Converter_R() {}

      long makeFixedSubfield(sc_Subfield* subfield,
                             char const* data,
                             long length) const;
};

class sio_Converter_S : public sio_Converter
{
   public:

      sio_Converter_S() {}

      long makeFixedSubfield(sc_Subfield* subfield,
                             char const* data,
                             long length) const;
};

class sio_Converter_BI8 : public sio_Converter
{
   public:
      sio_Converter_BI8() {}

      long makeFixedSubfield(sc_Subfield* subfield,
                             char const* data,
                             long length) const;
};

class sio_Converter_BI16 : public sio_Converter
{
   public:
      sio_Converter_BI16() {}

      long makeFixedSubfield(sc_Subfield* subfield,
                             char const* data,
                             long length) const;
};

class sio_Converter_BI24 : public sio_Converter
{
   public:
      sio_Converter_BI24() {}

      long makeFixedSubfield(sc_Subfield* subfield,
                             char const* data,
                             long length) const;
};
class sio_Converter_BI32 : public sio_Converter
{
   public:
      sio_Converter_BI32() {}

      long makeFixedSubfield(sc_Subfield* subfield,
                             char const* data,
                             long length) const;
};

class sio_Converter_BUI8 : public sio_Converter
{
   public:
      sio_Converter_BUI8() {}

      long makeFixedSubfield(sc_Subfield* subfield,
                             char const* data,
                             long length) const;
};

class sio_Converter_BUI16 : public sio_Converter
{
   public:
      sio_Converter_BUI16() {}

      long makeFixedSubfield(sc_Subfield* subfield,
                             char const* data,
                             long length) const;
};

class sio_Converter_BUI24 : public sio_Converter
{
   public:
      sio_Converter_BUI24() {}

      long makeFixedSubfield(sc_Subfield* subfield,
                             char const* data,
                             long length) const;
};

class sio_Converter_BUI32 : public sio_Converter
{
   public:
      sio_Converter_BUI32() {}

      long makeFixedSubfield(sc_Subfield* subfield,
                             char const* data,
                             long length) const;
};

#endif  // INCLUDED_SIO_CONVERTER_H
