
#include <iostream>
#include <fstream>

#ifdef WIN32
   using namespace std;
#endif

#include "io/sio_8211FieldFormat.h"
#include "io/sio_8211DDR.h"
#include "io/sio_8211DirEntry.h"
#include "io/sio_8211Field.h"
#include "io/sio_8211DDRField.h"
#include "io/sio_8211DDRLeader.h"
#include "io/sio_Converter.h"


// binary 32 bit integer converter function
sio_Converter_BI32  bui32_converter;

int
main(int argc, char** argv)
{
	
	if (argc != 2)
	{
		cerr << "Usage: " << argv[0] << " <8211 filename>\n";
      return 1;
   }

   ifstream istr(argv[1],ios::binary);
   if (!istr)
   {
      cerr << "Error opening file '" << argv[1] << "'. Aborting.\n";
      return 2;
   }


   cout << "\n\nFilename: '" << argv[1] << "'\n\n";



   sio_8211DDR ddr;


	istr >> ddr;

   if ( ! istr )
	{
		return 3;
	}


	map<string, sio_Converter*> converters;
	converters["Y"] = &bui32_converter;
	converters["y"] = &bui32_converter;

	sio_8211Directory const* ddr_directory = ddr.getDirectory();

	list<sio_8211FieldFormat> field_formats;

	for ( vector<sio_8211DirEntry*>::const_iterator dir_entry =
				ddr_directory->begin();
		   dir_entry != ddr_directory->end();
			dir_entry++ )
	{

		if ( (*dir_entry)->getTag().substr(0,3) == "000" )
		{
			cout << (*dir_entry)->getTag() << " is reserved ... skipping" << endl;
			continue;				// this must be reserved field, skip it
		}

										// build a ddr field given the DDR's leader
										// and the current field data

		sio_8211DDRLeader const * ddr_leader = 
			dynamic_cast<sio_8211DDRLeader const *>(ddr.getLeader());

		if ( ! ddr_leader ) return 4;


										// XXX for some reason, I can't pass in
										// XXX the results of getField() directory to
										// XXX sio_8211DDRField's ctor.

		sio_8211Field const * field = (*dir_entry)->getField();
		sio_8211DDRField  ddr_field( *ddr_leader, *field );


		field_formats.push_back( sio_8211FieldFormat() );


		sio_8211MakeFieldFormat( field_formats.back(),
										 ddr_field,
										 (*dir_entry)->getTag(), 
										 & converters );

		cout << field_formats.back().size() << " subfields added" << endl;
	}


	// now dump out the field formats and their respective subfield formats

	for ( list<sio_8211FieldFormat>::const_iterator ff_itr = 
					field_formats.begin();
		   ff_itr != field_formats.end();
			ff_itr++ )
	{
		cout << ff_itr->getTag() << " has " << ff_itr->size()
			  << " subfields" << endl;

		for ( list<sio_8211SubfieldFormat>::const_iterator sf_itr = 
					ff_itr->begin();
		      sf_itr != ff_itr->end();
			   sf_itr++ )
		{
			cout << sf_itr->getLabel() << " : " << sf_itr->getType() 
				  << "\tconverter address: " << sf_itr->getConverter() << endl;
		}

	}

	return 0;

}