#include <iostream>
#include <fstream>


#ifdef WIN32
   using namespace std;
#endif

#include "io/sio_Reader.h"
#include "io/sio_8211DirEntry.h"
#include "io/sio_Converter.h"

#include "container/sc_Subfield.h"
#include "container/sc_Field.h"
#include "container/sc_Record.h"


void
dumpSubfield( sc_Subfield const & subfield )
{
	cout << "\t" 
		  << subfield.getMnemonic() << subfield.getName() << " : ";

	string tmp_string;
	long   tmp_long;
	double tmp_double;

	switch ( subfield.getSubfieldType() )
	{
		case sc_Subfield::is_A : subfield.getA( tmp_string ); cout << tmp_string; break;
      case sc_Subfield::is_I : subfield.getI( tmp_long ); cout << tmp_long; break;
      case sc_Subfield::is_R : subfield.getR( tmp_double ); cout << tmp_double; break;
      case sc_Subfield::is_S : subfield.getS( tmp_double ); cout << tmp_double; break;
      case sc_Subfield::is_C : subfield.getC( tmp_string ); cout << tmp_string; break;
//      case sc_Subfield::is_B : subfield.getI( tmp_long ); cout << tmp_long; break;
      case sc_Subfield::is_BI8 : subfield.getBI8( tmp_long ); cout << tmp_long; break;
      case sc_Subfield::is_BI16 : subfield.getBI16( tmp_long ); cout << tmp_long; break;
      case sc_Subfield::is_BI24 : subfield.getBI24( tmp_long ); cout << tmp_long; break;
      case sc_Subfield::is_BI32 : subfield.getBI32( tmp_long ); cout << /* hex << */ tmp_long /* << dec*/; break;
//      case sc_Subfield::is_BUI : subfield.getBUI( tmp_long ); cout << tmp_long; break;
//      case sc_Subfield::is_BUI8 : subfield.getBUI8( tmp_long ); cout << tmp_long; break;
//      case sc_Subfield::is_BUI16 : subfield.getBUI16( tmp_long ); cout << tmp_long; break;
//      case sc_Subfield::is_BUI24 : subfield.getBUI24( tmp_long ); cout << tmp_long; break;
//      case sc_Subfield::is_BUI32 : subfield.getBUI32( tmp_long ); cout << tmp_long; break;
      //case sc_Subfield::is_BFP32 : subfield.getBFP32( tmp_double ); cout << tmp_long; break;
      //case sc_Subfield::is_BFP64 : subfield.getBFP64( tmp_double ); cout << tmp_long; break;
	}

	cout << endl;

} // dumpSubfield


void
dumpRecord( sc_Record const & record )
{

#ifdef VERBOSE

	for ( sc_FieldCntr::const_iterator field_itr = record.begin();
			field_itr != record.end();
			field_itr++ )
	{
		cout << field_itr->getMnemonic() << " : " << field_itr->getName() << endl;
		for ( sc_SubfieldCntr::const_iterator subfield_itr = field_itr->begin();
				subfield_itr != field_itr->end();
				subfield_itr++ )
		{
			dumpSubfield( (*subfield_itr) );
		}
	}
#else
   cout << " has " << record.size() << " fields" << endl;
#endif

} // dumpRecord



int
main( int argc, char** argv )
{
	if ( ! argv[1] ) 
	{
		cerr << "usage: "
			<< argv[0] << " 8211file " << endl;
		return 1;
	}

   ifstream ddf( argv[1], ios::binary );

	if ( ! ddf ) 
	{
		cerr << "couldn't open "
			<< argv[1] << endl;
		return 2;
	}

	map<string, sio_Converter*> converters;

   // binary 32 bit integer converter function
   sio_Converter_BI32  bi32_converter;
   sio_Converter_BI16  bi16_converter;


	converters["X"] = &bi32_converter;
	converters["Y"] = &bi32_converter;
   converters["ELEVATION"] = &bi16_converter;
	
	sio_8211Reader  reader( ddf, &converters );
	

	sc_Record record;

   for( sio_8211ForwardIterator i( reader ); ! i.done(); ++i )
	{
		i.get( record );
		dumpRecord( record );
	}

	return 0;
}